@@ -23,6 +23,10 @@
 #ifndef __AUTH_AUTH_SAM_REPLY_H__
 #define __AUTH_AUTH_SAM_REPLY_H__
 
+#include "libcli/util/ntstatus.h"
+#include "libcli/util/werror.h"
+#include "librpc/gen_ndr/auth.h"
+
 #undef _PRINTF_ATTRIBUTE
 #define _PRINTF_ATTRIBUTE(a1, a2) PRINTF_ATTRIBUTE(a1, a2)
 /* this file contains prototypes for functions that are private
@@ -43,13 +47,18 @@ struct auth_user_info *auth_user_info_copy(TALLOC_CTX *mem_ctx,
 
 NTSTATUS auth_convert_user_info_dc_saminfo6(TALLOC_CTX *mem_ctx,
 					   const struct auth_user_info_dc *user_info_dc,
-					   struct netr_SamInfo6 **_sam6);
+					   enum auth_group_inclusion group_inclusion,
+					   struct netr_SamInfo6 **_sam6,
+					   struct PAC_DOMAIN_GROUP_MEMBERSHIP **_resource_groups);
 NTSTATUS auth_convert_user_info_dc_saminfo2(TALLOC_CTX *mem_ctx,
 					   const struct auth_user_info_dc *user_info_dc,
+					   enum auth_group_inclusion group_inclusion,
 					   struct netr_SamInfo2 **_sam2);
 NTSTATUS auth_convert_user_info_dc_saminfo3(TALLOC_CTX *mem_ctx,
 					   const struct auth_user_info_dc *user_info_dc,
-					   struct netr_SamInfo3 **_sam3);
+					   enum auth_group_inclusion group_inclusion,
+					   struct netr_SamInfo3 **_sam3,
+					   struct PAC_DOMAIN_GROUP_MEMBERSHIP **_resource_groups);
 
 /**
  * Make a user_info_dc struct from the info3 returned by a domain logon
@@ -67,6 +76,7 @@ NTSTATUS make_user_info_dc_netlogon_validation(TALLOC_CTX *mem_ctx,
 NTSTATUS make_user_info_dc_pac(TALLOC_CTX *mem_ctx,
 			      const struct PAC_LOGON_INFO *pac_logon_info,
 			      const struct PAC_UPN_DNS_INFO *pac_upn_dns_info,
+			      enum auth_group_inclusion group_inclusion,
 			      struct auth_user_info_dc **_user_info_dc);
 
 /* The following definitions come from auth/wbc_auth_util.c  */
