@@ -40,6 +40,9 @@
 #include "ctdb_private.h"
 #include "ctdb_client.h"
 
+#include "protocol/protocol.h"
+#include "protocol/protocol_api.h"
+
 #include "common/rb_tree.h"
 #include "common/reqid.h"
 #include "common/system.h"
@@ -48,6 +51,8 @@
 #include "common/pidfile.h"
 #include "common/sock_io.h"
 
+#include "conf/node.h"
+
 struct ctdb_client_pid_list {
 	struct ctdb_client_pid_list *next, *prev;
 	struct ctdb_context *ctdb;
@@ -745,7 +750,7 @@ static void daemon_request_call_from_client(struct ctdb_client *client,
 
 	ctdb_db = find_ctdb_db(client->ctdb, c->db_id);
 	if (!ctdb_db) {
-		DEBUG(DEBUG_ERR, (__location__ " Unknown database in request. db_id==0x%08x",
+		DEBUG(DEBUG_ERR, (__location__ " Unknown database in request. db_id==0x%08x\n",
 			  c->db_id));
 		CTDB_DECREMENT_STAT(ctdb, pending_calls);
 		return;
@@ -1331,7 +1336,7 @@ static void ctdb_tevent_trace(enum tevent_trace_point tp,
 
 	switch (tp) {
 	case TEVENT_TRACE_BEFORE_WAIT:
-		diff = timeval_until(&tevent_after_wait_ts, &now);
+		diff = tevent_timeval_until(&tevent_after_wait_ts, &now);
 		if (diff.tv_sec > 3) {
 			DEBUG(DEBUG_ERR,
 			      ("Handling event took %ld seconds!\n",
@@ -1341,7 +1346,7 @@ static void ctdb_tevent_trace(enum tevent_trace_point tp,
 		break;
 
 	case TEVENT_TRACE_AFTER_WAIT:
-		diff = timeval_until(&tevent_before_wait_ts, &now);
+		diff = tevent_timeval_until(&tevent_before_wait_ts, &now);
 		if (diff.tv_sec > 3) {
 			DEBUG(DEBUG_ERR,
 			      ("No event for %ld seconds!\n",
@@ -2016,7 +2021,10 @@ static int ctdb_client_notify_destructor(struct ctdb_client_notify_list *nl)
 
 	DEBUG(DEBUG_ERR,("Sending client notify message for srvid:%llu\n", (unsigned long long)nl->srvid));
 
-	ret = ctdb_daemon_send_message(nl->ctdb, CTDB_BROADCAST_CONNECTED, (unsigned long long)nl->srvid, nl->data);
+	ret = ctdb_daemon_send_message(nl->ctdb,
+				       CTDB_BROADCAST_CONNECTED,
+				       nl->srvid,
+				       nl->data);
 	if (ret != 0) {
 		DEBUG(DEBUG_ERR,("Failed to send client notify message\n"));
 	}
@@ -2172,22 +2180,43 @@ int32_t ctdb_control_check_pid_srvid(struct ctdb_context *ctdb,
 	return -1;
 }
 
-int ctdb_control_getnodesfile(struct ctdb_context *ctdb, uint32_t opcode, TDB_DATA indata, TDB_DATA *outdata)
+int ctdb_control_getnodesfile(struct ctdb_context *ctdb,
+			      uint32_t opcode,
+			      TDB_DATA indata,
+			      TDB_DATA *outdata)
 {
-	struct ctdb_node_map_old *node_map = NULL;
+	struct ctdb_node_map *node_map = NULL;
+	size_t len;
+	uint8_t *buf = NULL;
+	size_t npush = 0;
+	int ret = -1;
 
 	CHECK_CONTROL_DATA_SIZE(0);
 
-	node_map = ctdb_read_nodes_file(ctdb, ctdb->nodes_file);
+	node_map = ctdb_read_nodes(ctdb, ctdb->nodes_file);
 	if (node_map == NULL) {
-		DEBUG(DEBUG_ERR, ("Failed to read nodes file\n"));
+		D_ERR("Failed to read nodes file\n");
 		return -1;
 	}
 
-	outdata->dptr  = (unsigned char *)node_map;
-	outdata->dsize = talloc_get_size(outdata->dptr);
+	len = ctdb_node_map_len(node_map);
+	buf = talloc_size(ctdb, len);
+	if (buf == NULL) {
+		goto done;
+	}
 
-	return 0;
+	ctdb_node_map_push(node_map, buf, &npush);
+	if (len != npush) {
+		talloc_free(buf);
+		goto done;
+	}
+
+	outdata->dptr  = buf;
+	outdata->dsize = len;
+	ret = 0;
+done:
+	talloc_free(node_map);
+	return ret;
 }
 
 void ctdb_shutdown_sequence(struct ctdb_context *ctdb, int exit_code)
