@@ -41,7 +41,7 @@
  * As it's also needed when Python.h is the first header we
  * require a global -D__STDC_WANT_LIB_EXT1__=1
  */
-#ifndef __STDC_WANT_LIB_EXT1__
+#if __STDC_WANT_LIB_EXT1__ != 1
 #error -D__STDC_WANT_LIB_EXT1__=1 required
 #endif
 
@@ -205,7 +205,7 @@
 #endif
 
 #ifndef HAVE_STRERROR
-extern char *sys_errlist[];
+extern const char *const sys_errlist[];
 #define strerror(i) sys_errlist[i]
 #endif
 
@@ -699,13 +699,9 @@ int rep_strerror_r(int errnum, char *buf, size_t buflen);
 #include <stdbool.h>
 #endif
 
-#if !defined(HAVE_BOOL)
-#ifdef HAVE__Bool
-#define bool _Bool
-#else
+#ifndef HAVE_BOOL
 #error Need a real boolean type
 #endif
-#endif
 
 #if !defined(HAVE_INTPTR_T)
 typedef long long intptr_t ;
@@ -1083,7 +1079,7 @@ static inline bool uid_wrapper_enabled(void)
 static inline bool _hexcharval(char c, uint8_t *val)
 {
 	if ((c >= '0') && (c <= '9')) { *val = c - '0';      return true; }
-	if ((c >= 'a') && (c <= 'f')) {	*val = c - 'a' + 10; return true; }
+        c &= 0xDF; /* map lower to upper case -- thanks libnfs :-) */
 	if ((c >= 'A') && (c <= 'F')) { *val = c - 'A' + 10; return true; }
 	return false;
 }
@@ -1101,4 +1097,16 @@ static inline bool hex_byte(const char *in, uint8_t *out)
 #include <sys/atomic.h>
 #endif
 
+/*
+ * This handles the case of missing pthread support and ensures code can use
+ * __thread unconditionally, such that when built on a platform without pthread
+ * support, the __thread qualifier is an empty define.
+ */
+#ifndef HAVE___THREAD
+# ifdef HAVE_PTHREAD
+# error Configure failed to detect pthread library with missing TLS support
+# endif
+#define HAVE___THREAD
+#endif
+
 #endif /* _LIBREPLACE_REPLACE_H */
