@@ -22,6 +22,7 @@
 #include "lib/util/asn1.h"
 #include "lib/util/debug.h"
 #include "lib/util/samba_util.h"
+#include "lib/util/smb_strtox.h"
 
 struct nesting {
 	off_t start;
@@ -275,7 +276,7 @@ bool asn1_write_BitString(struct asn1_data *data, const void *p, size_t length,
 
 bool ber_write_OID_String(TALLOC_CTX *mem_ctx, DATA_BLOB *blob, const char *OID)
 {
-	unsigned int v, v2;
+	unsigned long int v, v2;
 	const char *p = (const char *)OID;
 	char *newp;
 	int i;
@@ -307,8 +308,11 @@ bool ber_write_OID_String(TALLOC_CTX *mem_ctx, DATA_BLOB *blob, const char *OID)
 		v = smb_strtoul(p, &newp, 10, &error, SMB_STR_STANDARD);
 		if (newp[0] == '.' || error != 0) {
 			p = newp + 1;
-			/* check for empty last component */
-			if (!*p) return false;
+			if (!*p) {
+				/* empty last component */
+				data_blob_free(blob);
+				return false;
+			}
 		} else if (newp[0] == '\0') {
 			p = newp;
 		} else {
@@ -767,7 +771,7 @@ int asn1_tag_remaining(struct asn1_data *data)
 static bool _ber_read_OID_String_impl(TALLOC_CTX *mem_ctx, DATA_BLOB blob,
 				      char **OID, size_t *bytes_eaten)
 {
-	int i;
+	size_t i;
 	uint8_t *b;
 	unsigned int v;
 	char *tmp_oid = NULL;
@@ -777,7 +781,6 @@ static bool _ber_read_OID_String_impl(TALLOC_CTX *mem_ctx, DATA_BLOB blob,
 	b = blob.data;
 
 	tmp_oid = talloc_asprintf(mem_ctx, "%u.%u", b[0]/40, b[0]%40);
-	if (!tmp_oid) goto nomem;
 
 	if (bytes_eaten != NULL) {
 		*bytes_eaten = 0;
@@ -786,12 +789,15 @@ static bool _ber_read_OID_String_impl(TALLOC_CTX *mem_ctx, DATA_BLOB blob,
 	for(i = 1, v = 0; i < blob.length; i++) {
 		v = (v<<7) | (b[i]&0x7f);
 		if ( ! (b[i] & 0x80)) {
-			tmp_oid = talloc_asprintf_append_buffer(tmp_oid, ".%u",  v);
+			talloc_asprintf_addbuf(&tmp_oid, ".%u",  v);
 			v = 0;
 			if (bytes_eaten)
 				*bytes_eaten = i+1;
 		}
-		if (!tmp_oid) goto nomem;
+	}
+
+	if (tmp_oid == NULL) {
+		goto nomem;
 	}
 
 	*OID = tmp_oid;
@@ -1051,7 +1057,7 @@ bool asn1_read_BitString(struct asn1_data *data, TALLOC_CTX *mem_ctx, DATA_BLOB
 /* read a non-negative enumerated value */
 bool asn1_read_enumerated(struct asn1_data *data, int *v)
 {
-	unsigned int val_will_wrap = (0xFF << ((sizeof(int)*8)-8));
+	unsigned int val_will_wrap = (0xFFU << ((sizeof(int)*8)-8));
 	*v = 0;
 
 	if (!asn1_start_tag(data, ASN1_ENUMERATED)) return false;
@@ -1070,7 +1076,11 @@ bool asn1_read_enumerated(struct asn1_data *data, int *v)
 			data->has_error = true;
 			return false;
 		}
-		*v = (*v << 8) + b;
+		/*
+		 * To please/fool the Undefined Behaviour Sanitizer we cast to
+		 * unsigned for the left shift.
+		 */
+		*v = ((unsigned int)*v << 8) + b;
 		if (*v < 0) {
 			/* ASN1_ENUMERATED can't be -ve. */
 			data->has_error = true;
@@ -1080,20 +1090,6 @@ bool asn1_read_enumerated(struct asn1_data *data, int *v)
 	return asn1_end_tag(data);
 }
 
-/* check a enumerated value is correct */
-bool asn1_check_enumerated(struct asn1_data *data, int v)
-{
-	uint8_t b;
-	if (!asn1_start_tag(data, ASN1_ENUMERATED)) return false;
-	if (!asn1_read_uint8(data, &b)) return false;
-	if (!asn1_end_tag(data)) return false;
-
-	if (v != b)
-		data->has_error = false;
-
-	return !data->has_error;
-}
-
 /* write an enumerated value to the stream */
 bool asn1_write_enumerated(struct asn1_data *data, uint8_t v)
 {
