@@ -39,7 +39,7 @@ struct dns_udp_request_state {
 	size_t reply_len;
 };
 
-#define DNS_REQUEST_TIMEOUT 2
+#define DNS_REQUEST_TIMEOUT 10
 
 /* Declare callback functions used below. */
 static void dns_udp_request_get_reply(struct tevent_req *subreq);
@@ -72,8 +72,8 @@ static struct tevent_req *dns_udp_request_send(TALLOC_CTX *mem_ctx,
 		return tevent_req_post(req, ev);
 	}
 
-	ret = tsocket_address_inet_from_strings(state, "ip", server_addr_string,
-						DNS_SERVICE_PORT, &server_addr);
+	ret = tsocket_address_inet_from_hostport_strings(
+	    state, "ip", server_addr_string, DNS_SERVICE_PORT, &server_addr);
 	if (ret != 0) {
 		tevent_req_error(req, errno);
 		return tevent_req_post(req, ev);
@@ -231,8 +231,8 @@ static struct tevent_req *dns_tcp_request_send(TALLOC_CTX *mem_ctx,
 		return tevent_req_post(req, ev);
 	}
 
-	ret = tsocket_address_inet_from_strings(
-		state, "ip", server_addr_string, DNS_SERVICE_PORT, &remote);
+	ret = tsocket_address_inet_from_hostport_strings(
+	    state, "ip", server_addr_string, DNS_SERVICE_PORT, &remote);
 	if (ret != 0) {
 		tevent_req_error(req, errno);
 		return tevent_req_post(req, ev);
@@ -475,6 +475,7 @@ static void dns_cli_request_udp_done(struct tevent_req *subreq)
 		req, struct dns_cli_request_state);
 	DATA_BLOB reply;
 	enum ndr_err_code ndr_err;
+	uint16_t reply_id, operation;
 	int ret;
 
 	ret = dns_udp_request_recv(subreq, state, &reply.data, &reply.length);
@@ -483,6 +484,36 @@ static void dns_cli_request_udp_done(struct tevent_req *subreq)
 		return;
 	}
 
+	if (reply.length < 4) {
+		DBG_DEBUG("Short DNS packet: length=%zu\n", reply.length);
+		tevent_req_error(req, EINVAL);
+		return;
+	}
+
+	reply_id = PULL_BE_U16(reply.data, 0);
+	if (reply_id != state->req_id) {
+		DBG_DEBUG("Got id %"PRIu16", expected %"PRIu16"\n",
+			  state->reply->id, state->req_id);
+		tevent_req_error(req, ENOMSG);
+		return;
+	}
+
+	operation = PULL_BE_U16(reply.data, 2);
+	if ((operation & DNS_FLAG_TRUNCATION) != 0) {
+		DBG_DEBUG("Reply was truncated, retrying TCP\n");
+		subreq = dns_tcp_request_send(
+			state,
+			state->ev,
+			state->nameserver,
+			state->query.data,
+			state->query.length);
+		if (tevent_req_nomem(subreq, req)) {
+			return;
+		}
+		tevent_req_set_callback(subreq, dns_cli_request_tcp_done, req);
+		return;
+	}
+
 	state->reply = talloc(state, struct dns_name_packet);
 	if (tevent_req_nomem(state->reply, req)) {
 		return;
@@ -497,32 +528,7 @@ static void dns_cli_request_udp_done(struct tevent_req *subreq)
 	}
 	TALLOC_FREE(reply.data);
 
-	if (state->reply->id != state->req_id) {
-		DBG_DEBUG("Got id %"PRIu16", expected %"PRIu16"\n",
-			  state->reply->id, state->req_id);
-		tevent_req_error(req, ENOMSG);
-		return;
-	}
-
-	if ((state->reply->operation & DNS_FLAG_TRUNCATION) == 0) {
-		DBG_DEBUG("Got op=%x %"PRIu16"/%"PRIu16"/%"PRIu16"/%"PRIu16
-			  " recs\n", (int)state->reply->operation,
-			  state->reply->qdcount, state->reply->ancount,
-			  state->reply->nscount, state->reply->nscount);
-		tevent_req_done(req);
-		return;
-	}
-
-	DBG_DEBUG("Reply was truncated, retrying TCP\n");
-
-	TALLOC_FREE(state->reply);
-
-	subreq = dns_tcp_request_send(state, state->ev, state->nameserver,
-				      state->query.data, state->query.length);
-	if (tevent_req_nomem(subreq, req)) {
-		return;
-	}
-	tevent_req_set_callback(subreq, dns_cli_request_tcp_done, req);
+	tevent_req_done(req);
 }
 
 static void dns_cli_request_tcp_done(struct tevent_req *subreq)
