@@ -4,6 +4,7 @@
    Winbind client API
 
    Copyright (C) Gerald (Jerry) Carter 2007
+   Copyright (C) Matthew Newton 2015
 
 
    This library is free software; you can redistribute it and/or
@@ -27,12 +28,28 @@
 
 /* From wb_common.c */
 
-NSS_STATUS winbindd_request_response(int req_type,
+struct winbindd_context;
+
+NSS_STATUS winbindd_request_response(struct winbindd_context *wbctx,
+				     int req_type,
 				     struct winbindd_request *request,
 				     struct winbindd_response *response);
-NSS_STATUS winbindd_priv_request_response(int req_type,
+NSS_STATUS winbindd_priv_request_response(struct winbindd_context *wbctx,
+					  int req_type,
 					  struct winbindd_request *request,
 					  struct winbindd_response *response);
+struct winbindd_context *winbindd_ctx_create(void);
+void winbindd_ctx_free(struct winbindd_context *ctx);
+
+/* Global context used for non-Ctx functions */
+
+static struct wbcContext wbcGlobalCtx = {
+	.winbindd_ctx = NULL,
+	.pw_cache_size = 0,
+	.pw_cache_idx = 0,
+	.gr_cache_size = 0,
+	.gr_cache_idx = 0
+};
 
 /*
  result == NSS_STATUS_UNAVAIL: winbind not around
@@ -48,10 +65,11 @@ NSS_STATUS winbindd_priv_request_response(int req_type,
 */
 
 static wbcErr wbcRequestResponseInt(
+	struct winbindd_context *wbctx,
 	int cmd,
 	struct winbindd_request *request,
 	struct winbindd_response *response,
-	NSS_STATUS (*fn)(int req_type,
+	NSS_STATUS (*fn)(struct winbindd_context *wbctx, int req_type,
 			 struct winbindd_request *request,
 			 struct winbindd_response *response))
 {
@@ -60,7 +78,7 @@ static wbcErr wbcRequestResponseInt(
 
 	/* for some calls the request and/or response can be NULL */
 
-	nss_status = fn(cmd, request, response);
+	nss_status = fn(wbctx, cmd, request, response);
 
 	switch (nss_status) {
 	case NSS_STATUS_SUCCESS:
@@ -83,25 +101,40 @@ static wbcErr wbcRequestResponseInt(
 /**
  * @brief Wrapper around Winbind's send/receive API call
  *
+ * @param ctx       Context
  * @param cmd       Winbind command operation to perform
  * @param request   Send structure
  * @param response  Receive structure
  *
  * @return #wbcErr
  */
-wbcErr wbcRequestResponse(int cmd,
+_PUBLIC_ /* this is internal to wbclient_internal.h, but part of the ABI */
+wbcErr wbcRequestResponse(struct wbcContext *ctx, int cmd,
 			  struct winbindd_request *request,
 			  struct winbindd_response *response)
 {
-	return wbcRequestResponseInt(cmd, request, response,
+	struct winbindd_context *wbctx = NULL;
+
+	if (ctx) {
+		wbctx = ctx->winbindd_ctx;
+	}
+
+	return wbcRequestResponseInt(wbctx, cmd, request, response,
 				     winbindd_request_response);
 }
 
-wbcErr wbcRequestResponsePriv(int cmd,
+_PUBLIC_ /* this is internal to wbclient_internal.h, but part of the ABI */
+wbcErr wbcRequestResponsePriv(struct wbcContext *ctx, int cmd,
 			      struct winbindd_request *request,
 			      struct winbindd_response *response)
 {
-	return wbcRequestResponseInt(cmd, request, response,
+	struct winbindd_context *wbctx = NULL;
+
+	if (ctx) {
+		wbctx = ctx->winbindd_ctx;
+	}
+
+	return wbcRequestResponseInt(wbctx, cmd, request, response,
 				     winbindd_priv_request_response);
 }
 
@@ -111,6 +144,7 @@ wbcErr wbcRequestResponsePriv(int cmd,
  *
  * @return a pointer to a static string
  **/
+_PUBLIC_
 const char *wbcErrorString(wbcErr error)
 {
 	switch (error) {
@@ -142,6 +176,8 @@ const char *wbcErrorString(wbcErr error)
 		return "WBC_ERR_AUTH_ERROR";
 	case WBC_ERR_PWD_CHANGE_FAILED:
 		return "WBC_ERR_PWD_CHANGE_FAILED";
+	case WBC_ERR_NOT_MAPPED:
+		return "WBC_ERR_NOT_MAPPED";
 	}
 
 	return "unknown wbcErr value";
@@ -166,6 +202,7 @@ static struct wbcMemPrefix *wbcMemToPrefix(void *ptr)
 	return (struct wbcMemPrefix *)(((char *)ptr) - wbcPrefixLen());
 }
 
+_PUBLIC_ /* this is internal to wbclient_internal.h, but part of the ABI */
 void *wbcAllocateMemory(size_t nelem, size_t elsize,
 			void (*destructor)(void *ptr))
 {
@@ -187,6 +224,7 @@ void *wbcAllocateMemory(size_t nelem, size_t elsize,
 }
 
 /* Free library allocated memory */
+_PUBLIC_
 void wbcFreeMemory(void *p)
 {
 	struct wbcMemPrefix *wbcMem;
@@ -209,6 +247,7 @@ void wbcFreeMemory(void *p)
 	return;
 }
 
+_PUBLIC_ /* this is internal to wbclient_internal.h, but part of the ABI */
 char *wbcStrDup(const char *str)
 {
 	char *result;
@@ -232,6 +271,7 @@ static void wbcStringArrayDestructor(void *ptr)
 	}
 }
 
+_PUBLIC_ /* this is internal to wbclient_internal.h, but part of the ABI */
 const char **wbcAllocateStringArray(int num_strings)
 {
 	return (const char **)wbcAllocateMemory(
@@ -239,6 +279,7 @@ const char **wbcAllocateStringArray(int num_strings)
 		wbcStringArrayDestructor);
 }
 
+_PUBLIC_
 wbcErr wbcLibraryDetails(struct wbcLibraryDetails **_details)
 {
 	struct wbcLibraryDetails *info;
@@ -257,3 +298,49 @@ wbcErr wbcLibraryDetails(struct wbcLibraryDetails **_details)
 	*_details = info;
 	return WBC_ERR_SUCCESS;
 }
+
+/* Context handling functions */
+
+static void wbcContextDestructor(void *ptr)
+{
+	struct wbcContext *ctx = (struct wbcContext *)ptr;
+
+	winbindd_ctx_free(ctx->winbindd_ctx);
+}
+
+_PUBLIC_
+struct wbcContext *wbcCtxCreate(void)
+{
+	struct wbcContext *ctx;
+	struct winbindd_context *wbctx;
+
+	ctx = (struct wbcContext *)wbcAllocateMemory(
+		1, sizeof(struct wbcContext), wbcContextDestructor);
+
+	if (!ctx) {
+		return NULL;
+	}
+
+	wbctx = winbindd_ctx_create();
+
+	if (!wbctx) {
+		wbcFreeMemory(ctx);
+		return NULL;
+	}
+
+	ctx->winbindd_ctx = wbctx;
+
+	return ctx;
+}
+
+_PUBLIC_
+void wbcCtxFree(struct wbcContext *ctx)
+{
+	wbcFreeMemory(ctx);
+}
+
+_PUBLIC_ /* this is internal to wbclient_internal.h, but part of the ABI */
+struct wbcContext *wbcGetGlobalCtx(void)
+{
+	return &wbcGlobalCtx;
+}
