@@ -967,61 +967,146 @@ sub PythonFunctionUnpackOut($$$)
 	my $env = GenerateFunctionOutEnv($fn, "r->");
 	my $result_size = 0;
 
-	$self->pidl("static PyObject *$outfnname(struct $fn->{NAME} *r)");
+	$self->pidl("static PyObject *$outfnname(struct $fn->{NAME} *r, bool raise_result_exception)");
 	$self->pidl("{");
 	$self->indent;
-	$self->pidl("PyObject *result;");
 	foreach my $e (@{$fn->{ELEMENTS}}) {
 		next unless (grep(/out/,@{$e->{DIRECTION}}));
 		next if (($metadata_args->{in}->{$e->{NAME}} and grep(/in/, @{$e->{DIRECTION}})) or
 		         ($metadata_args->{out}->{$e->{NAME}}) and grep(/out/, @{$e->{DIRECTION}}));
-		$self->pidl("PyObject *py_$e->{NAME};");
+		$self->pidl("PyObject *py_$e->{NAME} = NULL;");
 		$result_size++;
 	}
 
-	if ($fn->{RETURN_TYPE}) {
-		$result_size++ unless ($fn->{RETURN_TYPE} eq "WERROR" or $fn->{RETURN_TYPE} eq "NTSTATUS");
+	my $is_raisable_return = 0;
+	if ($fn->{RETURN_TYPE} and ($fn->{RETURN_TYPE} eq "WERROR" or $fn->{RETURN_TYPE} eq "NTSTATUS")) {
+		$is_raisable_return = 1;
+	}
+
+	if ($fn->{RETURN_TYPE} and not $is_raisable_return) {
+		$result_size++;
+	}
+
+	my $max_result_size = $result_size;
+	my $alloc_size = "$result_size";
+	if ($fn->{RETURN_TYPE} and $is_raisable_return) {
+		$max_result_size++;
+	}
+	$self->pidl("PyObject *result = NULL;");
+	if ($max_result_size != $result_size) {
+		$self->pidl("size_t result_size = $result_size;");
+		$alloc_size = "result_size";
+	}
+	$self->pidl("");
+
+	if ($result_size > 1) {
+		$signature .= "(";
+	} elsif ($result_size == 0) {
+		$signature .= "None";
+	}
+
+	if ($fn->{RETURN_TYPE} and $is_raisable_return) {
+		$self->pidl("if (raise_result_exception) {");
+		$self->indent;
+		if (defined($fn->{RETURN_TYPE}) and $fn->{RETURN_TYPE} eq "NTSTATUS") {
+			$self->handle_ntstatus("r->out.result", "NULL", undef);
+		} elsif (defined($fn->{RETURN_TYPE}) and $fn->{RETURN_TYPE} eq "WERROR") {
+			$self->handle_werror("r->out.result", "NULL", undef);
+		}
+		$self->deindent;
+		$self->pidl("} else {");
+		$self->indent;
+		$self->pidl("/* $fn->{RETURN_TYPE} will be part of the results */");
+		$self->pidl("result_size += 1;");
+		$self->deindent;
+		$self->pidl("}");
+		$self->pidl("");
+	}
+
+	if ($max_result_size > 1) {
+		if ($max_result_size != $result_size) {
+			$self->pidl("if (result_size > 1) {");
+			$self->indent;
+		}
+		$self->pidl("result = PyTuple_New($alloc_size);");
+		$self->pidl("if (result == NULL) {");
+		$self->indent;
+		$self->pidl("return NULL;");
+		$self->deindent;
+		$self->pidl("}");
+		if ($max_result_size != $result_size) {
+			$self->deindent;
+			$self->pidl("}");
+		}
+		$self->pidl("");
 	}
 
 	my $i = 0;
 
-	if ($result_size > 1) {
-		$self->pidl("result = PyTuple_New($result_size);");
-		$signature .= "(";
-	} elsif ($result_size == 0) {
-		$self->pidl("result = Py_None;");
-		$self->pidl("Py_INCREF(result);");
-		$signature .= "None";
-	}
-
 	foreach my $e (@{$fn->{ELEMENTS}}) {
 		next if ($metadata_args->{out}->{$e->{NAME}});
 		my $py_name = "py_$e->{NAME}";
 		if (grep(/out/,@{$e->{DIRECTION}})) {
 			$self->ConvertObjectToPython("r", $env, $e, "r->out.$e->{NAME}", $py_name, "return NULL;");
 			if ($result_size > 1) {
-				$self->pidl("PyTuple_SetItem(result, $i, $py_name);");
-				$i++;
 				$signature .= "$e->{NAME}, ";
 			} else {
-				$self->pidl("result = $py_name;");
 				$signature .= $e->{NAME};
 			}
+
+			if ($max_result_size > 1) {
+				if ($max_result_size != $result_size and $result_size == 1) {
+					$self->pidl("if (result_size > 1) {");
+					$self->indent;
+				}
+				$self->pidl("PyTuple_SetItem(result, $i, $py_name);");
+				if ($max_result_size != $result_size and $result_size == 1) {
+					$self->deindent;
+					$self->pidl("}");
+				}
+			}
+			if ($result_size == 1) {
+				if ($max_result_size != $result_size) {
+					$self->pidl("if (result_size == 1) {");
+					$self->indent;
+				}
+				$self->pidl("result = $py_name;");
+				if ($max_result_size != $result_size) {
+					$self->deindent;
+					$self->pidl("}");
+				}
+			}
+			$self->pidl("");
+			$i++;
 		}
 	}
 
-	if (defined($fn->{RETURN_TYPE}) and $fn->{RETURN_TYPE} eq "NTSTATUS") {
-		$self->handle_ntstatus("r->out.result", "NULL", undef);
-	} elsif (defined($fn->{RETURN_TYPE}) and $fn->{RETURN_TYPE} eq "WERROR") {
-		$self->handle_werror("r->out.result", "NULL", undef);
-	} elsif (defined($fn->{RETURN_TYPE})) {
+	if ($fn->{RETURN_TYPE} and $is_raisable_return) {
+		$self->pidl("if (!raise_result_exception) {");
+		$self->indent;
+	}
+
+	if ($fn->{RETURN_TYPE}) {
 		my $conv = $self->ConvertObjectToPythonData("r", $fn->{RETURN_TYPE}, "r->out.result", $fn);
-		if ($result_size > 1) {
+		if ($max_result_size > 1) {
 			$self->pidl("PyTuple_SetItem(result, $i, $conv);");
-		} else {
+		} elsif ($max_result_size == 1) {
 			$self->pidl("result = $conv;");
+		} else {
+			fatal($fn->{ORIGINAL}, "Internal error max_result_size=$max_result_size");
 		}
-		$signature .= "result";
+
+		if (not $is_raisable_return) {
+			$signature .= "result";
+		}
+	}
+
+	if ($fn->{RETURN_TYPE} and $is_raisable_return) {
+		$self->deindent;
+		$self->pidl("}");
+		$self->pidl("");
+	} elsif ($fn->{RETURN_TYPE}) {
+		$self->pidl("");
 	}
 
 	if (substr($signature, -2) eq ", ") {
@@ -1031,6 +1116,20 @@ sub PythonFunctionUnpackOut($$$)
 		$signature .= ")";
 	}
 
+	if ($result_size == 0) {
+		if ($max_result_size != $result_size) {
+			$self->pidl("if (result_size == 0) {");
+			$self->indent;
+		}
+		$self->pidl("result = Py_None;");
+		$self->pidl("Py_INCREF(result);");
+		if ($max_result_size != $result_size) {
+			$self->deindent;
+			$self->pidl("}");
+		}
+		$self->pidl("");
+	}
+
 	$self->pidl("return result;");
 	$self->deindent;
 	$self->pidl("}");
@@ -1417,9 +1516,23 @@ sub Interface($$$)
 		$self->indent;
 		foreach my $d (@fns) {
 			my ($infn, $outfn, $callfn, $prettyname, $docstring, $opnum) = @$d;
-			$self->pidl("{ \"$prettyname\", $docstring, (py_dcerpc_call_fn)$callfn, (py_data_pack_fn)$infn, (py_data_unpack_fn)$outfn, $opnum, &ndr_table_$interface->{NAME} },");
+			$self->pidl("{");
+			$self->indent;
+			$self->pidl(".name = \"$prettyname\",");
+			$self->pidl(".doc = $docstring,");
+			$self->pidl(".call = (py_dcerpc_call_fn)$callfn,");
+			$self->pidl(".pack_in_data = (py_data_pack_fn)$infn,");
+			$self->pidl(".unpack_out_data = (py_data_unpack_fn)$outfn,");
+			$self->pidl(".opnum = $opnum,");
+			$self->pidl(".table = &ndr_table_$interface->{NAME},");
+			$self->deindent;
+			$self->pidl("},");
 		}
-		$self->pidl("{0}");
+		$self->pidl("{");
+		$self->indent;
+		$self->pidl(".name = NULL,");
+		$self->deindent;
+		$self->pidl("},");
 		$self->deindent;
 		$self->pidl("};");
 		$self->pidl("");
@@ -1829,7 +1942,8 @@ sub ConvertObjectFromPythonData($$$$$$;$$)
 		my $ctype_name = $self->use_type_variable($ctype);
 		unless (defined ($ctype_name)) {
 			error($location, "Unable to determine origin of type `" . mapTypeName($ctype) . "'");
-			$self->pidl("PyErr_SetString(PyExc_TypeError, \"Can not convert C Type " . mapTypeName($ctype) . " from Python\");");
+			$self->pidl("PyErr_SetString(PyExc_TypeError, \"Cannot convert Python object to NDR $target\");");
+			$self->pidl("$fail");
 			return;
 		}
 		$self->pidl("PY_CHECK_TYPE($ctype_name, $cvar, $fail);");
@@ -1925,8 +2039,7 @@ sub ConvertObjectFromPythonLevel($$$$$$$$$)
 	if ($recurse == 0) {
 	        $self->pidl("if ($py_var == NULL) {");
 		$self->indent;
-		$self->pidl("PyErr_Format(PyExc_AttributeError, \"Cannot delete NDR object: " .
-                    mapTypeName($var_name) . "\");");
+		$self->pidl("PyErr_Format(PyExc_AttributeError, \"Cannot delete NDR object: $var_name\");");
 		$self->pidl($fail);
 		$self->deindent;
 		$self->pidl("}");
@@ -2295,6 +2408,10 @@ sub ConvertObjectToPythonLevel($$$$$$$)
 		}
 		my $conv = $self->ConvertObjectToPythonData($mem_ctx, $l->{DATA_TYPE}, $var_name, $e->{ORIGINAL});
 		$self->pidl("$py_var = $conv;");
+		if ($conv eq "NULL") {
+			$self->pidl("PyErr_SetString(PyExc_NotImplementedError, \"Cannot convert NDR $var_name to Python\");");
+			$self->pidl("$fail");
+		}
 	} elsif ($l->{TYPE} eq "SUBCONTEXT") {
 		$self->ConvertObjectToPythonLevel($mem_ctx, $env, $e, $nl, $var_name, $py_var, $fail, $recurse);
 	} else {
