@@ -91,6 +91,7 @@ struct cli_state {
 		struct smbXcli_session *session;
 		struct smbXcli_tcon *tcon;
 		struct idr_context *open_handles;
+		bool client_smb311_posix;
 	} smb2;
 };
 
@@ -98,8 +99,6 @@ struct file_info {
 	uint64_t size;
 	uint64_t allocated_size;
 	uint32_t attr;
-	uid_t uid;
-	gid_t gid;
 	uint64_t ino;
 	/* these times are normally kept in GMT */
 	struct timespec btime_ts; /* Birth-time if supported by system */
@@ -108,6 +107,12 @@ struct file_info {
 	struct timespec ctime_ts;
 	char *name;
 	char *short_name;
+	uint32_t reparse_tag;
+	dev_t st_ex_dev;
+	mode_t st_ex_mode;
+	nlink_t st_ex_nlink;
+	struct dom_sid owner_sid;
+	struct dom_sid group_sid;
 };
 
 #define CLI_FULL_CONNECTION_DONT_SPNEGO 0x0001
@@ -119,5 +124,6 @@ struct file_info {
 #define CLI_FULL_CONNECTION_FORCE_SMB1 0x0400
 #define CLI_FULL_CONNECTION_DISABLE_SMB1 0x0800
 #define CLI_FULL_CONNECTION_IPC          0x1000
+#define CLI_FULL_CONNECTION_REQUEST_POSIX 0x2000
 
 #endif /* _CLIENT_H */
