@@ -210,7 +210,7 @@ static const uint32_t set_eid[] = {
 	AD_DEV, AD_INO, AD_SYN, AD_ID
 };
 
-static char empty_resourcefork[] = {
+static const char empty_resourcefork[] = {
 	0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x00,
 	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1E,
 	0x54, 0x68, 0x69, 0x73, 0x20, 0x72, 0x65, 0x73,
