@@ -37,14 +37,6 @@ char *file_id_str_buf(struct file_id fid, struct file_id_buf *dst);
 /*
   an allocated string for a file_id structure
  */
-void push_file_id_16(char *buf, const struct file_id *id);
-void push_file_id_24(char *buf, const struct file_id *id);
-void pull_file_id_24(const char *buf, struct file_id *id);
-
-/*
- * Make a SMB File-ID from itime
- */
-struct stat_ex;
-uint64_t make_file_id_from_itime(const struct stat_ex *st);
+void push_file_id_16(uint8_t *buf, const struct file_id *id);
 
 #endif
