@@ -33,6 +33,7 @@
 #include "../librpc/gen_ndr/ndr_ODJ.h"
 #include "lib/util/base64.h"
 #include "libnet/libnet_join_offline.h"
+#include "libcli/security/dom_sid.h"
 
 /****************************************************************
 ****************************************************************/
@@ -82,14 +83,24 @@ WERROR NetJoinDomain_l(struct libnetapi_ctx *mem_ctx,
 		W_ERROR_HAVE_NO_MEMORY(j->in.account_ou);
 	}
 
-	if (r->in.account) {
-		j->in.admin_account = talloc_strdup(mem_ctx, r->in.account);
-		W_ERROR_HAVE_NO_MEMORY(j->in.admin_account);
-	}
+	if (r->in.account != NULL) {
+		NTSTATUS status;
 
-	if (r->in.password) {
-		j->in.admin_password = talloc_strdup(mem_ctx, r->in.password);
-		W_ERROR_HAVE_NO_MEMORY(j->in.admin_password);
+		status = ads_simple_creds(j,
+					  r->in.domain,
+					  r->in.account,
+					  r->in.password,
+					  &j->in.admin_credentials);
+		if (!NT_STATUS_IS_OK(status)) {
+			TALLOC_FREE(j);
+			return WERR_NERR_BADUSERNAME;
+		}
+	} else {
+		libnetapi_get_creds(mem_ctx, &j->in.admin_credentials);
+		if (j->in.admin_credentials == NULL) {
+			TALLOC_FREE(j);
+			return WERR_NERR_BADUSERNAME;
+		}
 	}
 
 	j->in.join_flags = r->in.join_flags;
@@ -227,14 +238,24 @@ WERROR NetUnjoinDomain_l(struct libnetapi_ctx *mem_ctx,
 		u->in.domain_name = domain;
 	}
 
-	if (r->in.account) {
-		u->in.admin_account = talloc_strdup(mem_ctx, r->in.account);
-		W_ERROR_HAVE_NO_MEMORY(u->in.admin_account);
-	}
+	if (r->in.account != NULL) {
+		NTSTATUS status;
 
-	if (r->in.password) {
-		u->in.admin_password = talloc_strdup(mem_ctx, r->in.password);
-		W_ERROR_HAVE_NO_MEMORY(u->in.admin_password);
+		status = ads_simple_creds(u,
+					  domain,
+					  r->in.account,
+					  r->in.password,
+					  &u->in.admin_credentials);
+		if (!NT_STATUS_IS_OK(status)) {
+			TALLOC_FREE(u);
+			return WERR_NERR_BADUSERNAME;
+		}
+	} else {
+		libnetapi_get_creds(mem_ctx, &u->in.admin_credentials);
+		if (u->in.admin_credentials == NULL) {
+			TALLOC_FREE(u);
+			return WERR_NERR_BADUSERNAME;
+		}
 	}
 
 	u->in.domain_name = domain;
@@ -402,6 +423,7 @@ WERROR NetGetJoinableOUs_l(struct libnetapi_ctx *ctx,
 	NTSTATUS status;
 	ADS_STATUS ads_status;
 	ADS_STRUCT *ads = NULL;
+	struct cli_credentials *creds = NULL;
 	struct netr_DsRGetDCNameInfo *info = NULL;
 	const char *dc = NULL;
 	uint32_t flags = DS_DIRECTORY_SERVICE_REQUIRED |
@@ -434,47 +456,21 @@ WERROR NetGetJoinableOUs_l(struct libnetapi_ctx *ctx,
 		goto out;
 	}
 
-	TALLOC_FREE(ads->auth.user_name);
-	if (r->in.account) {
-		ads->auth.user_name = talloc_strdup(ads, r->in.account);
-		if (ads->auth.user_name == NULL) {
-			ret = WERR_NOT_ENOUGH_MEMORY;
+	if (r->in.account != NULL) {
+		status = ads_simple_creds(ads,
+					  r->in.domain,
+					  r->in.account,
+					  r->in.password,
+					  &creds);
+		if (!NT_STATUS_IS_OK(status)) {
+			ret = WERR_NERR_DEFAULTJOINREQUIRED;
 			goto out;
 		}
 	} else {
-		const char *username = NULL;
-
-		libnetapi_get_username(ctx, &username);
-		if (username != NULL) {
-			ads->auth.user_name = talloc_strdup(ads, username);
-			if (ads->auth.user_name == NULL) {
-				ret = WERR_NOT_ENOUGH_MEMORY;
-				goto out;
-			}
-		}
+		libnetapi_get_creds(ctx, &creds);
 	}
 
-	TALLOC_FREE(ads->auth.password);
-	if (r->in.password) {
-		ads->auth.password = talloc_strdup(ads, r->in.password);
-		if (ads->auth.password == NULL) {
-			ret = WERR_NOT_ENOUGH_MEMORY;
-			goto out;
-		}
-	} else {
-		const char *password = NULL;
-
-		libnetapi_get_password(ctx, &password);
-		if (password != NULL) {
-			ads->auth.password = talloc_strdup(ads, password);
-			if (ads->auth.password == NULL) {
-				ret = WERR_NOT_ENOUGH_MEMORY;
-				goto out;
-			}
-		}
-	}
-
-	ads_status = ads_connect_user_creds(ads);
+	ads_status = ads_connect_creds(ads, creds);
 	if (!ADS_ERR_OK(ads_status)) {
 		ret = WERR_NERR_DEFAULTJOINREQUIRED;
 		goto out;
@@ -637,8 +633,6 @@ static WERROR NetProvisionComputerAccount_backend(struct libnetapi_ctx *ctx,
 {
 	WERROR werr;
 	struct libnet_JoinCtx *j = NULL;
-	int use_kerberos = 0;
-	const char *username = NULL;
 
 	werr = libnet_init_JoinCtx(mem_ctx, &j);
 	if (!W_ERROR_IS_OK(werr)) {
@@ -674,35 +668,12 @@ static WERROR NetProvisionComputerAccount_backend(struct libnetapi_ctx *ctx,
 		}
 	}
 
-	libnetapi_get_username(ctx, &username);
-	if (username == NULL) {
+	libnetapi_get_creds(ctx, &j->in.admin_credentials);
+	if (j->in.admin_credentials == NULL) {
 		talloc_free(j);
 		return WERR_NERR_BADUSERNAME;
 	}
 
-	j->in.admin_account = talloc_strdup(j, username);
-	if (j->in.admin_account == NULL) {
-		talloc_free(j);
-		return WERR_NOT_ENOUGH_MEMORY;
-	}
-
-	libnetapi_get_use_kerberos(ctx, &use_kerberos);
-	if (!use_kerberos) {
-		const char *password = NULL;
-
-		libnetapi_get_password(ctx, &password);
-		if (password == NULL) {
-			talloc_free(j);
-			return WERR_NERR_BADPASSWORD;
-		}
-		j->in.admin_password = talloc_strdup(j, password);
-		if (j->in.admin_password == NULL) {
-			talloc_free(j);
-			return WERR_NOT_ENOUGH_MEMORY;
-		}
-	}
-
-	j->in.use_kerberos = use_kerberos;
 	j->in.debug = true;
 	j->in.join_flags	= WKSSVC_JOIN_FLAGS_JOIN_TYPE |
 				  WKSSVC_JOIN_FLAGS_ACCOUNT_CREATE;
@@ -862,8 +833,10 @@ static WERROR NetRequestOfflineDomainJoin_backend(struct libnetapi_ctx *ctx,
 				  WKSSVC_JOIN_FLAGS_MACHINE_PWD_PASSED;
 
 	werr = libnet_Join(j, j);
-	if (!W_ERROR_IS_OK(werr) && j->out.error_string) {
-		libnetapi_set_error_string(ctx, "%s", j->out.error_string);
+	if (!W_ERROR_IS_OK(werr)) {
+		if (j->out.error_string != NULL) {
+			libnetapi_set_error_string(ctx, "%s", j->out.error_string);
+		}
 		talloc_free(j);
 		return werr;
 	}
@@ -892,6 +865,10 @@ WERROR NetRequestOfflineDomainJoin_l(struct libnetapi_ctx *ctx,
 		return W_ERROR(NERR_BadOfflineJoinInfo);
 	}
 
+	/*
+	 * Windows produces and consumes UTF16/UCS2 encoded blobs. Check for the
+	 * unicode BOM mark and convert back to UNIX charset if necessary.
+	 */
 	if (r->in.provision_bin_data[0] == 0xff &&
 	    r->in.provision_bin_data[1] == 0xfe) {
 		ok = convert_string_talloc(ctx, CH_UTF16LE, CH_UNIX,
@@ -941,3 +918,198 @@ WERROR NetRequestOfflineDomainJoin_l(struct libnetapi_ctx *ctx,
 
 	return W_ERROR(NERR_JoinPerformedMustRestart);
 }
+
+/****************************************************************
+****************************************************************/
+
+WERROR NetComposeOfflineDomainJoin_r(struct libnetapi_ctx *ctx,
+				     struct NetComposeOfflineDomainJoin *r)
+{
+	return WERR_NOT_SUPPORTED;
+}
+
+/****************************************************************
+****************************************************************/
+
+static WERROR NetComposeOfflineDomainJoin_backend(struct libnetapi_ctx *ctx,
+						  struct NetComposeOfflineDomainJoin *r,
+						  TALLOC_CTX *mem_ctx,
+						  struct ODJ_PROVISION_DATA **p)
+{
+	struct libnet_JoinCtx *j = NULL;
+	WERROR werr;
+
+	werr = libnet_init_JoinCtx(ctx, &j);
+	if (!W_ERROR_IS_OK(werr)) {
+		return werr;
+	}
+
+	j->in.domain_name = talloc_strdup(j, r->in.dns_domain_name);
+	if (j->in.domain_name == NULL) {
+		return WERR_NOT_ENOUGH_MEMORY;
+	}
+
+	j->in.dc_name = talloc_strdup(j, r->in.dc_name);
+	W_ERROR_HAVE_NO_MEMORY(j->in.dc_name);
+
+	j->in.machine_password = talloc_strdup(j, r->in.machine_account_password);
+	W_ERROR_HAVE_NO_MEMORY(j->in.machine_password);
+
+	j->out.account_name = talloc_strdup(j, r->in.machine_account_name);
+	W_ERROR_HAVE_NO_MEMORY(j->out.account_name);
+
+	j->out.dns_domain_name = talloc_strdup(j, r->in.dns_domain_name);
+	W_ERROR_HAVE_NO_MEMORY(j->out.dns_domain_name);
+
+	j->out.netbios_domain_name = talloc_strdup(j, r->in.netbios_domain_name);
+	W_ERROR_HAVE_NO_MEMORY(j->out.netbios_domain_name);
+
+	j->out.domain_sid = dom_sid_dup(j, (struct dom_sid *)r->in.domain_sid);
+	W_ERROR_HAVE_NO_MEMORY(j->out.domain_sid);
+
+	j->out.domain_guid = *r->in.domain_guid;
+
+	j->out.forest_name = talloc_strdup(j, r->in.forest_name);
+	W_ERROR_HAVE_NO_MEMORY(j->out.forest_name);
+
+	j->out.domain_is_ad = r->in.domain_is_ad;
+
+	j->out.dcinfo = talloc_zero(j, struct netr_DsRGetDCNameInfo);
+	W_ERROR_HAVE_NO_MEMORY(j->out.dcinfo);
+
+	j->out.dcinfo->dc_unc = talloc_asprintf(j->out.dcinfo, "\\\\%s", r->in.dc_name);
+	W_ERROR_HAVE_NO_MEMORY(j->out.dcinfo->dc_unc);
+
+	j->out.dcinfo->dc_address = talloc_asprintf(j->out.dcinfo, "\\\\%s", r->in.dc_address);
+	W_ERROR_HAVE_NO_MEMORY(j->out.dcinfo->dc_address);
+
+	j->out.dcinfo->dc_address_type = DS_ADDRESS_TYPE_INET;
+
+	j->out.dcinfo->domain_guid = *r->in.domain_guid;
+
+	j->out.dcinfo->domain_name = talloc_strdup(j->out.dcinfo, r->in.dns_domain_name);
+	W_ERROR_HAVE_NO_MEMORY(j->out.dcinfo->domain_name);
+
+	j->out.dcinfo->forest_name = talloc_strdup(j->out.dcinfo, r->in.forest_name);
+	W_ERROR_HAVE_NO_MEMORY(j->out.dcinfo->forest_name);
+
+	werr = libnet_odj_compose_ODJ_PROVISION_DATA(mem_ctx, j, p);
+	if (!W_ERROR_IS_OK(werr)) {
+		return werr;
+	}
+
+	return WERR_OK;
+}
+
+WERROR NetComposeOfflineDomainJoin_l(struct libnetapi_ctx *ctx,
+				     struct NetComposeOfflineDomainJoin *r)
+{
+	WERROR werr;
+	enum ndr_err_code ndr_err;
+	const char *b64_bin_data_str;
+	DATA_BLOB blob;
+	struct ODJ_PROVISION_DATA_serialized_ptr odj_compose_data;
+	struct ODJ_PROVISION_DATA *p;
+	TALLOC_CTX *tmp_ctx = talloc_stackframe();
+
+	if (r->in.compose_bin_data == NULL &&
+	    r->in.compose_text_data == NULL) {
+		werr = WERR_INVALID_PARAMETER;
+		goto out;
+	}
+	if (r->in.compose_bin_data != NULL &&
+	    r->in.compose_text_data != NULL) {
+		werr = WERR_INVALID_PARAMETER;
+		goto out;
+	}
+	if (r->in.compose_bin_data == NULL &&
+	    r->in.compose_bin_data_size != NULL) {
+		werr = WERR_INVALID_PARAMETER;
+		goto out;
+	}
+	if (r->in.compose_bin_data != NULL &&
+	    r->in.compose_bin_data_size == NULL) {
+		werr = WERR_INVALID_PARAMETER;
+		goto out;
+	}
+
+	if (r->in.dns_domain_name == NULL) {
+		werr = WERR_INVALID_PARAMETER;
+		goto out;
+	}
+
+	if (r->in.netbios_domain_name == NULL) {
+		werr = WERR_INVALID_PARAMETER;
+		goto out;
+	}
+
+	if (r->in.domain_sid == NULL) {
+		werr = WERR_INVALID_PARAMETER;
+		goto out;
+	}
+
+	if (r->in.domain_guid == NULL) {
+		werr = WERR_INVALID_PARAMETER;
+		goto out;
+	}
+
+	if (r->in.forest_name == NULL) {
+		werr = WERR_INVALID_PARAMETER;
+		goto out;
+	}
+
+	if (r->in.machine_account_name == NULL) {
+		werr = WERR_INVALID_PARAMETER;
+		goto out;
+	}
+
+	if (r->in.machine_account_password == NULL) {
+		werr = WERR_INVALID_PARAMETER;
+		goto out;
+	}
+
+	if (r->in.dc_name == NULL) {
+		werr = WERR_INVALID_PARAMETER;
+		goto out;
+	}
+
+	if (r->in.dc_address == NULL) {
+		werr = WERR_INVALID_PARAMETER;
+		goto out;
+	}
+
+	werr = NetComposeOfflineDomainJoin_backend(ctx, r, tmp_ctx, &p);
+	if (!W_ERROR_IS_OK(werr)) {
+		goto out;
+	}
+
+	ZERO_STRUCT(odj_compose_data);
+
+	odj_compose_data.s.p = p;
+
+	ndr_err = ndr_push_struct_blob(&blob, ctx, &odj_compose_data,
+		(ndr_push_flags_fn_t)ndr_push_ODJ_PROVISION_DATA_serialized_ptr);
+	if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
+		werr = W_ERROR(NERR_BadOfflineJoinInfo);
+		goto out;
+	}
+
+	if (r->out.compose_text_data != NULL) {
+		b64_bin_data_str = base64_encode_data_blob(ctx, blob);
+		if (b64_bin_data_str == NULL) {
+			werr = WERR_NOT_ENOUGH_MEMORY;
+		}
+		*r->out.compose_text_data = b64_bin_data_str;
+	}
+
+	if (r->out.compose_bin_data != NULL &&
+	    r->out.compose_bin_data_size != NULL) {
+		*r->out.compose_bin_data = blob.data;
+		*r->out.compose_bin_data_size = blob.length;
+	}
+
+	werr = WERR_OK;
+out:
+	talloc_free(tmp_ctx);
+	return werr;
+}
