@@ -25,7 +25,6 @@
 #include "rpc_client/cli_pipe.h"
 #include "../libcli/drsuapi/drsuapi.h"
 #include "../librpc/gen_ndr/ndr_drsuapi_c.h"
-#include "libads/krb5_errs.h"
 
 /****************************************************************
 ****************************************************************/
@@ -523,7 +522,7 @@ static NTSTATUS libnet_dssync_getncchanges(TALLOC_CTX *mem_ctx,
 			ctr6 = &ctr.ctr7.ctr.mszip6.ts->ctr6;
 		} else if (level_out == 7
 			   && ctr.ctr7.level == 6
-			   && ctr.ctr7.type == DRSUAPI_COMPRESSION_TYPE_XPRESS
+			   && ctr.ctr7.type == DRSUAPI_COMPRESSION_TYPE_WIN2K3_LZ77_DIRECT2
 			   && ctr.ctr7.ctr.xpress6.ts) {
 			out_level = 6;
 			ctr6 = &ctr.ctr7.ctr.xpress6.ts->ctr6;
