@@ -43,7 +43,6 @@
 #include "lib/param/loadparm.h"
 #include "libcli/auth/netlogon_creds_cli.h"
 #include "auth/credentials/credentials.h"
-#include "krb5_env.h"
 #include "libsmb/dsgetdcname.h"
 #include "rpc_client/util_netlogon.h"
 #include "libnet/libnet_join_offline.h"
@@ -131,17 +130,13 @@ static void libnet_unjoin_set_error_string(TALLOC_CTX *mem_ctx,
 static ADS_STATUS libnet_connect_ads(const char *dns_domain_name,
 				     const char *netbios_domain_name,
 				     const char *dc_name,
-				     const char *user_name,
-				     const char *password,
-				     const char *ccname,
+				     struct cli_credentials *creds,
 				     TALLOC_CTX *mem_ctx,
 				     ADS_STRUCT **ads)
 {
 	TALLOC_CTX *tmp_ctx = talloc_stackframe();
 	ADS_STATUS status;
 	ADS_STRUCT *my_ads = NULL;
-	char *cp;
-	enum credentials_use_kerberos krb5_state;
 
 	my_ads = ads_init(tmp_ctx,
 			  dns_domain_name,
@@ -153,61 +148,7 @@ static ADS_STATUS libnet_connect_ads(const char *dns_domain_name,
 		goto out;
 	}
 
-	/* In FIPS mode, client use kerberos is forced to required. */
-	krb5_state = lp_client_use_kerberos();
-	switch (krb5_state) {
-	case CRED_USE_KERBEROS_REQUIRED:
-		my_ads->auth.flags &= ~ADS_AUTH_DISABLE_KERBEROS;
-		my_ads->auth.flags &= ~ADS_AUTH_ALLOW_NTLMSSP;
-		break;
-	case CRED_USE_KERBEROS_DESIRED:
-		my_ads->auth.flags &= ~ADS_AUTH_DISABLE_KERBEROS;
-		my_ads->auth.flags |= ADS_AUTH_ALLOW_NTLMSSP;
-		break;
-	case CRED_USE_KERBEROS_DISABLED:
-		my_ads->auth.flags |= ADS_AUTH_DISABLE_KERBEROS;
-		my_ads->auth.flags |= ADS_AUTH_ALLOW_NTLMSSP;
-		break;
-	}
-
-	if (user_name) {
-		ADS_TALLOC_CONST_FREE(my_ads->auth.user_name);
-		my_ads->auth.user_name = talloc_strdup(my_ads, user_name);
-		if (my_ads->auth.user_name == NULL) {
-			status = ADS_ERROR_NT(NT_STATUS_NO_MEMORY);
-			goto out;
-		}
-		if ((cp = strchr_m(my_ads->auth.user_name, '@'))!=0) {
-			*cp++ = '\0';
-			ADS_TALLOC_CONST_FREE(my_ads->auth.realm);
-			my_ads->auth.realm = talloc_asprintf_strupper_m(my_ads, "%s", cp);
-			if (my_ads->auth.realm == NULL) {
-				status = ADS_ERROR_LDAP(LDAP_NO_MEMORY);
-				goto out;
-			}
-		}
-	}
-
-	if (password) {
-		ADS_TALLOC_CONST_FREE(my_ads->auth.password);
-		my_ads->auth.password = talloc_strdup(my_ads, password);
-		if (my_ads->auth.password == NULL) {
-			status = ADS_ERROR_NT(NT_STATUS_NO_MEMORY);
-			goto out;
-		}
-	}
-
-	if (ccname != NULL) {
-		ADS_TALLOC_CONST_FREE(my_ads->auth.ccache_name);
-		my_ads->auth.ccache_name = talloc_strdup(my_ads, ccname);
-		if (my_ads->auth.ccache_name == NULL) {
-			status = ADS_ERROR_NT(NT_STATUS_NO_MEMORY);
-			goto out;
-		}
-		setenv(KRB5_ENV_CCNAME, my_ads->auth.ccache_name, 1);
-	}
-
-	status = ads_connect_user_creds(my_ads);
+	status = ads_connect_creds(my_ads, creds);
 	if (!ADS_ERR_OK(status)) {
 		goto out;
 	}
@@ -228,54 +169,47 @@ static ADS_STATUS libnet_join_connect_ads(TALLOC_CTX *mem_ctx,
 					  bool use_machine_creds)
 {
 	ADS_STATUS status;
-	const char *username;
-	const char *password;
-	const char *ccname = NULL;
+	struct cli_credentials *creds = NULL;
 
 	if (use_machine_creds) {
+		const char *username = NULL;
+		NTSTATUS ntstatus;
+
 		if (r->in.machine_name == NULL ||
 		    r->in.machine_password == NULL) {
 			return ADS_ERROR_NT(NT_STATUS_INVALID_PARAMETER);
 		}
-		username = talloc_asprintf(mem_ctx, "%s$",
-					   r->in.machine_name);
-		if (username == NULL) {
-			return ADS_ERROR(LDAP_NO_MEMORY);
+		if (r->out.dns_domain_name != NULL) {
+			username = talloc_asprintf(mem_ctx, "%s$@%s",
+						   r->in.machine_name,
+						   r->out.dns_domain_name);
+			if (username == NULL) {
+				return ADS_ERROR(LDAP_NO_MEMORY);
+			}
+		} else {
+			username = talloc_asprintf(mem_ctx, "%s$",
+						   r->in.machine_name);
+			if (username == NULL) {
+				return ADS_ERROR(LDAP_NO_MEMORY);
+			}
+		}
+
+		ntstatus = ads_simple_creds(mem_ctx,
+					    r->out.netbios_domain_name,
+					    username,
+					    r->in.machine_password,
+					    &creds);
+		if (!NT_STATUS_IS_OK(ntstatus)) {
+			return ADS_ERROR_NT(ntstatus);
 		}
-		password = r->in.machine_password;
-		ccname = "MEMORY:libnet_join_machine_creds";
 	} else {
-		char *p = NULL;
-
-		username = r->in.admin_account;
-
-		p = strchr(r->in.admin_account, '@');
-		if (p == NULL) {
-			username = talloc_asprintf(mem_ctx, "%s@%s",
-						   r->in.admin_account,
-						   r->in.admin_domain);
-		}
-		if (username == NULL) {
-			return ADS_ERROR(LDAP_NO_MEMORY);
-		}
-		password = r->in.admin_password;
-
-		/*
-		 * when r->in.use_kerberos is set to allow "net ads join -k" we
-		 * may not override the provided credential cache - gd
-		 */
-
-		if (!r->in.use_kerberos) {
-			ccname = "MEMORY:libnet_join_user_creds";
-		}
+		creds = r->in.admin_credentials;
 	}
 
 	status = libnet_connect_ads(r->out.dns_domain_name,
 				    r->out.netbios_domain_name,
 				    r->in.dc_name,
-				    username,
-				    password,
-				    ccname,
+				    creds,
 				    r,
 				    &r->in.ads);
 	if (!ADS_ERR_OK(status)) {
@@ -331,9 +265,7 @@ static ADS_STATUS libnet_unjoin_connect_ads(TALLOC_CTX *mem_ctx,
 	status = libnet_connect_ads(r->in.domain_name,
 				    r->in.domain_name,
 				    r->in.dc_name,
-				    r->in.admin_account,
-				    r->in.admin_password,
-				    NULL,
+				    r->in.admin_credentials,
 				    r,
 				    &r->in.ads);
 	if (!ADS_ERR_OK(status)) {
@@ -559,6 +491,7 @@ static ADS_STATUS libnet_join_set_machine_spn(TALLOC_CTX *mem_ctx,
 	char *spn = NULL;
 	const char **netbios_aliases = NULL;
 	const char **addl_hostnames = NULL;
+	const char *dns_hostname = NULL;
 
 	/* Find our DN */
 
@@ -615,6 +548,34 @@ static ADS_STATUS libnet_join_set_machine_spn(TALLOC_CTX *mem_ctx,
 		goto done;
 	}
 
+	/*
+	 * Register dns_hostname if needed, add_uniq_spn() will avoid
+	 * duplicates.
+	 */
+	if (r->in.dnshostname != NULL) {
+		dns_hostname = talloc_strdup(frame, r->in.dnshostname);
+	} else {
+		dns_hostname = talloc_asprintf(frame,
+					       "%s.%s",
+					       r->in.machine_name,
+					       r->out.dns_domain_name);
+	}
+	if (dns_hostname == NULL) {
+		status = ADS_ERROR_LDAP(LDAP_NO_MEMORY);
+		goto done;
+	}
+
+	spn = talloc_asprintf(frame, "HOST/%s", dns_hostname);
+	if (spn == NULL) {
+		status = ADS_ERROR_LDAP(LDAP_NO_MEMORY);
+		goto done;
+	}
+
+	status = add_uniq_spn(frame, spn, &spn_array, &num_spns);
+	if (!ADS_ERR_OK(status)) {
+		goto done;
+	}
+
 	for (netbios_aliases = lp_netbios_aliases();
 	     netbios_aliases != NULL && *netbios_aliases != NULL;
 	     netbios_aliases++) {
@@ -642,6 +603,10 @@ static ADS_STATUS libnet_join_set_machine_spn(TALLOC_CTX *mem_ctx,
 		fstr_sprintf(my_alias, "%s.%s",
 			     *netbios_aliases,
 			     lp_dnsdomain());
+		if (!strlower_m(my_alias)) {
+			status = ADS_ERROR_LDAP(LDAP_NO_MEMORY);
+			goto done;
+		}
 
 		spn = talloc_asprintf(frame, "HOST/%s", my_alias);
 		if (spn == NULL) {
@@ -901,15 +866,9 @@ static ADS_STATUS libnet_join_set_etypes(TALLOC_CTX *mem_ctx,
 static bool libnet_join_create_keytab(TALLOC_CTX *mem_ctx,
 				      struct libnet_JoinCtx *r)
 {
-	if (!USE_SYSTEM_KEYTAB) {
-		return true;
-	}
+	NTSTATUS ntstatus = sync_pw2keytabs();
 
-	if (ads_keytab_create_default(r->in.ads) != 0) {
-		return false;
-	}
-
-	return true;
+	return NT_STATUS_IS_OK(ntstatus);
 }
 
 /****************************************************************
@@ -1043,11 +1002,6 @@ static ADS_STATUS libnet_join_post_processing_ads_modify(TALLOC_CTX *mem_ctx,
 		 * to update msDS-SupportedEncryptionTypes reliable
 		 */
 
-		if (r->in.ads->auth.ccache_name != NULL) {
-			ads_kdestroy(r->in.ads->auth.ccache_name);
-			ADS_TALLOC_CONST_FREE(r->in.ads->auth.ccache_name);
-		}
-
 		TALLOC_FREE(r->in.ads);
 
 		status = libnet_join_connect_ads_machine(mem_ctx, r);
@@ -1110,40 +1064,16 @@ static bool libnet_join_joindomain_store_secrets(TALLOC_CTX *mem_ctx,
  Connect dc's IPC$ share
 ****************************************************************/
 
-static NTSTATUS libnet_join_connect_dc_ipc(const char *dc,
-					   const char *user,
-					   const char *domain,
-					   const char *pass,
-					   bool use_kerberos,
+static NTSTATUS libnet_join_connect_dc_ipc(TALLOC_CTX *mem_ctx,
+					   const char *dc,
+					   struct cli_credentials *creds,
 					   struct cli_state **cli)
 {
-	TALLOC_CTX *frame = talloc_stackframe();
-	bool fallback_after_kerberos = false;
-	bool use_ccache = false;
-	bool pw_nt_hash = false;
-	struct cli_credentials *creds = NULL;
 	int flags = CLI_FULL_CONNECTION_IPC;
 	NTSTATUS status;
 
-	if (use_kerberos && pass) {
-		fallback_after_kerberos = true;
-	}
-
-	creds = cli_session_creds_init(frame,
-				       user,
-				       domain,
-				       NULL, /* realm (use default) */
-				       pass,
-				       use_kerberos,
-				       fallback_after_kerberos,
-				       use_ccache,
-				       pw_nt_hash);
-	if (creds == NULL) {
-		TALLOC_FREE(frame);
-		return NT_STATUS_NO_MEMORY;
-	}
-
-	status = cli_full_connection_creds(cli,
+	status = cli_full_connection_creds(mem_ctx,
+					   cli,
 					   NULL,
 					   dc,
 					   NULL, 0,
@@ -1151,11 +1081,9 @@ static NTSTATUS libnet_join_connect_dc_ipc(const char *dc,
 					   creds,
 					   flags);
 	if (!NT_STATUS_IS_OK(status)) {
-		TALLOC_FREE(frame);
 		return status;
 	}
 
-	TALLOC_FREE(frame);
 	return NT_STATUS_OK;
 }
 
@@ -1167,28 +1095,27 @@ static NTSTATUS libnet_join_lookup_dc_rpc(TALLOC_CTX *mem_ctx,
 					  struct libnet_JoinCtx *r,
 					  struct cli_state **cli)
 {
+	TALLOC_CTX *frame = talloc_stackframe();
 	struct rpc_pipe_client *pipe_hnd = NULL;
 	struct policy_handle lsa_pol;
 	NTSTATUS status, result;
 	union lsa_PolicyInformation *info = NULL;
+	struct cli_credentials *creds = NULL;
 	struct dcerpc_binding_handle *b;
-	const char *account = r->in.admin_account;
-	const char *domain = r->in.admin_domain;
-	const char *password = r->in.admin_password;
-	bool use_kerberos = r->in.use_kerberos;
 
 	if (r->in.join_flags & WKSSVC_JOIN_FLAGS_JOIN_UNSECURE) {
-		account = "";
-		domain = "";
-		password = NULL;
-		use_kerberos = false;
+		creds = cli_credentials_init_anon(frame);
+		if (creds == NULL) {
+			status = NT_STATUS_NO_MEMORY;
+			goto done;
+		}
+	} else {
+		creds = r->in.admin_credentials;
 	}
 
-	status = libnet_join_connect_dc_ipc(r->in.dc_name,
-					    account,
-					    domain,
-					    password,
-					    use_kerberos,
+	status = libnet_join_connect_dc_ipc(mem_ctx,
+					    r->in.dc_name,
+					    creds,
 					    cli);
 	if (!NT_STATUS_IS_OK(status)) {
 		goto done;
@@ -1248,6 +1175,7 @@ static NTSTATUS libnet_join_lookup_dc_rpc(TALLOC_CTX *mem_ctx,
 	TALLOC_FREE(pipe_hnd);
 
  done:
+	TALLOC_FREE(frame);
 	return status;
 }
 
@@ -1305,7 +1233,8 @@ static NTSTATUS libnet_join_joindomain_rpc_unsecure(TALLOC_CTX *mem_ctx,
 						r->in.secure_channel_type);
 
 	/* according to WKSSVC_JOIN_FLAGS_MACHINE_PWD_PASSED */
-	cli_credentials_set_password(cli_creds, r->in.admin_password,
+	cli_credentials_set_password(cli_creds,
+				     r->in.passed_machine_password,
 				     CRED_SPECIFIED);
 
 	status = rpccli_create_netlogon_creds_ctx(
@@ -1357,7 +1286,7 @@ static NTSTATUS libnet_join_joindomain_rpc_unsecure(TALLOC_CTX *mem_ctx,
 	len = strlen(r->in.machine_password);
 	ok = convert_string_talloc(frame, CH_UNIX, CH_UTF16,
 				   r->in.machine_password, len,
-				   (void **)&new_trust_blob.data,
+				   &new_trust_blob.data,
 				   &new_trust_blob.length);
 	if (!ok) {
 		status = NT_STATUS_UNMAPPABLE_CHARACTER;
@@ -1733,7 +1662,7 @@ error:
 NTSTATUS libnet_join_ok(struct messaging_context *msg_ctx,
 			const char *netbios_domain_name,
 			const char *dc_name,
-			const bool use_kerberos)
+			enum credentials_use_kerberos kerberos_state)
 {
 	TALLOC_CTX *frame = talloc_stackframe();
 	struct cli_state *cli = NULL;
@@ -1767,13 +1696,13 @@ NTSTATUS libnet_join_ok(struct messaging_context *msg_ctx,
 	/* we don't want any old password */
 	cli_credentials_set_old_password(cli_creds, NULL, CRED_SPECIFIED);
 
-	if (use_kerberos) {
-		cli_credentials_set_kerberos_state(cli_creds,
-						   CRED_USE_KERBEROS_REQUIRED,
-						   CRED_SPECIFIED);
-	}
+	cli_credentials_set_kerberos_state(cli_creds,
+					   kerberos_state,
+					   CRED_SPECIFIED);
 
-	status = cli_full_connection_creds(&cli, NULL,
+	status = cli_full_connection_creds(frame,
+					   &cli,
+					   NULL,
 					   dc_name,
 					   NULL, 0,
 					   "IPC$", "IPC",
@@ -1789,7 +1718,8 @@ NTSTATUS libnet_join_ok(struct messaging_context *msg_ctx,
 			return NT_STATUS_NO_MEMORY;
 		}
 
-		status = cli_full_connection_creds(&cli,
+		status = cli_full_connection_creds(frame,
+						   &cli,
 						   NULL,
 						   dc_name,
 						   NULL, 0,
@@ -1881,11 +1811,17 @@ static WERROR libnet_join_post_verify(TALLOC_CTX *mem_ctx,
 				      struct libnet_JoinCtx *r)
 {
 	NTSTATUS status;
+	enum credentials_use_kerberos kerberos_state = CRED_USE_KERBEROS_DESIRED;
+
+	if (r->in.admin_credentials != NULL) {
+		kerberos_state = cli_credentials_get_kerberos_state(
+					r->in.admin_credentials);
+	}
 
 	status = libnet_join_ok(r->in.msg_ctx,
 				r->out.netbios_domain_name,
 				r->in.dc_name,
-				r->in.use_kerberos);
+				kerberos_state);
 	if (!NT_STATUS_IS_OK(status)) {
 		libnet_join_set_error_string(mem_ctx, r,
 			"failed to verify domain membership after joining: %s",
@@ -1930,11 +1866,9 @@ static NTSTATUS libnet_join_unjoindomain_rpc(TALLOC_CTX *mem_ctx,
 	ZERO_STRUCT(domain_pol);
 	ZERO_STRUCT(user_pol);
 
-	status = libnet_join_connect_dc_ipc(r->in.dc_name,
-					    r->in.admin_account,
-					    r->in.admin_domain,
-					    r->in.admin_password,
-					    r->in.use_kerberos,
+	status = libnet_join_connect_dc_ipc(mem_ctx,
+					    r->in.dc_name,
+					    r->in.admin_credentials,
 					    &cli);
 	if (!NT_STATUS_IS_OK(status)) {
 		goto done;
@@ -2348,27 +2282,6 @@ static WERROR libnet_join_pre_processing(TALLOC_CTX *mem_ctx,
 		return WERR_OK;
 	}
 
-	if (!r->in.admin_domain) {
-		char *admin_domain = NULL;
-		char *admin_account = NULL;
-		bool ok;
-
-		ok = split_domain_user(mem_ctx,
-				       r->in.admin_account,
-				       &admin_domain,
-				       &admin_account);
-		if (!ok) {
-			return WERR_NOT_ENOUGH_MEMORY;
-		}
-
-		if (admin_domain != NULL) {
-			r->in.admin_domain = admin_domain;
-		} else {
-			r->in.admin_domain = r->in.domain_name;
-		}
-		r->in.admin_account = admin_account;
-	}
-
 	if (r->in.provision_computer_account_only) {
 		/*
 		 * When in the "provision_computer_account_only" path we do not
@@ -2713,8 +2626,7 @@ static WERROR libnet_DomainJoin(TALLOC_CTX *mem_ctx,
 	/* Before contacting a DC, we can securely know
 	 * the realm only if the user specifies it.
 	 */
-	if (r->in.use_kerberos &&
-	    r->in.domain_name_type == JoinDomNameTypeDNS) {
+	if (r->in.domain_name_type == JoinDomNameTypeDNS) {
 		pre_connect_realm = r->in.domain_name;
 	}
 
@@ -2795,7 +2707,7 @@ static WERROR libnet_DomainJoin(TALLOC_CTX *mem_ctx,
 		}
 
 		/* The domain parameter is only used as modifier
-		 * to krb5.conf file name. _JOIN_ is is not a valid
+		 * to krb5.conf file name. _JOIN_ is not a valid
 		 * NetBIOS name so it cannot clash with another domain
 		 * -- Uri.
 		 */
@@ -2997,10 +2909,8 @@ static WERROR libnet_join_rollback(TALLOC_CTX *mem_ctx,
 	u->in.debug		= r->in.debug;
 	u->in.dc_name		= r->in.dc_name;
 	u->in.domain_name	= r->in.domain_name;
-	u->in.admin_account	= r->in.admin_account;
-	u->in.admin_password	= r->in.admin_password;
+	u->in.admin_credentials	= r->in.admin_credentials;
 	u->in.modify_config	= r->in.modify_config;
-	u->in.use_kerberos	= r->in.use_kerberos;
 	u->in.unjoin_flags	= WKSSVC_JOIN_FLAGS_JOIN_TYPE |
 				  WKSSVC_JOIN_FLAGS_ACCOUNT_DELETE;
 
@@ -3206,27 +3116,6 @@ static WERROR libnet_unjoin_pre_processing(TALLOC_CTX *mem_ctx,
 		return WERR_NERR_SETUPDOMAINCONTROLLER;
 	}
 
-	if (!r->in.admin_domain) {
-		char *admin_domain = NULL;
-		char *admin_account = NULL;
-		bool ok;
-
-		ok = split_domain_user(mem_ctx,
-				       r->in.admin_account,
-				       &admin_domain,
-				       &admin_account);
-		if (!ok) {
-			return WERR_NOT_ENOUGH_MEMORY;
-		}
-
-		if (admin_domain != NULL) {
-			r->in.admin_domain = admin_domain;
-		} else {
-			r->in.admin_domain = r->in.domain_name;
-		}
-		r->in.admin_account = admin_account;
-	}
-
 	if (!secrets_init()) {
 		libnet_unjoin_set_error_string(mem_ctx, r,
 			"Unable to open secrets database");
