@@ -261,6 +261,12 @@ static struct tevent_req *btrfs_offload_write_send(struct vfs_handle_struct *han
 
 	state->handle = handle;
 
+	status = vfs_offload_token_ctx_init(handle->conn->sconn->client,
+					    &btrfs_offload_ctx);
+	if (tevent_req_nterror(req, status)) {
+		return tevent_req_post(req, ev);
+	}
+
 	tevent_req_set_cleanup_fn(req, btrfs_offload_write_cleanup);
 
 	status = vfs_offload_token_db_fetch_fsp(btrfs_offload_ctx,
@@ -466,8 +472,7 @@ static NTSTATUS btrfs_fget_compression(struct vfs_handle_struct *handle,
 				       struct files_struct *fsp,
 				       uint16_t *_compression_fmt)
 {
-	char buf[PATH_MAX];
-	const char *p = NULL;
+	struct sys_proc_fd_path_buf buf;
 	int ret;
 	long flags = 0;
 	int fsp_fd = fsp_get_pathref_fd(fsp);
@@ -493,14 +498,11 @@ static NTSTATUS btrfs_fget_compression(struct vfs_handle_struct *handle,
 		return NT_STATUS_NOT_IMPLEMENTED;
 	}
 
-	p = sys_proc_fd_path(fsp_fd, buf, sizeof(buf));
-	if (p == NULL) {
-		return NT_STATUS_NO_MEMORY;
-	}
-
-	fd = open(p, O_RDONLY);
+	fd = open(sys_proc_fd_path(fsp_fd, &buf), O_RDONLY);
 	if (fd == -1) {
-		DBG_DEBUG("/proc open of %s failed: %s\n", p, strerror(errno));
+		DBG_DEBUG("/proc open of %s failed: %s\n",
+			  buf.buf,
+			  strerror(errno));
 		return map_nt_error_from_unix(errno);
 	}
 
@@ -779,7 +781,7 @@ static NTSTATUS btrfs_snap_delete(struct vfs_handle_struct *handle,
 				  char *snap_path)
 {
 	char *tstr;
-	struct tm t_gmt;
+	struct tm t_gmt = {};
 	DIR *dest_dir;
 	int dest_fd;
 	struct btrfs_ioctl_vol_args ioctl_arg;
