@@ -49,21 +49,34 @@
 #endif
 
 /*
- * Use %llu whenever we have a 64bit unsigned int, and cast to (long long unsigned)
+ * Use %llu whenever we have a 64bit unsigned int, and cast to (long long
+ * unsigned)
  */
 #define llu(_var) ((long long unsigned)_var)
 
 /*
- * Note, libcephfs's return code model is to return -errno! So we have to
- * convert to what Samba expects, which is to set errno to -return and return -1
+ * Note, libcephfs's return code model is to return -errno. Thus we have to
+ * convert to what Samba expects: set errno to non-negative value and return -1.
+ *
+ * Using convenience helper functions to avoid non-hygienic macro.
  */
-#define WRAP_RETURN(_res) \
-	errno = 0; \
-	if (_res < 0) { \
-		errno = -_res; \
-		return -1; \
-	} \
-	return _res \
+static inline int status_code(int ret)
+{
+	if (ret < 0) {
+		errno = -ret;
+		return -1;
+	}
+	return ret;
+}
+
+static inline ssize_t lstatus_code(intmax_t ret)
+{
+	if (ret < 0) {
+		errno = -((int)ret);
+		return -1;
+	}
+	return (ssize_t)ret;
+}
 
 /*
  * Track unique connections, as virtual mounts, to cephfs file systems.
@@ -159,23 +172,6 @@ static char *cephmount_get_cookie(TALLOC_CTX * mem_ctx, const int snum)
 			       fsname);
 }
 
-static int cephmount_select_fs(struct ceph_mount_info *mnt, const char *fsname)
-{
-	/*
-	 * ceph_select_filesystem was added in ceph 'nautilus' (v14).
-	 * Earlier versions of libcephfs will lack that API function.
-	 * At the time of this writing (Feb 2023) all versions of ceph
-	 * supported by ceph upstream have this function.
-	 */
-#if defined(HAVE_CEPH_SELECT_FILESYSTEM)
-	DBG_DEBUG("[CEPH] calling: ceph_select_filesystem with %s\n", fsname);
-	return ceph_select_filesystem(mnt, fsname);
-#else
-	DBG_ERR("[CEPH] ceph_select_filesystem not available\n");
-	return -ENOTSUP;
-#endif
-}
-
 static struct ceph_mount_info *cephmount_mount_fs(const int snum)
 {
 	int ret;
@@ -221,11 +217,11 @@ static struct ceph_mount_info *cephmount_mount_fs(const int snum)
 	}
 	/*
 	 * select a cephfs file system to use:
-	 * In ceph, multiple file system support has been stable since 'pacific'.
-	 * Permit different shares to access different file systems.
+	 * In ceph, multiple file system support has been stable since
+	 * 'pacific'. Permit different shares to access different file systems.
 	 */
 	if (fsname != NULL) {
-		ret = cephmount_select_fs(mnt, fsname);
+		ret = ceph_select_filesystem(mnt, fsname);
 		if (ret < 0) {
 			goto err_cm_release;
 		}
@@ -331,65 +327,20 @@ static uint64_t cephwrap_disk_free(struct vfs_handle_struct *handle,
 	struct statvfs statvfs_buf = { 0 };
 	int ret;
 
-	if (!(ret = ceph_statfs(handle->data, smb_fname->base_name,
-			&statvfs_buf))) {
-		/*
-		 * Provide all the correct values.
-		 */
-		*bsize = statvfs_buf.f_bsize;
-		*dfree = statvfs_buf.f_bavail;
-		*dsize = statvfs_buf.f_blocks;
-		DBG_DEBUG("[CEPH] bsize: %llu, dfree: %llu, dsize: %llu\n",
-			llu(*bsize), llu(*dfree), llu(*dsize));
-		return *dfree;
-	} else {
+	ret = ceph_statfs(handle->data, smb_fname->base_name, &statvfs_buf);
+	if (ret < 0) {
 		DBG_DEBUG("[CEPH] ceph_statfs returned %d\n", ret);
-		WRAP_RETURN(ret);
+		return (uint64_t)status_code(ret);
 	}
-}
-
-static int cephwrap_get_quota(struct vfs_handle_struct *handle,
-				const struct smb_filename *smb_fname,
-				enum SMB_QUOTA_TYPE qtype,
-				unid_t id,
-				SMB_DISK_QUOTA *qt)
-{
-	/* libcephfs: Ceph does not implement this */
-#if 0
-/* was ifdef HAVE_SYS_QUOTAS */
-	int ret;
-
-	ret = ceph_get_quota(handle->conn->connectpath, qtype, id, qt);
-
-	if (ret) {
-		errno = -ret;
-		ret = -1;
-	}
-
-	return ret;
-#else
-	errno = ENOSYS;
-	return -1;
-#endif
-}
-
-static int cephwrap_set_quota(struct vfs_handle_struct *handle,  enum SMB_QUOTA_TYPE qtype, unid_t id, SMB_DISK_QUOTA *qt)
-{
-	/* libcephfs: Ceph does not implement this */
-#if 0
-/* was ifdef HAVE_SYS_QUOTAS */
-	int ret;
-
-	ret = ceph_set_quota(handle->conn->connectpath, qtype, id, qt);
-	if (ret) {
-		errno = -ret;
-		ret = -1;
-	}
-
-	return ret;
-#else
-	WRAP_RETURN(-ENOSYS);
-#endif
+	/*
+	 * Provide all the correct values.
+	 */
+	*bsize = statvfs_buf.f_bsize;
+	*dfree = statvfs_buf.f_bavail;
+	*dsize = statvfs_buf.f_blocks;
+	DBG_DEBUG("[CEPH] bsize: %llu, dfree: %llu, dsize: %llu\n", llu(*bsize),
+		llu(*dfree), llu(*dsize));
+	return *dfree;
 }
 
 static int cephwrap_statvfs(struct vfs_handle_struct *handle,
@@ -401,7 +352,7 @@ static int cephwrap_statvfs(struct vfs_handle_struct *handle,
 
 	ret = ceph_statfs(handle->data, smb_fname->base_name, &statvfs_buf);
 	if (ret < 0) {
-		WRAP_RETURN(ret);
+		return status_code(ret);
 	}
 
 	statbuf->OptimalTransferSize = statvfs_buf.f_frsize;
@@ -412,15 +363,19 @@ static int cephwrap_statvfs(struct vfs_handle_struct *handle,
 	statbuf->TotalFileNodes = statvfs_buf.f_files;
 	statbuf->FreeFileNodes = statvfs_buf.f_ffree;
 	statbuf->FsIdentifier = statvfs_buf.f_fsid;
-	DBG_DEBUG("[CEPH] f_bsize: %ld, f_blocks: %ld, f_bfree: %ld, f_bavail: %ld\n",
-		(long int)statvfs_buf.f_bsize, (long int)statvfs_buf.f_blocks,
-		(long int)statvfs_buf.f_bfree, (long int)statvfs_buf.f_bavail);
+	DBG_DEBUG("[CEPH] f_bsize: %ld, f_blocks: %ld, f_bfree: %ld, "
+		  "f_bavail: %ld\n",
+		  (long int)statvfs_buf.f_bsize,
+		  (long int)statvfs_buf.f_blocks,
+		  (long int)statvfs_buf.f_bfree,
+		  (long int)statvfs_buf.f_bavail);
 
 	return ret;
 }
 
-static uint32_t cephwrap_fs_capabilities(struct vfs_handle_struct *handle,
-					 enum timestamp_set_resolution *p_ts_res)
+static uint32_t cephwrap_fs_capabilities(
+	struct vfs_handle_struct *handle,
+	enum timestamp_set_resolution *p_ts_res)
 {
 	uint32_t caps = FILE_CASE_SENSITIVE_SEARCH | FILE_CASE_PRESERVED_NAMES;
 
@@ -439,14 +394,9 @@ static DIR *cephwrap_fdopendir(struct vfs_handle_struct *handle,
 	int ret = 0;
 	struct ceph_dir_result *result = NULL;
 
-#ifdef HAVE_CEPH_FDOPENDIR
 	int dirfd = fsp_get_io_fd(fsp);
 	DBG_DEBUG("[CEPH] fdopendir(%p, %d)\n", handle, dirfd);
 	ret = ceph_fdopendir(handle->data, dirfd, &result);
-#else
-	DBG_DEBUG("[CEPH] fdopendir(%p, %p)\n", handle, fsp);
-	ret = ceph_opendir(handle->data, fsp->fsp_name->base_name, &result);
-#endif
 	if (ret < 0) {
 		result = NULL;
 		errno = -ret; /* We return result which is NULL in this case */
@@ -481,7 +431,6 @@ static int cephwrap_mkdirat(struct vfs_handle_struct *handle,
 			mode_t mode)
 {
 	int result = -1;
-#ifdef HAVE_CEPH_MKDIRAT
 	int dirfd = fsp_get_pathref_fd(dirfsp);
 
 	DBG_DEBUG("[CEPH] mkdirat(%p, %d, %s)\n",
@@ -493,26 +442,7 @@ static int cephwrap_mkdirat(struct vfs_handle_struct *handle,
 
 	DBG_DEBUG("[CEPH] mkdirat(...) = %d\n", result);
 
-	WRAP_RETURN(result);
-#else
-	struct smb_filename *full_fname = NULL;
-
-	full_fname = full_path_from_dirfsp_atname(talloc_tos(),
-						dirfsp,
-						smb_fname);
-	if (full_fname == NULL) {
-		return -1;
-	}
-
-	DBG_DEBUG("[CEPH] mkdir(%p, %s)\n",
-		  handle, smb_fname_str_dbg(full_fname));
-
-	result = ceph_mkdir(handle->data, full_fname->base_name, mode);
-
-	TALLOC_FREE(full_fname);
-
-	WRAP_RETURN(result);
-#endif
+	return status_code(result);
 }
 
 static int cephwrap_closedir(struct vfs_handle_struct *handle, DIR *dirp)
@@ -522,7 +452,7 @@ static int cephwrap_closedir(struct vfs_handle_struct *handle, DIR *dirp)
 	DBG_DEBUG("[CEPH] closedir(%p, %p)\n", handle, dirp);
 	result = ceph_closedir(handle->data, (struct ceph_dir_result *) dirp);
 	DBG_DEBUG("[CEPH] closedir(...) = %d\n", result);
-	WRAP_RETURN(result);
+	return status_code(result);
 }
 
 /* File operations */
@@ -539,9 +469,7 @@ static int cephwrap_openat(struct vfs_handle_struct *handle,
 	bool have_opath = false;
 	bool became_root = false;
 	int result = -ENOENT;
-#ifdef HAVE_CEPH_OPENAT
 	int dirfd = -1;
-#endif
 
 	if (how->resolve != 0) {
 		errno = ENOSYS;
@@ -559,7 +487,6 @@ static int cephwrap_openat(struct vfs_handle_struct *handle,
 	}
 #endif
 
-#ifdef HAVE_CEPH_OPENAT
 	dirfd = fsp_get_pathref_fd(dirfsp);
 
 	DBG_DEBUG("[CEPH] openat(%p, %d, %p, %d, %d)\n",
@@ -576,35 +503,15 @@ static int cephwrap_openat(struct vfs_handle_struct *handle,
 			     flags,
 			     mode);
 
-#else
-	if (fsp_get_pathref_fd(dirfsp) != AT_FDCWD) {
-		name = full_path_from_dirfsp_atname(talloc_tos(),
-						    dirfsp,
-						    smb_fname);
-		if (name == NULL) {
-			return -1;
-		}
-		smb_fname = name;
-	}
-
-	DBG_DEBUG("[CEPH] openat(%p, %s, %p, %d, %d)\n", handle,
-		  smb_fname_str_dbg(smb_fname), fsp, flags, mode);
-
-	if (fsp->fsp_flags.is_pathref && !have_opath) {
-		become_root();
-		became_root = true;
-	}
-
-	result = ceph_open(handle->data, smb_fname->base_name, flags, mode);
-#endif
 	if (became_root) {
 		unbecome_root();
 	}
+
 out:
 	TALLOC_FREE(name);
 	fsp->fsp_flags.have_proc_fds = false;
 	DBG_DEBUG("[CEPH] open(...) = %d\n", result);
-	WRAP_RETURN(result);
+	return status_code(result);
 }
 
 static int cephwrap_close(struct vfs_handle_struct *handle, files_struct *fsp)
@@ -614,20 +521,27 @@ static int cephwrap_close(struct vfs_handle_struct *handle, files_struct *fsp)
 	DBG_DEBUG("[CEPH] close(%p, %p)\n", handle, fsp);
 	result = ceph_close(handle->data, fsp_get_pathref_fd(fsp));
 	DBG_DEBUG("[CEPH] close(...) = %d\n", result);
-
-	WRAP_RETURN(result);
+	return status_code(result);
 }
 
-static ssize_t cephwrap_pread(struct vfs_handle_struct *handle, files_struct *fsp, void *data,
-			size_t n, off_t offset)
+static ssize_t cephwrap_pread(struct vfs_handle_struct *handle,
+			      files_struct *fsp,
+			      void *data,
+			      size_t n,
+			      off_t offset)
 {
 	ssize_t result;
 
-	DBG_DEBUG("[CEPH] pread(%p, %p, %p, %llu, %llu)\n", handle, fsp, data, llu(n), llu(offset));
+	DBG_DEBUG("[CEPH] pread(%p, %p, %p, %llu, %llu)\n",
+		  handle,
+		  fsp,
+		  data,
+		  llu(n),
+		  llu(offset));
 
 	result = ceph_read(handle->data, fsp_get_io_fd(fsp), data, n, offset);
 	DBG_DEBUG("[CEPH] pread(...) = %llu\n", llu(result));
-	WRAP_RETURN(result);
+	return lstatus_code(result);
 }
 
 struct cephwrap_pread_state {
@@ -682,15 +596,23 @@ static ssize_t cephwrap_pread_recv(struct tevent_req *req,
 	return state->bytes_read;
 }
 
-static ssize_t cephwrap_pwrite(struct vfs_handle_struct *handle, files_struct *fsp, const void *data,
-			size_t n, off_t offset)
+static ssize_t cephwrap_pwrite(struct vfs_handle_struct *handle,
+			       files_struct *fsp,
+			       const void *data,
+			       size_t n,
+			       off_t offset)
 {
 	ssize_t result;
 
-	DBG_DEBUG("[CEPH] pwrite(%p, %p, %p, %llu, %llu)\n", handle, fsp, data, llu(n), llu(offset));
+	DBG_DEBUG("[CEPH] pwrite(%p, %p, %p, %llu, %llu)\n",
+		  handle,
+		  fsp,
+		  data,
+		  llu(n),
+		  llu(offset));
 	result = ceph_write(handle->data, fsp_get_io_fd(fsp), data, n, offset);
 	DBG_DEBUG("[CEPH] pwrite(...) = %llu\n", llu(result));
-	WRAP_RETURN(result);
+	return lstatus_code(result);
 }
 
 struct cephwrap_pwrite_state {
@@ -745,17 +667,24 @@ static ssize_t cephwrap_pwrite_recv(struct tevent_req *req,
 	return state->bytes_written;
 }
 
-static off_t cephwrap_lseek(struct vfs_handle_struct *handle, files_struct *fsp, off_t offset, int whence)
+static off_t cephwrap_lseek(struct vfs_handle_struct *handle,
+			    files_struct *fsp,
+			    off_t offset,
+			    int whence)
 {
 	off_t result = 0;
 
 	DBG_DEBUG("[CEPH] cephwrap_lseek\n");
 	result = ceph_lseek(handle->data, fsp_get_io_fd(fsp), offset, whence);
-	WRAP_RETURN(result);
+	return lstatus_code(result);
 }
 
-static ssize_t cephwrap_sendfile(struct vfs_handle_struct *handle, int tofd, files_struct *fromfsp, const DATA_BLOB *hdr,
-			off_t offset, size_t n)
+static ssize_t cephwrap_sendfile(struct vfs_handle_struct *handle,
+				 int tofd,
+				 files_struct *fromfsp,
+				 const DATA_BLOB *hdr,
+				 off_t offset,
+				 size_t n)
 {
 	/*
 	 * We cannot support sendfile because libcephfs is in user space.
@@ -775,7 +704,7 @@ static ssize_t cephwrap_recvfile(struct vfs_handle_struct *handle,
 	 * We cannot support recvfile because libcephfs is in user space.
 	 */
 	DBG_DEBUG("[CEPH] cephwrap_recvfile\n");
-	errno=ENOTSUP;
+	errno = ENOTSUP;
 	return -1;
 }
 
@@ -818,7 +747,7 @@ static int cephwrap_renameat(struct vfs_handle_struct *handle,
 	TALLOC_FREE(full_fname_src);
 	TALLOC_FREE(full_fname_dst);
 
-	WRAP_RETURN(result);
+	return status_code(result);
 }
 
 /*
@@ -873,7 +802,8 @@ static int cephwrap_fsync_recv(struct tevent_req *req,
 
 #define SAMBA_STATX_ATTR_MASK	(CEPH_STATX_BASIC_STATS|CEPH_STATX_BTIME)
 
-static void init_stat_ex_from_ceph_statx(struct stat_ex *dst, const struct ceph_statx *stx)
+static void init_stat_ex_from_ceph_statx(struct stat_ex *dst,
+					 const struct ceph_statx *stx)
 {
 	DBG_DEBUG("[CEPH]\tstx = {dev = %llx, ino = %llu, mode = 0x%x, "
 		  "nlink = %llu, uid = %d, gid = %d, rdev = %llx, size = %llu, "
@@ -887,8 +817,11 @@ static void init_stat_ex_from_ceph_statx(struct stat_ex *dst, const struct ceph_
 		  llu(stx->stx_btime.tv_sec));
 
 	if ((stx->stx_mask & SAMBA_STATX_ATTR_MASK) != SAMBA_STATX_ATTR_MASK) {
-		DBG_WARNING("%s: stx->stx_mask is incorrect (wanted %x, got %x)\n",
-				__func__, SAMBA_STATX_ATTR_MASK, stx->stx_mask);
+		DBG_WARNING("%s: stx->stx_mask is incorrect "
+			    "(wanted %x, got %x)\n",
+			    __func__,
+			    SAMBA_STATX_ATTR_MASK,
+			    stx->stx_mask);
 	}
 
 	dst->st_ex_dev = stx->stx_dev;
@@ -913,7 +846,9 @@ static int cephwrap_stat(struct vfs_handle_struct *handle,
 	int result = -1;
 	struct ceph_statx stx = { 0 };
 
-	DBG_DEBUG("[CEPH] stat(%p, %s)\n", handle, smb_fname_str_dbg(smb_fname));
+	DBG_DEBUG("[CEPH] stat(%p, %s)\n",
+		  handle,
+		  smb_fname_str_dbg(smb_fname));
 
 	if (smb_fname->stream_name) {
 		errno = ENOENT;
@@ -924,7 +859,7 @@ static int cephwrap_stat(struct vfs_handle_struct *handle,
 				SAMBA_STATX_ATTR_MASK, 0);
 	DBG_DEBUG("[CEPH] statx(...) = %d\n", result);
 	if (result < 0) {
-		WRAP_RETURN(result);
+		return status_code(result);
 	}
 
 	init_stat_ex_from_ceph_statx(&smb_fname->st, &stx);
@@ -932,7 +867,9 @@ static int cephwrap_stat(struct vfs_handle_struct *handle,
 	return result;
 }
 
-static int cephwrap_fstat(struct vfs_handle_struct *handle, files_struct *fsp, SMB_STRUCT_STAT *sbuf)
+static int cephwrap_fstat(struct vfs_handle_struct *handle,
+			  files_struct *fsp,
+			  SMB_STRUCT_STAT *sbuf)
 {
 	int result = -1;
 	struct ceph_statx stx = { 0 };
@@ -943,7 +880,7 @@ static int cephwrap_fstat(struct vfs_handle_struct *handle, files_struct *fsp, S
 				SAMBA_STATX_ATTR_MASK, 0);
 	DBG_DEBUG("[CEPH] fstat(...) = %d\n", result);
 	if (result < 0) {
-		WRAP_RETURN(result);
+		return status_code(result);
 	}
 
 	init_stat_ex_from_ceph_statx(sbuf, &stx);
@@ -959,35 +896,16 @@ static int cephwrap_fstatat(struct vfs_handle_struct *handle,
 {
 	int result = -1;
 	struct ceph_statx stx = { 0 };
-#ifdef HAVE_CEPH_STATXAT
 	int dirfd = fsp_get_pathref_fd(dirfsp);
 
 	DBG_DEBUG("[CEPH] fstatat(%p, %d, %s)\n",
 		  handle, dirfd, smb_fname->base_name);
 	result = ceph_statxat(handle->data, dirfd, smb_fname->base_name,
 			      &stx, SAMBA_STATX_ATTR_MASK, 0);
-#else
-	struct smb_filename *full_fname = NULL;
-
-	full_fname = full_path_from_dirfsp_atname(talloc_tos(),
-						  dirfsp,
-						  smb_fname);
-	if (full_fname == NULL) {
-		errno = ENOMEM;
-		return -1;
-	}
-
-	DBG_DEBUG("[CEPH] fstatat(%p, %s)\n",
-		  handle, smb_fname_str_dbg(full_fname));
-	result = ceph_statx(handle->data, full_fname->base_name,
-			    &stx, SAMBA_STATX_ATTR_MASK, 0);
-
-	TALLOC_FREE(full_fname);
-#endif
 
 	DBG_DEBUG("[CEPH] fstatat(...) = %d\n", result);
 	if (result < 0) {
-		WRAP_RETURN(result);
+		return status_code(result);
 	}
 
 	init_stat_ex_from_ceph_statx(sbuf, &stx);
@@ -1002,7 +920,9 @@ static int cephwrap_lstat(struct vfs_handle_struct *handle,
 	int result = -1;
 	struct ceph_statx stx = { 0 };
 
-	DBG_DEBUG("[CEPH] lstat(%p, %s)\n", handle, smb_fname_str_dbg(smb_fname));
+	DBG_DEBUG("[CEPH] lstat(%p, %s)\n",
+		  handle,
+		  smb_fname_str_dbg(smb_fname));
 
 	if (smb_fname->stream_name) {
 		errno = ENOENT;
@@ -1013,7 +933,7 @@ static int cephwrap_lstat(struct vfs_handle_struct *handle,
 				SAMBA_STATX_ATTR_MASK, AT_SYMLINK_NOFOLLOW);
 	DBG_DEBUG("[CEPH] lstat(...) = %d\n", result);
 	if (result < 0) {
-		WRAP_RETURN(result);
+		return status_code(result);
 	}
 
 	init_stat_ex_from_ceph_statx(&smb_fname->st, &stx);
@@ -1077,7 +997,6 @@ static int cephwrap_unlinkat(struct vfs_handle_struct *handle,
 			int flags)
 {
 	int result = -1;
-#ifdef HAVE_CEPH_UNLINKAT
 	int dirfd = fsp_get_pathref_fd(dirfsp);
 
 	DBG_DEBUG("[CEPH] unlinkat(%p, %d, %s)\n",
@@ -1095,38 +1014,12 @@ static int cephwrap_unlinkat(struct vfs_handle_struct *handle,
 			       smb_fname->base_name,
 			       flags);
 	DBG_DEBUG("[CEPH] unlinkat(...) = %d\n", result);
-	WRAP_RETURN(result);
-#else
-	struct smb_filename *full_fname = NULL;
-
-	DBG_DEBUG("[CEPH] unlink(%p, %s)\n",
-		handle,
-		smb_fname_str_dbg(smb_fname));
-
-	if (smb_fname->stream_name) {
-		errno = ENOENT;
-		return result;
-	}
-
-	full_fname = full_path_from_dirfsp_atname(talloc_tos(),
-						  dirfsp,
-						  smb_fname);
-	if (full_fname == NULL) {
-		return -1;
-	}
-
-	if (flags & AT_REMOVEDIR) {
-		result = ceph_rmdir(handle->data, full_fname->base_name);
-	} else {
-		result = ceph_unlink(handle->data, full_fname->base_name);
-	}
-	TALLOC_FREE(full_fname);
-	DBG_DEBUG("[CEPH] unlink(...) = %d\n", result);
-	WRAP_RETURN(result);
-#endif
+	return status_code(result);
 }
 
-static int cephwrap_fchmod(struct vfs_handle_struct *handle, files_struct *fsp, mode_t mode)
+static int cephwrap_fchmod(struct vfs_handle_struct *handle,
+			   files_struct *fsp,
+			   mode_t mode)
 {
 	int result;
 
@@ -1145,10 +1038,13 @@ static int cephwrap_fchmod(struct vfs_handle_struct *handle, files_struct *fsp,
 				    mode);
 	}
 	DBG_DEBUG("[CEPH] fchmod(...) = %d\n", result);
-	WRAP_RETURN(result);
+	return status_code(result);
 }
 
-static int cephwrap_fchown(struct vfs_handle_struct *handle, files_struct *fsp, uid_t uid, gid_t gid)
+static int cephwrap_fchown(struct vfs_handle_struct *handle,
+			   files_struct *fsp,
+			   uid_t uid,
+			   gid_t gid)
 {
 	int result;
 
@@ -1172,7 +1068,7 @@ static int cephwrap_fchown(struct vfs_handle_struct *handle, files_struct *fsp,
 	}
 
 	DBG_DEBUG("[CEPH] fchown(...) = %d\n", result);
-	WRAP_RETURN(result);
+	return status_code(result);
 }
 
 static int cephwrap_lchown(struct vfs_handle_struct *handle,
@@ -1181,10 +1077,14 @@ static int cephwrap_lchown(struct vfs_handle_struct *handle,
 			gid_t gid)
 {
 	int result;
-	DBG_DEBUG("[CEPH] lchown(%p, %s, %d, %d)\n", handle, smb_fname->base_name, uid, gid);
+	DBG_DEBUG("[CEPH] lchown(%p, %s, %d, %d)\n",
+		  handle,
+		  smb_fname->base_name,
+		  uid,
+		  gid);
 	result = ceph_lchown(handle->data, smb_fname->base_name, uid, gid);
 	DBG_DEBUG("[CEPH] lchown(...) = %d\n", result);
-	WRAP_RETURN(result);
+	return status_code(result);
 }
 
 static int cephwrap_chdir(struct vfs_handle_struct *handle,
@@ -1194,7 +1094,7 @@ static int cephwrap_chdir(struct vfs_handle_struct *handle,
 	DBG_DEBUG("[CEPH] chdir(%p, %s)\n", handle, smb_fname->base_name);
 	result = ceph_chdir(handle->data, smb_fname->base_name);
 	DBG_DEBUG("[CEPH] chdir(...) = %d\n", result);
-	WRAP_RETURN(result);
+	return status_code(result);
 }
 
 static struct smb_filename *cephwrap_getwd(struct vfs_handle_struct *handle,
@@ -1202,15 +1102,12 @@ static struct smb_filename *cephwrap_getwd(struct vfs_handle_struct *handle,
 {
 	const char *cwd = ceph_getcwd(handle->data);
 	DBG_DEBUG("[CEPH] getwd(%p) = %s\n", handle, cwd);
-	return synthetic_smb_fname(ctx,
-				cwd,
-				NULL,
-				NULL,
-				0,
-				0);
+	return synthetic_smb_fname(ctx, cwd, NULL, NULL, 0, 0);
 }
 
-static int strict_allocate_ftruncate(struct vfs_handle_struct *handle, files_struct *fsp, off_t len)
+static int strict_allocate_ftruncate(struct vfs_handle_struct *handle,
+				     files_struct *fsp,
+				     off_t len)
 {
 	off_t space_to_write;
 	int result;
@@ -1234,16 +1131,21 @@ static int strict_allocate_ftruncate(struct vfs_handle_struct *handle, files_str
 	/* Shrink - just ftruncate. */
 	if (pst->st_ex_size > len) {
 		result = ceph_ftruncate(handle->data, fsp_get_io_fd(fsp), len);
-		WRAP_RETURN(result);
+		return status_code(result);
 	}
 
 	space_to_write = len - pst->st_ex_size;
-	result = ceph_fallocate(handle->data, fsp_get_io_fd(fsp), 0, pst->st_ex_size,
+	result = ceph_fallocate(handle->data,
+				fsp_get_io_fd(fsp),
+				0,
+				pst->st_ex_size,
 				space_to_write);
-	WRAP_RETURN(result);
+	return status_code(result);
 }
 
-static int cephwrap_ftruncate(struct vfs_handle_struct *handle, files_struct *fsp, off_t len)
+static int cephwrap_ftruncate(struct vfs_handle_struct *handle,
+			      files_struct *fsp,
+			      off_t len)
 {
 	int result = -1;
 
@@ -1254,7 +1156,7 @@ static int cephwrap_ftruncate(struct vfs_handle_struct *handle, files_struct *fs
 	}
 
 	result = ceph_ftruncate(handle->data, fsp_get_io_fd(fsp), len);
-	WRAP_RETURN(result);
+	return status_code(result);
 }
 
 static int cephwrap_fallocate(struct vfs_handle_struct *handle,
@@ -1268,12 +1170,18 @@ static int cephwrap_fallocate(struct vfs_handle_struct *handle,
 	DBG_DEBUG("[CEPH] fallocate(%p, %p, %u, %llu, %llu\n",
 		  handle, fsp, mode, llu(offset), llu(len));
 	/* unsupported mode flags are rejected by libcephfs */
-	result = ceph_fallocate(handle->data, fsp_get_io_fd(fsp), mode, offset, len);
+	result = ceph_fallocate(
+		handle->data, fsp_get_io_fd(fsp), mode, offset, len);
 	DBG_DEBUG("[CEPH] fallocate(...) = %d\n", result);
-	WRAP_RETURN(result);
+	return status_code(result);
 }
 
-static bool cephwrap_lock(struct vfs_handle_struct *handle, files_struct *fsp, int op, off_t offset, off_t count, int type)
+static bool cephwrap_lock(struct vfs_handle_struct *handle,
+			  files_struct *fsp,
+			  int op,
+			  off_t offset,
+			  off_t count,
+			  int type)
 {
 	DBG_DEBUG("[CEPH] lock\n");
 	return true;
@@ -1287,8 +1195,10 @@ static int cephwrap_filesystem_sharemode(struct vfs_handle_struct *handle,
 	DBG_ERR("[CEPH] filesystem sharemodes unsupported! Consider setting "
 		"\"kernel share modes = no\"\n");
 
-	errno = ENOSYS;
-	return -1;
+	return vfs_not_implemented_filesystem_sharemode(handle,
+							fsp,
+							share_access,
+							access_mask);
 }
 
 static int cephwrap_fcntl(vfs_handle_struct *handle,
@@ -1319,7 +1229,12 @@ err_out:
 	return -1;
 }
 
-static bool cephwrap_getlock(struct vfs_handle_struct *handle, files_struct *fsp, off_t *poffset, off_t *pcount, int *ptype, pid_t *ppid)
+static bool cephwrap_getlock(struct vfs_handle_struct *handle,
+			     files_struct *fsp,
+			     off_t *poffset,
+			     off_t *pcount,
+			     int *ptype,
+			     pid_t *ppid)
 {
 	DBG_DEBUG("[CEPH] getlock returning false and errno=0\n");
 
@@ -1327,28 +1242,12 @@ static bool cephwrap_getlock(struct vfs_handle_struct *handle, files_struct *fsp
 	return false;
 }
 
-/*
- * We cannot let this fall through to the default, because the file might only
- * be accessible from libcephfs (which is a user-space client) but the fd might
- * be for some file the kernel knows about.
- */
-static int cephwrap_linux_setlease(struct vfs_handle_struct *handle, files_struct *fsp,
-				int leasetype)
-{
-	int result = -1;
-
-	DBG_DEBUG("[CEPH] linux_setlease\n");
-	errno = ENOSYS;
-	return result;
-}
-
 static int cephwrap_symlinkat(struct vfs_handle_struct *handle,
 		const struct smb_filename *link_target,
 		struct files_struct *dirfsp,
 		const struct smb_filename *new_smb_fname)
 {
 	int result = -1;
-#ifdef HAVE_CEPH_SYMLINKAT
 	int dirfd = fsp_get_pathref_fd(dirfsp);
 
 	DBG_DEBUG("[CEPH] symlinkat(%p, %s, %d, %s)\n",
@@ -1362,28 +1261,7 @@ static int cephwrap_symlinkat(struct vfs_handle_struct *handle,
 				dirfd,
 				new_smb_fname->base_name);
 	DBG_DEBUG("[CEPH] symlinkat(...) = %d\n", result);
-	WRAP_RETURN(result);
-#else
-	struct smb_filename *full_fname = NULL;
-
-	full_fname = full_path_from_dirfsp_atname(talloc_tos(),
-						dirfsp,
-						new_smb_fname);
-	if (full_fname == NULL) {
-		return -1;
-	}
-
-	DBG_DEBUG("[CEPH] symlink(%p, %s, %s)\n", handle,
-			link_target->base_name,
-			full_fname->base_name);
-
-	result = ceph_symlink(handle->data,
-			link_target->base_name,
-			full_fname->base_name);
-	TALLOC_FREE(full_fname);
-	DBG_DEBUG("[CEPH] symlink(...) = %d\n", result);
-	WRAP_RETURN(result);
-#endif
+	return status_code(result);
 }
 
 static int cephwrap_readlinkat(struct vfs_handle_struct *handle,
@@ -1393,7 +1271,6 @@ static int cephwrap_readlinkat(struct vfs_handle_struct *handle,
 		size_t bufsiz)
 {
 	int result = -1;
-#ifdef HAVE_CEPH_READLINKAT
 	int dirfd = fsp_get_pathref_fd(dirfsp);
 
 	DBG_DEBUG("[CEPH] readlinkat(%p, %d, %s, %p, %llu)\n",
@@ -1410,25 +1287,7 @@ static int cephwrap_readlinkat(struct vfs_handle_struct *handle,
 				 bufsiz);
 
 	DBG_DEBUG("[CEPH] readlinkat(...) = %d\n", result);
-	WRAP_RETURN(result);
-#else
-	struct smb_filename *full_fname = NULL;
-
-	full_fname = full_path_from_dirfsp_atname(talloc_tos(),
-						dirfsp,
-						smb_fname);
-	if (full_fname == NULL) {
-		return -1;
-	}
-
-	DBG_DEBUG("[CEPH] readlink(%p, %s, %p, %llu)\n", handle,
-			full_fname->base_name, buf, llu(bufsiz));
-
-	result = ceph_readlink(handle->data, full_fname->base_name, buf, bufsiz);
-	TALLOC_FREE(full_fname);
-	DBG_DEBUG("[CEPH] readlink(...) = %d\n", result);
-	WRAP_RETURN(result);
-#endif
+	return status_code(result);
 }
 
 static int cephwrap_linkat(struct vfs_handle_struct *handle,
@@ -1466,7 +1325,7 @@ static int cephwrap_linkat(struct vfs_handle_struct *handle,
 	DBG_DEBUG("[CEPH] link(...) = %d\n", result);
 	TALLOC_FREE(full_fname_old);
 	TALLOC_FREE(full_fname_new);
-	WRAP_RETURN(result);
+	return status_code(result);
 }
 
 static int cephwrap_mknodat(struct vfs_handle_struct *handle,
@@ -1491,7 +1350,7 @@ static int cephwrap_mknodat(struct vfs_handle_struct *handle,
 
 	TALLOC_FREE(full_fname);
 
-	WRAP_RETURN(result);
+	return status_code(result);
 }
 
 /*
@@ -1503,50 +1362,33 @@ static struct smb_filename *cephwrap_realpath(struct vfs_handle_struct *handle,
 				const struct smb_filename *smb_fname)
 {
 	char *result = NULL;
+	const char *cwd = handle->conn->cwd_fsp->fsp_name->base_name;
 	const char *path = smb_fname->base_name;
 	size_t len = strlen(path);
 	struct smb_filename *result_fname = NULL;
-	int r = -1;
 
-	if (len && (path[0] == '/')) {
-		r = asprintf(&result, "%s", path);
+	if (path[0] == '/') {
+		result = talloc_strdup(ctx, path);
 	} else if ((len >= 2) && (path[0] == '.') && (path[1] == '/')) {
 		if (len == 2) {
-			r = asprintf(&result, "%s",
-					handle->conn->cwd_fsp->fsp_name->base_name);
+			result = talloc_strdup(ctx, cwd);
 		} else {
-			r = asprintf(&result, "%s/%s",
-					handle->conn->cwd_fsp->fsp_name->base_name, &path[2]);
+			result = talloc_asprintf(ctx, "%s/%s", cwd, &path[2]);
 		}
 	} else {
-		r = asprintf(&result, "%s/%s",
-				handle->conn->cwd_fsp->fsp_name->base_name, path);
+		result = talloc_asprintf(ctx, "%s/%s", cwd, path);
 	}
 
-	if (r < 0) {
+	if (result == NULL) {
 		return NULL;
 	}
 
 	DBG_DEBUG("[CEPH] realpath(%p, %s) = %s\n", handle, path, result);
-	result_fname = synthetic_smb_fname(ctx,
-				result,
-				NULL,
-				NULL,
-				0,
-				0);
-	SAFE_FREE(result);
+	result_fname = synthetic_smb_fname(ctx, result, NULL, NULL, 0, 0);
+	TALLOC_FREE(result);
 	return result_fname;
 }
 
-
-static int cephwrap_fchflags(struct vfs_handle_struct *handle,
-			struct files_struct *fsp,
-			unsigned int flags)
-{
-	errno = ENOSYS;
-	return -1;
-}
-
 static NTSTATUS cephwrap_get_real_filename_at(
 	struct vfs_handle_struct *handle,
 	struct files_struct *dirfsp,
@@ -1569,6 +1411,46 @@ static const char *cephwrap_connectpath(
 	return handle->conn->connectpath;
 }
 
+static NTSTATUS cephwrap_fget_dos_attributes(struct vfs_handle_struct *handle,
+					     struct files_struct *fsp,
+					     uint32_t *dosmode)
+{
+	struct timespec saved_btime = fsp->fsp_name->st.st_ex_btime;
+	NTSTATUS status;
+
+	status = fget_ea_dos_attribute(fsp, dosmode);
+
+	/*
+	 * Restore previously stored btime from statx timestamps as it should be
+	 * the only source of truth. create_time from dos attribute, if any, may
+	 * have older values which isn't trustworthy to be looked at for other
+	 * open file handle operations.
+	 */
+	fsp->fsp_name->st.st_ex_btime = saved_btime;
+
+	return status;
+}
+
+static NTSTATUS cephwrap_fset_dos_attributes(struct vfs_handle_struct *handle,
+					     struct files_struct *fsp,
+					     uint32_t dosmode)
+{
+	struct timespec saved_btime = fsp->fsp_name->st.st_ex_btime;
+	NTSTATUS status;
+
+	status = set_ea_dos_attribute(handle->conn, fsp->fsp_name, dosmode);
+
+	/*
+	 * Restore previously stored btime from statx timestamps. This is done
+	 * to ensure that we have the exact btime in fsp stat information while
+	 * the file handle is still open since the create_time stored as part of
+	 * dos attributes can loose its precision when converted back to btime.
+	 */
+	fsp->fsp_name->st.st_ex_btime = saved_btime;
+
+	return status;
+}
+
 /****************************************************************
  Extended attribute operations.
 *****************************************************************/
@@ -1600,13 +1482,13 @@ static ssize_t cephwrap_fgetxattr(struct vfs_handle_struct *handle,
 				    size);
 	}
 	DBG_DEBUG("[CEPH] fgetxattr(...) = %d\n", ret);
-	if (ret < 0) {
-		WRAP_RETURN(ret);
-	}
-	return (ssize_t)ret;
+	return lstatus_code(ret);
 }
 
-static ssize_t cephwrap_flistxattr(struct vfs_handle_struct *handle, struct files_struct *fsp, char *list, size_t size)
+static ssize_t cephwrap_flistxattr(struct vfs_handle_struct *handle,
+				   struct files_struct *fsp,
+				   char *list,
+				   size_t size)
 {
 	int ret;
 	DBG_DEBUG("[CEPH] flistxattr(%p, %p, %p, %llu)\n",
@@ -1629,13 +1511,12 @@ static ssize_t cephwrap_flistxattr(struct vfs_handle_struct *handle, struct file
 					size);
 	}
 	DBG_DEBUG("[CEPH] flistxattr(...) = %d\n", ret);
-	if (ret < 0) {
-		WRAP_RETURN(ret);
-	}
-	return (ssize_t)ret;
+	return lstatus_code(ret);
 }
 
-static int cephwrap_fremovexattr(struct vfs_handle_struct *handle, struct files_struct *fsp, const char *name)
+static int cephwrap_fremovexattr(struct vfs_handle_struct *handle,
+				 struct files_struct *fsp,
+				 const char *name)
 {
 	int ret;
 	DBG_DEBUG("[CEPH] fremovexattr(%p, %p, %s)\n", handle, fsp, name);
@@ -1653,13 +1534,24 @@ static int cephwrap_fremovexattr(struct vfs_handle_struct *handle, struct files_
 					name);
 	}
 	DBG_DEBUG("[CEPH] fremovexattr(...) = %d\n", ret);
-	WRAP_RETURN(ret);
+	return status_code(ret);
 }
 
-static int cephwrap_fsetxattr(struct vfs_handle_struct *handle, struct files_struct *fsp, const char *name, const void *value, size_t size, int flags)
+static int cephwrap_fsetxattr(struct vfs_handle_struct *handle,
+			      struct files_struct *fsp,
+			      const char *name,
+			      const void *value,
+			      size_t size,
+			      int flags)
 {
 	int ret;
-	DBG_DEBUG("[CEPH] fsetxattr(%p, %p, %s, %p, %llu, %d)\n", handle, fsp, name, value, llu(size), flags);
+	DBG_DEBUG("[CEPH] fsetxattr(%p, %p, %s, %p, %llu, %d)\n",
+		  handle,
+		  fsp,
+		  name,
+		  value,
+		  llu(size),
+		  flags);
 	if (!fsp->fsp_flags.is_pathref) {
 		/*
 		 * We can use an io_fd to set xattrs.
@@ -1682,19 +1574,7 @@ static int cephwrap_fsetxattr(struct vfs_handle_struct *handle, struct files_str
 				flags);
 	}
 	DBG_DEBUG("[CEPH] fsetxattr(...) = %d\n", ret);
-	WRAP_RETURN(ret);
-}
-
-static bool cephwrap_aio_force(struct vfs_handle_struct *handle, struct files_struct *fsp)
-{
-
-	/*
-	 * We do not support AIO yet.
-	 */
-
-	DBG_DEBUG("[CEPH] cephwrap_aio_force(%p, %p) = false (errno = ENOTSUP)\n", handle, fsp);
-	errno = ENOTSUP;
-	return false;
+	return status_code(ret);
 }
 
 static NTSTATUS cephwrap_create_dfs_pathat(struct vfs_handle_struct *handle,
@@ -1876,8 +1756,8 @@ static struct vfs_fn_pointers ceph_fns = {
 	.connect_fn = cephwrap_connect,
 	.disconnect_fn = cephwrap_disconnect,
 	.disk_free_fn = cephwrap_disk_free,
-	.get_quota_fn = cephwrap_get_quota,
-	.set_quota_fn = cephwrap_set_quota,
+	.get_quota_fn = vfs_not_implemented_get_quota,
+	.set_quota_fn = vfs_not_implemented_set_quota,
 	.statvfs_fn = cephwrap_statvfs,
 	.fs_capabilities_fn = cephwrap_fs_capabilities,
 
@@ -1923,16 +1803,18 @@ static struct vfs_fn_pointers ceph_fns = {
 	.lock_fn = cephwrap_lock,
 	.filesystem_sharemode_fn = cephwrap_filesystem_sharemode,
 	.fcntl_fn = cephwrap_fcntl,
-	.linux_setlease_fn = cephwrap_linux_setlease,
+	.linux_setlease_fn = vfs_not_implemented_linux_setlease,
 	.getlock_fn = cephwrap_getlock,
 	.symlinkat_fn = cephwrap_symlinkat,
 	.readlinkat_fn = cephwrap_readlinkat,
 	.linkat_fn = cephwrap_linkat,
 	.mknodat_fn = cephwrap_mknodat,
 	.realpath_fn = cephwrap_realpath,
-	.fchflags_fn = cephwrap_fchflags,
+	.fchflags_fn = vfs_not_implemented_fchflags,
 	.get_real_filename_at_fn = cephwrap_get_real_filename_at,
 	.connectpath_fn = cephwrap_connectpath,
+	.fget_dos_attributes_fn = cephwrap_fget_dos_attributes,
+	.fset_dos_attributes_fn = cephwrap_fset_dos_attributes,
 
 	/* EA operations. */
 	.getxattrat_send_fn = vfs_not_implemented_getxattrat_send,
@@ -1949,7 +1831,7 @@ static struct vfs_fn_pointers ceph_fns = {
 	.sys_acl_delete_def_fd_fn = posixacl_xattr_acl_delete_def_fd,
 
 	/* aio operations */
-	.aio_force_fn = cephwrap_aio_force,
+	.aio_force_fn = vfs_not_implemented_aio_force,
 };
 
 static_decl_vfs;
