@@ -1560,7 +1560,7 @@ static int shadow_copy2_openat(vfs_handle_struct *handle,
 	int ret;
 	bool ok;
 
-	if (how.resolve != 0) {
+	if ((how.resolve & ~VFS_OPEN_HOW_WITH_BACKUP_INTENT) != 0) {
 		errno = ENOSYS;
 		return -1;
 	}
@@ -1974,7 +1974,7 @@ static const char *shadow_copy2_find_snapdir(TALLOC_CTX *mem_ctx,
 	config = priv->config;
 
 	/*
-	 * If the non-snapdisrseverywhere mode, we should not search!
+	 * If the non-snapdirseverywhere mode, we should not search!
 	 */
 	if (!config->snapdirseverywhere) {
 		return config->snapshot_basepath;
