@@ -138,11 +138,13 @@ bool secrets_store_domain_sid(const char *domain, const struct dom_sid  *sid)
 			    &clean_sid,
 			    sizeof(struct dom_sid));
 
-	/* Force a re-query, in the case where we modified our domain */
+	/* Force a re-query */
 	if (ret) {
-		if (dom_sid_equal(get_global_sam_sid(), sid) == false) {
-			reset_global_sam_sid();
-		}
+		/*
+		 * Do not call get_global_domain_sid() here, or we will call it
+		 * recursively.
+		 */
+		reset_global_sam_sid();
 	}
 	return ret;
 }
@@ -1671,7 +1673,8 @@ NTSTATUS secrets_prepare_password_change(const char *domain, const char *dcname,
 					 const char *cleartext_unix,
 					 TALLOC_CTX *mem_ctx,
 					 struct secrets_domain_info1 **pinfo,
-					 struct secrets_domain_info1_change **pprev)
+					 struct secrets_domain_info1_change **pprev,
+					 NTSTATUS (*sync_pw2keytabs_fn)(void))
 {
 	TALLOC_CTX *frame = talloc_stackframe();
 	struct db_context *db = NULL;
@@ -1766,6 +1769,16 @@ NTSTATUS secrets_prepare_password_change(const char *domain, const char *dcname,
 		return NT_STATUS_INTERNAL_DB_ERROR;
 	}
 
+	if (prev == NULL && sync_pw2keytabs_fn != NULL) {
+		status = sync_pw2keytabs_fn();
+		if (!NT_STATUS_IS_OK(status)) {
+			DBG_ERR("Sync of machine password failed.\n");
+			dbwrap_transaction_cancel(db);
+			TALLOC_FREE(frame);
+			return status;
+		}
+	}
+
 	*pinfo = talloc_move(mem_ctx, &info);
 	if (prev != NULL) {
 		*pprev = talloc_move(mem_ctx, &prev);
@@ -2009,7 +2022,8 @@ NTSTATUS secrets_defer_password_change(const char *change_server,
 
 NTSTATUS secrets_finish_password_change(const char *change_server,
 					NTTIME change_time,
-					const struct secrets_domain_info1 *cookie)
+					const struct secrets_domain_info1 *cookie,
+					NTSTATUS (*sync_pw2keytabs_fn)(void))
 {
 	const char *domain = cookie->domain_info.name.string;
 	TALLOC_CTX *frame = talloc_stackframe();
@@ -2065,6 +2079,20 @@ NTSTATUS secrets_finish_password_change(const char *change_server,
 		return status;
 	}
 
+	/*
+	 * For the clustered samba, it is important to have following order:
+	 * 1. dbwrap_transaction_commit()
+	 * 2. sync_pw2keytabs()
+	 * Only this order ensures a correct behavior of
+	 * the 'sync machine password script' that does:
+	 * 'onnode all net ads keytab create'
+	 *
+	 * If we would call sync_pw2keytabs() before committing the changes to
+	 * the secrets.tdb, it will not be updated on other nodes, so triggering
+	 * 'net ads keytab create' will not see the new password yet.
+	 *
+	 * This applies also to secrets_prepare_password_change().
+	 */
 	ret = dbwrap_transaction_commit(db);
 	if (ret != 0) {
 		DBG_ERR("dbwrap_transaction_commit() failed for %s\n",
@@ -2073,6 +2101,15 @@ NTSTATUS secrets_finish_password_change(const char *change_server,
 		return NT_STATUS_INTERNAL_DB_ERROR;
 	}
 
+	if (sync_pw2keytabs_fn != NULL) {
+		status = sync_pw2keytabs_fn();
+		if (!NT_STATUS_IS_OK(status)) {
+			DBG_ERR("Sync of machine password failed.\n");
+			TALLOC_FREE(frame);
+			return status;
+		}
+	}
+
 	TALLOC_FREE(frame);
 	return NT_STATUS_OK;
 }
