@@ -25,18 +25,11 @@
 
 #include "rpc_server.h"
 
-struct pf_listen_fd;
-
-NTSTATUS dcesrv_create_ncacn_ip_tcp_sockets(struct dcesrv_endpoint *e,
-					    struct pf_listen_fd *listen_fd,
-					    int *listen_fd_size);
-
-NTSTATUS dcesrv_setup_ncacn_ip_tcp_sockets(struct tevent_context *ev_ctx,
-					   struct messaging_context *msg_ctx,
-					   struct dcesrv_context *dce_ctx,
-					   struct dcesrv_endpoint *e,
-					   dcerpc_ncacn_termination_fn t_fn,
-					   void *t_data);
+NTSTATUS dcesrv_create_binding_sockets(
+	struct dcerpc_binding *b,
+	TALLOC_CTX *mem_ctx,
+	size_t *pnum_fds,
+	int **fds);
 
 #endif /* _RPC_SOCK_HELPER_H_ */
 
