@@ -75,11 +75,7 @@ NTSTATUS vfs_default_durable_cookie(struct files_struct *fsp,
 		return NT_STATUS_NOT_SUPPORTED;
 	}
 
-	if (fsp_get_io_fd(fsp) == -1) {
-		return NT_STATUS_NOT_SUPPORTED;
-	}
-
-	if (is_ntfs_stream_smb_fname(fsp->fsp_name)) {
+	if (fsp_is_alternate_stream(fsp)) {
 		/*
 		 * We do not support durable handles
 		 * on streams for now.
@@ -177,6 +173,12 @@ NTSTATUS vfs_default_durable_disconnect(struct files_struct *fsp,
 		return NT_STATUS_NOT_SUPPORTED;
 	}
 
+	if (fsp->current_lock_count != 0 &&
+	    (fsp_lease_type(fsp) & SMB2_LEASE_WRITE) == 0)
+	{
+		return NT_STATUS_NOT_SUPPORTED;
+	}
+
 	/*
 	 * For now let it be simple and do not keep
 	 * delete on close files durable open
@@ -490,33 +492,47 @@ static bool vfs_default_durable_reconnect_check_stat(
 	}
 
 	if (cookie_st->st_ex_flags != fsp_st->st_ex_flags) {
-		DEBUG(1, ("vfs_default_durable_reconnect (%s): "
-			  "stat_ex.%s differs: "
-			  "cookie:%llu != stat:%llu, "
-			  "denying durable reconnect\n",
-			  name,
-			  "st_ex_flags",
-			  (unsigned long long)cookie_st->st_ex_flags,
-			  (unsigned long long)fsp_st->st_ex_flags));
+		DBG_WARNING(" (%s): "
+			    "stat_ex.%s differs: "
+			    "cookie:%"PRIu32" != stat:%"PRIu32", "
+			    "denying durable reconnect\n",
+			    name,
+			    "st_ex_flags",
+			    cookie_st->st_ex_flags,
+			    fsp_st->st_ex_flags);
 		return false;
 	}
 
 	return true;
 }
 
+struct durable_reconnect_state {
+	struct smbXsrv_open *op;
+	struct share_mode_entry *e;
+};
+
 static bool durable_reconnect_fn(
 	struct share_mode_entry *e,
 	bool *modified,
 	void *private_data)
 {
-	struct share_mode_entry *dst_e = private_data;
+	struct durable_reconnect_state *state = private_data;
+	uint64_t id = state->op->global->open_persistent_id;
 
-	if (dst_e->pid.pid != 0) {
+	if (e->share_file_id != id) {
+		return false; /* Look at potential other entries */
+	}
+
+	if (!server_id_is_disconnected(&e->pid)) {
+		return false; /* Look at potential other entries */
+	}
+
+	if (state->e->share_file_id == id) {
 		DBG_INFO("Found more than one entry, invalidating previous\n");
-		dst_e->pid.pid = 0;
+		*state->e = (struct share_mode_entry) { .pid = { .pid = 0, }};
 		return true;	/* end the loop through share mode entries */
 	}
-	*dst_e = *e;
+	*state->e = *e;
 	return false;		/* Look at potential other entries */
 }
 
@@ -531,17 +547,19 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 	const struct loadparm_substitution *lp_sub =
 		loadparm_s3_global_substitution();
 	struct share_mode_lock *lck;
-	struct share_mode_entry e;
+	struct share_mode_entry e = { .pid = { .pid = 0, }};
+	struct durable_reconnect_state rstate = { .op = op, .e = &e, };
 	struct files_struct *fsp = NULL;
 	NTSTATUS status;
 	bool ok;
 	int ret;
-	int flags = 0;
+	struct vfs_open_how how = { .flags = 0, };
 	struct file_id file_id;
 	struct smb_filename *smb_fname = NULL;
 	enum ndr_err_code ndr_err;
 	struct vfs_default_durable_cookie cookie;
 	DATA_BLOB new_cookie_blob = data_blob_null;
+	bool have_share_mode_entry = false;
 
 	*result = NULL;
 	*new_cookie = data_blob_null;
@@ -623,27 +641,25 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 		return NT_STATUS_OBJECT_NAME_NOT_FOUND;
 	}
 
-	e = (struct share_mode_entry) { .pid.pid = 0 };
-
-	ok = share_mode_forall_entries(lck, durable_reconnect_fn, &e);
+	ok = share_mode_forall_entries(lck, durable_reconnect_fn, &rstate);
 	if (!ok) {
 		DBG_WARNING("share_mode_forall_entries failed\n");
-		TALLOC_FREE(lck);
-		return NT_STATUS_INTERNAL_DB_ERROR;
+		status = NT_STATUS_INTERNAL_DB_ERROR;
+		goto fail;
 	}
 
 	if (e.pid.pid == 0) {
 		DBG_WARNING("Did not find a unique valid share mode entry\n");
-		TALLOC_FREE(lck);
-		return NT_STATUS_OBJECT_NAME_NOT_FOUND;
+		status = NT_STATUS_OBJECT_NAME_NOT_FOUND;
+		goto fail;
 	}
 
 	if (!server_id_is_disconnected(&e.pid)) {
 		DEBUG(5, ("vfs_default_durable_reconnect: denying durable "
 			  "reconnect for handle that was not marked "
 			  "disconnected (e.g. smbd or cluster node died)\n"));
-		TALLOC_FREE(lck);
-		return NT_STATUS_OBJECT_NAME_NOT_FOUND;
+		status = NT_STATUS_OBJECT_NAME_NOT_FOUND;
+		goto fail;
 	}
 
 	if (e.share_file_id != op->global->open_persistent_id) {
@@ -652,8 +668,8 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 			 "(e.g. another client had opened the file)\n",
 			 e.share_file_id,
 			 op->global->open_persistent_id);
-		TALLOC_FREE(lck);
-		return NT_STATUS_OBJECT_NAME_NOT_FOUND;
+		status = NT_STATUS_OBJECT_NAME_NOT_FOUND;
+		goto fail;
 	}
 
 	if ((e.access_mask & (FILE_WRITE_DATA|FILE_APPEND_DATA)) &&
@@ -662,8 +678,8 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 		DEBUG(5, ("vfs_default_durable_reconnect: denying durable "
 			  "share[%s] is not writeable anymore\n",
 			  lp_servicename(talloc_tos(), lp_sub, SNUM(conn))));
-		TALLOC_FREE(lck);
-		return NT_STATUS_OBJECT_NAME_NOT_FOUND;
+		status = NT_STATUS_OBJECT_NAME_NOT_FOUND;
+		goto fail;
 	}
 
 	/*
@@ -674,8 +690,7 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 	if (!NT_STATUS_IS_OK(status)) {
 		DEBUG(0, ("vfs_default_durable_reconnect: failed to create "
 			  "new fsp: %s\n", nt_errstr(status)));
-		TALLOC_FREE(lck);
-		return status;
+		goto fail;
 	}
 
 	fh_set_private_options(fsp->fh, e.private_options);
@@ -718,9 +733,8 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 		 */
 		if (!GUID_equal(fsp_client_guid(fsp),
 				&e.client_guid)) {
-			TALLOC_FREE(lck);
-			file_free(smb1req, fsp);
-			return NT_STATUS_OBJECT_NAME_NOT_FOUND;
+			status = NT_STATUS_OBJECT_NAME_NOT_FOUND;
+			goto fail;
 		}
 
 		status = leases_db_get(
@@ -734,9 +748,7 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 			&lease_version, /* lease_version */
 			&epoch); /* epoch */
 		if (!NT_STATUS_IS_OK(status)) {
-			TALLOC_FREE(lck);
-			file_free(smb1req, fsp);
-			return status;
+			goto fail;
 		}
 
 		fsp->lease = find_fsp_lease(
@@ -746,9 +758,8 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 			lease_version,
 			epoch);
 		if (fsp->lease == NULL) {
-			TALLOC_FREE(lck);
-			file_free(smb1req, fsp);
-			return NT_STATUS_NO_MEMORY;
+			status = NT_STATUS_NO_MEMORY;
+			goto fail;
 		}
 	}
 
@@ -764,12 +775,10 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 
 	status = fsp_set_smb_fname(fsp, smb_fname);
 	if (!NT_STATUS_IS_OK(status)) {
-		TALLOC_FREE(lck);
-		file_free(smb1req, fsp);
 		DEBUG(0, ("vfs_default_durable_reconnect: "
 			  "fsp_set_smb_fname failed: %s\n",
 			  nt_errstr(status)));
-		return status;
+		goto fail;
 	}
 
 	op->compat = fsp;
@@ -784,12 +793,10 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 		fh_get_gen_id(fsp->fh));
 	if (!ok) {
 		DBG_DEBUG("Could not set new share_mode_entry values\n");
-		TALLOC_FREE(lck);
-		op->compat = NULL;
-		fsp->op = NULL;
-		file_free(smb1req, fsp);
-		return NT_STATUS_INTERNAL_ERROR;
+		status = NT_STATUS_INTERNAL_ERROR;
+		goto fail;
 	}
+	have_share_mode_entry = true;
 
 	ok = brl_reconnect_disconnected(fsp);
 	if (!ok) {
@@ -797,33 +804,25 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 		DEBUG(1, ("vfs_default_durable_reconnect: "
 			  "failed to reopen brlocks: %s\n",
 			  nt_errstr(status)));
-		TALLOC_FREE(lck);
-		op->compat = NULL;
-		fsp->op = NULL;
-		file_free(smb1req, fsp);
-		return status;
+		goto fail;
 	}
 
 	/*
 	 * TODO: properly calculate open flags
 	 */
 	if (fsp->fsp_flags.can_write && fsp->fsp_flags.can_read) {
-		flags = O_RDWR;
+		how.flags = O_RDWR;
 	} else if (fsp->fsp_flags.can_write) {
-		flags = O_WRONLY;
+		how.flags = O_WRONLY;
 	} else if (fsp->fsp_flags.can_read) {
-		flags = O_RDONLY;
+		how.flags = O_RDONLY;
 	}
 
-	status = fd_openat(conn->cwd_fsp, fsp->fsp_name, fsp, flags, 0);
+	status = fd_openat(conn->cwd_fsp, fsp->fsp_name, fsp, &how);
 	if (!NT_STATUS_IS_OK(status)) {
-		TALLOC_FREE(lck);
 		DEBUG(1, ("vfs_default_durable_reconnect: failed to open "
 			  "file: %s\n", nt_errstr(status)));
-		op->compat = NULL;
-		fsp->op = NULL;
-		file_free(smb1req, fsp);
-		return status;
+		goto fail;
 	}
 
 	/*
@@ -837,48 +836,22 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 
 	ret = SMB_VFS_FSTAT(fsp, &fsp->fsp_name->st);
 	if (ret == -1) {
-		NTSTATUS close_status;
 		status = map_nt_error_from_unix_common(errno);
 		DEBUG(1, ("Unable to fstat stream: %s => %s\n",
 			  smb_fname_str_dbg(smb_fname),
 			  nt_errstr(status)));
-		close_status = fd_close(fsp);
-		if (!NT_STATUS_IS_OK(close_status)) {
-			DBG_ERR("fd_close failed (%s) - leaking file "
-				"descriptor\n", nt_errstr(close_status));
-		}
-		TALLOC_FREE(lck);
-		op->compat = NULL;
-		fsp->op = NULL;
-		file_free(smb1req, fsp);
-		return status;
+		goto fail;
 	}
 
 	if (!S_ISREG(fsp->fsp_name->st.st_ex_mode)) {
-		NTSTATUS close_status = fd_close(fsp);
-		if (!NT_STATUS_IS_OK(close_status)) {
-			DBG_ERR("fd_close failed (%s) - leaking file "
-				"descriptor\n", nt_errstr(close_status));
-		}
-		TALLOC_FREE(lck);
-		op->compat = NULL;
-		fsp->op = NULL;
-		file_free(smb1req, fsp);
-		return NT_STATUS_OBJECT_NAME_NOT_FOUND;
+		status = NT_STATUS_OBJECT_NAME_NOT_FOUND;
+		goto fail;
 	}
 
 	file_id = vfs_file_id_from_sbuf(conn, &fsp->fsp_name->st);
 	if (!file_id_equal(&cookie.id, &file_id)) {
-		NTSTATUS close_status = fd_close(fsp);
-		if (!NT_STATUS_IS_OK(close_status)) {
-			DBG_ERR("fd_close failed (%s) - leaking file "
-				"descriptor\n", nt_errstr(close_status));
-		}
-		TALLOC_FREE(lck);
-		op->compat = NULL;
-		fsp->op = NULL;
-		file_free(smb1req, fsp);
-		return NT_STATUS_OBJECT_NAME_NOT_FOUND;
+		status = NT_STATUS_OBJECT_NAME_NOT_FOUND;
+		goto fail;
 	}
 
 	(void)fdos_mode(fsp);
@@ -887,42 +860,21 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 						      &fsp->fsp_name->st,
 						      fsp_str_dbg(fsp));
 	if (!ok) {
-		NTSTATUS close_status = fd_close(fsp);
-		if (!NT_STATUS_IS_OK(close_status)) {
-			DBG_ERR("fd_close failed (%s) - leaking file "
-				"descriptor\n", nt_errstr(close_status));
-		}
-		TALLOC_FREE(lck);
-		op->compat = NULL;
-		fsp->op = NULL;
-		file_free(smb1req, fsp);
-		return NT_STATUS_OBJECT_NAME_NOT_FOUND;
+		status = NT_STATUS_OBJECT_NAME_NOT_FOUND;
+		goto fail;
 	}
 
 	status = set_file_oplock(fsp);
 	if (!NT_STATUS_IS_OK(status)) {
-		NTSTATUS close_status = fd_close(fsp);
-		if (!NT_STATUS_IS_OK(close_status)) {
-			DBG_ERR("fd_close failed (%s) - leaking file "
-				"descriptor\n", nt_errstr(close_status));
-		}
-		TALLOC_FREE(lck);
-		op->compat = NULL;
-		fsp->op = NULL;
-		file_free(smb1req, fsp);
-		return status;
+		goto fail;
 	}
 
 	status = vfs_default_durable_cookie(fsp, mem_ctx, &new_cookie_blob);
 	if (!NT_STATUS_IS_OK(status)) {
-		TALLOC_FREE(lck);
 		DEBUG(1, ("vfs_default_durable_reconnect: "
 			  "vfs_default_durable_cookie - %s\n",
 			  nt_errstr(status)));
-		op->compat = NULL;
-		fsp->op = NULL;
-		file_free(smb1req, fsp);
-		return status;
+		goto fail;
 	}
 
 	smb1req->chain_fsp = fsp;
@@ -939,4 +891,27 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 	*new_cookie = new_cookie_blob;
 
 	return NT_STATUS_OK;
+
+fail:
+	if (fsp != NULL && have_share_mode_entry) {
+		/*
+		 * Something is screwed up, delete the sharemode entry.
+		 */
+		del_share_mode(lck, fsp);
+	}
+	if (fsp != NULL && fsp_get_pathref_fd(fsp) != -1) {
+		NTSTATUS close_status;
+		close_status = fd_close(fsp);
+		if (!NT_STATUS_IS_OK(close_status)) {
+			DBG_ERR("fd_close failed (%s), leaking fd\n",
+				nt_errstr(close_status));
+		}
+	}
+	TALLOC_FREE(lck);
+	if (fsp != NULL) {
+		op->compat = NULL;
+		fsp->op = NULL;
+		file_free(smb1req, fsp);
+	}
+	return status;
 }
