@@ -51,6 +51,7 @@
 #include "lib/util/string_wrappers.h"
 #include "source3/printing/rap_jobid.h"
 #include "source3/lib/substitute.h"
+#include "source3/smbd/dir.h"
 
 /****************************************************************************
  Ensure we check the path in *exactly* the same way as W2K for a findfirst/findnext
@@ -964,7 +965,8 @@ NTSTATUS unlink_internals(connection_struct *conn,
 		 DELETE_ACCESS,		/* access_mask */
 		 FILE_SHARE_NONE,	/* share_access */
 		 FILE_OPEN,		/* create_disposition*/
-		 FILE_NON_DIRECTORY_FILE, /* create_options */
+		 FILE_NON_DIRECTORY_FILE |
+			FILE_OPEN_REPARSE_POINT, /* create_options */
 		 FILE_ATTRIBUTE_NORMAL,	/* file_attributes */
 		 0,			/* oplock_request */
 		 NULL,			/* lease */
@@ -1393,7 +1395,6 @@ static NTSTATUS parent_dirname_compatible_open(connection_struct *conn,
 
 NTSTATUS rename_internals_fsp(connection_struct *conn,
 			files_struct *fsp,
-			struct files_struct *dst_dirfsp,
 			struct smb_filename *smb_fname_dst_in,
 			const char *dst_original_lcomp,
 			uint32_t attrs,
@@ -1546,9 +1547,9 @@ NTSTATUS rename_internals_fsp(connection_struct *conn,
 	if (strcsequal(fsp->fsp_name->base_name, smb_fname_dst->base_name) &&
 	    strcsequal(fsp->fsp_name->stream_name,
 		       smb_fname_dst->stream_name)) {
-		DEBUG(3, ("rename_internals_fsp: identical names in rename %s "
-			  "- returning success\n",
-			  smb_fname_str_dbg(smb_fname_dst)));
+		DBG_NOTICE("identical names in rename %s "
+			   "- returning success\n",
+			   smb_fname_str_dbg(smb_fname_dst));
 		status = NT_STATUS_OK;
 		goto out;
 	}
@@ -1570,9 +1571,10 @@ NTSTATUS rename_internals_fsp(connection_struct *conn,
 	dst_exists = vfs_stat(conn, smb_fname_dst) == 0;
 
 	if(!replace_if_exists && dst_exists) {
-		DEBUG(3, ("rename_internals_fsp: dest exists doing rename "
-			  "%s -> %s\n", smb_fname_str_dbg(fsp->fsp_name),
-			  smb_fname_str_dbg(smb_fname_dst)));
+		DBG_NOTICE("dest exists doing rename "
+			   "%s -> %s\n",
+			   smb_fname_str_dbg(fsp->fsp_name),
+			   smb_fname_str_dbg(smb_fname_dst));
 		status = NT_STATUS_OBJECT_NAME_COLLISION;
 		goto out;
 	}
@@ -1594,7 +1596,7 @@ NTSTATUS rename_internals_fsp(connection_struct *conn,
 							   fileid, true);
 		/* The file can be open when renaming a stream */
 		if (dst_fsp && !new_is_stream) {
-			DEBUG(3, ("rename_internals_fsp: Target file open\n"));
+			DBG_NOTICE("Target file open\n");
 			status = NT_STATUS_ACCESS_DENIED;
 			goto out;
 		}
@@ -1609,9 +1611,10 @@ NTSTATUS rename_internals_fsp(connection_struct *conn,
 	status = can_rename(conn, fsp, attrs);
 
 	if (!NT_STATUS_IS_OK(status)) {
-		DEBUG(3, ("rename_internals_fsp: Error %s rename %s -> %s\n",
-			  nt_errstr(status), smb_fname_str_dbg(fsp->fsp_name),
-			  smb_fname_str_dbg(smb_fname_dst)));
+		DBG_NOTICE("Error %s rename %s -> %s\n",
+			   nt_errstr(status),
+			   smb_fname_str_dbg(fsp->fsp_name),
+			   smb_fname_str_dbg(smb_fname_dst));
 		if (NT_STATUS_EQUAL(status,NT_STATUS_SHARING_VIOLATION))
 			status = NT_STATUS_ACCESS_DENIED;
 		goto out;
@@ -1756,9 +1759,10 @@ NTSTATUS rename_internals_fsp(connection_struct *conn,
 	if (ret == 0) {
 		uint32_t create_options = fh_get_private_options(fsp->fh);
 
-		DEBUG(3, ("rename_internals_fsp: succeeded doing rename on "
-			  "%s -> %s\n", smb_fname_str_dbg(fsp->fsp_name),
-			  smb_fname_str_dbg(smb_fname_dst)));
+		DBG_NOTICE("succeeded doing rename on "
+			   "%s -> %s\n",
+			   smb_fname_str_dbg(fsp->fsp_name),
+			   smb_fname_str_dbg(smb_fname_dst));
 
 		notify_rename(conn,
 			      fsp->fsp_flags.is_directory,
@@ -1769,7 +1773,6 @@ NTSTATUS rename_internals_fsp(connection_struct *conn,
 				  smb_fname_dst);
 
 		if (!fsp->fsp_flags.is_directory &&
-		    !(fsp->posix_flags & FSP_POSIX_FLAGS_PATHNAMES) &&
 		    (lp_map_archive(SNUM(conn)) ||
 		     lp_store_dos_attributes(SNUM(conn))))
 		{
@@ -1825,9 +1828,10 @@ NTSTATUS rename_internals_fsp(connection_struct *conn,
 		status = map_nt_error_from_unix(errno);
 	}
 
-	DEBUG(3, ("rename_internals_fsp: Error %s rename %s -> %s\n",
-		  nt_errstr(status), smb_fname_str_dbg(fsp->fsp_name),
-		  smb_fname_str_dbg(smb_fname_dst)));
+	DBG_NOTICE("Error %s rename %s -> %s\n",
+		   nt_errstr(status),
+		   smb_fname_str_dbg(fsp->fsp_name),
+		   smb_fname_str_dbg(smb_fname_dst));
 
  out:
 
@@ -1855,7 +1859,6 @@ NTSTATUS rename_internals(TALLOC_CTX *ctx,
 			struct smb_request *req,
 			struct files_struct *src_dirfsp,
 			struct smb_filename *smb_fname_src,
-			struct files_struct *dst_dirfsp,
 			struct smb_filename *smb_fname_dst,
 			const char *dst_original_lcomp,
 			uint32_t attrs,
@@ -1863,7 +1866,7 @@ NTSTATUS rename_internals(TALLOC_CTX *ctx,
 			uint32_t access_mask)
 {
 	NTSTATUS status = NT_STATUS_OK;
-	int create_options = 0;
+	int create_options = FILE_OPEN_REPARSE_POINT;
 	struct smb2_create_blobs *posx = NULL;
 	struct files_struct *fsp = NULL;
 	bool posix_pathname = (smb_fname_src->flags & SMB_FILENAME_POSIX_PATH);
@@ -1945,7 +1948,6 @@ NTSTATUS rename_internals(TALLOC_CTX *ctx,
 
 	status = rename_internals_fsp(conn,
 					fsp,
-					dst_dirfsp,
 					smb_fname_dst,
 					dst_original_lcomp,
 					attrs,
