@@ -108,7 +108,7 @@ static int show_share(const struct connections_data *crec,
 	d_printf("%-10.10s   %s   %-12s  %s",
 	       crec->servicename, server_id_str_buf(crec->pid, &tmp),
 	       crec->machine,
-	       time_to_asc(crec->start));
+	       time_to_asc(nt_time_to_unix(crec->start)));
 
 	return 0;
 }
@@ -166,7 +166,7 @@ static int show_share_parseable(const struct connections_data *crec,
 		 guest ? "" : gidtoname(ids->entries[i].gid),
 		 crec->machine,
 		 guest ? "" : ids->entries[i].hostname,
-		 time_to_asc(crec->start));
+		 time_to_asc(nt_time_to_unix(crec->start)));
 
 	return 0;
 }
