@@ -540,7 +540,8 @@ static bool get_dc_name_via_netlogon(struct winbindd_domain *domain,
 			tmp = talloc_strdup(
 				mem_ctx, domain_info->dc_unc);
 			if (tmp == NULL) {
-				DEBUG(0, ("talloc_strdup failed\n"));
+				DBG_ERR("talloc_strdup failed for dc_unc[%s]\n",
+					domain_info->dc_unc);
 				talloc_destroy(mem_ctx);
 				return false;
 			}
@@ -548,7 +549,9 @@ static bool get_dc_name_via_netlogon(struct winbindd_domain *domain,
 				domain->alt_name = talloc_strdup(domain,
 								 domain_info->domain_name);
 				if (domain->alt_name == NULL) {
-					DEBUG(0, ("talloc_strdup failed\n"));
+					DBG_ERR("talloc_strdup failed for "
+						"domain_info->domain_name[%s]\n",
+						domain_info->domain_name);
 					talloc_destroy(mem_ctx);
 					return false;
 				}
@@ -557,7 +560,9 @@ static bool get_dc_name_via_netlogon(struct winbindd_domain *domain,
 				domain->forest_name = talloc_strdup(domain,
 								    domain_info->forest_name);
 				if (domain->forest_name == NULL) {
-					DEBUG(0, ("talloc_strdup failed\n"));
+					DBG_ERR("talloc_strdup failed for "
+						"domain_info->forest_name[%s]\n",
+						domain_info->forest_name);
 					talloc_destroy(mem_ctx);
 					return false;
 				}
@@ -607,10 +612,11 @@ static bool get_dc_name_via_netlogon(struct winbindd_domain *domain,
 /**
  * Helper function to assemble trust password and account name
  */
-static NTSTATUS get_trust_credentials(struct winbindd_domain *domain,
-				      TALLOC_CTX *mem_ctx,
-				      bool netlogon,
-				      struct cli_credentials **_creds)
+NTSTATUS winbindd_get_trust_credentials(struct winbindd_domain *domain,
+					TALLOC_CTX *mem_ctx,
+					bool netlogon,
+					bool allow_ipc_fallback,
+					struct cli_credentials **_creds)
 {
 	const struct winbindd_domain *creds_domain = NULL;
 	struct cli_credentials *creds;
@@ -665,6 +671,10 @@ static NTSTATUS get_trust_credentials(struct winbindd_domain *domain,
 		return NT_STATUS_CANT_ACCESS_DOMAIN_INFO;
 	}
 
+	if (!allow_ipc_fallback) {
+		return status;
+	}
+
 	status = cm_get_ipc_credentials(mem_ctx, &creds);
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
@@ -701,13 +711,14 @@ static NTSTATUS cm_prepare_connection(struct winbindd_domain *domain,
 
 	enum smb_signing_setting smb_sign_client_connections = lp_client_ipc_signing();
 
-	if (IS_AD_DC) {
+	if (IS_DC) {
 		if (domain->secure_channel_type == SEC_CHAN_NULL) {
 			/*
 			 * Make sure we don't even try to
 			 * connect to a foreign domain
 			 * without a direct outbound trust.
 			 */
+			close(sockfd);
 			return NT_STATUS_NO_TRUST_LSA_SECRET;
 		}
 
@@ -761,6 +772,13 @@ static NTSTATUS cm_prepare_connection(struct winbindd_domain *domain,
 		goto done;
 	}
 
+	/*
+	 * cm_prepare_connection() is responsible that sockfd does not leak.
+	 * Once cli_state_create() returns with success, the
+	 * smbXcli_conn_destructor() makes sure that close(sockfd) is finally
+	 * called. Till that, close(sockfd) must be called on every unsuccessful
+	 * return.
+	 */
 	*cli = cli_state_create(NULL, sockfd, controller,
 				smb_sign_client_connections, flags);
 	if (*cli == NULL) {
@@ -774,9 +792,13 @@ static NTSTATUS cm_prepare_connection(struct winbindd_domain *domain,
 
 	set_socket_options(sockfd, lp_socket_options());
 
-	result = smbXcli_negprot((*cli)->conn, (*cli)->timeout,
+	result = smbXcli_negprot((*cli)->conn,
+				 (*cli)->timeout,
 				 lp_client_ipc_min_protocol(),
-				 lp_client_ipc_max_protocol());
+				 lp_client_ipc_max_protocol(),
+				 NULL,
+				 NULL,
+				 NULL);
 
 	if (!NT_STATUS_IS_OK(result)) {
 		DEBUG(1, ("cli_negprot failed: %s\n", nt_errstr(result)));
@@ -797,7 +819,7 @@ static NTSTATUS cm_prepare_connection(struct winbindd_domain *domain,
 		try_ipc_auth = true;
 	}
 
-	if (IS_AD_DC) {
+	if (IS_DC) {
 		/*
 		 * As AD DC we only use netlogon and lsa
 		 * using schannel over an anonymous transport
@@ -814,10 +836,16 @@ static NTSTATUS cm_prepare_connection(struct winbindd_domain *domain,
 	}
 
 	if (try_ipc_auth) {
-		result = get_trust_credentials(domain, talloc_tos(), false, &creds);
+		result = winbindd_get_trust_credentials(domain,
+							talloc_tos(),
+							false, /* netlogon */
+							true, /* ipc_fallback */
+							&creds);
 		if (!NT_STATUS_IS_OK(result)) {
-			DEBUG(1, ("get_trust_credentials(%s) failed: %s\n",
-				  domain->name, nt_errstr(result)));
+			DBG_WARNING("winbindd_get_trust_credentials(%s) "
+				    "failed: %s\n",
+				    domain->name,
+				    nt_errstr(result));
 			goto done;
 		}
 	} else {
@@ -1093,6 +1121,9 @@ static bool dcip_check_name_ads(const struct winbindd_domain *domain,
 	char addr[INET6_ADDRSTRLEN];
 
 	print_sockaddr(addr, sizeof(addr), &sa->u.ss);
+	D_DEBUG("Trying to figure out the DC name for domain '%s' at IP '%s'.\n",
+		domain->name,
+		addr);
 
 	ads = ads_init(tmp_ctx,
 		       domain->alt_name,
@@ -1103,11 +1134,10 @@ static bool dcip_check_name_ads(const struct winbindd_domain *domain,
 		ads_status = ADS_ERROR_NT(NT_STATUS_NO_MEMORY);
 		goto out;
 	}
-	ads->auth.flags |= ADS_AUTH_NO_BIND;
 	ads->config.flags |= request_flags;
 	ads->server.no_fallback = true;
 
-	ads_status = ads_connect(ads);
+	ads_status = ads_connect_cldap_only(ads);
 	if (!ADS_ERR_OK(ads_status)) {
 		goto out;
 	}
@@ -1151,6 +1181,10 @@ static bool dcip_check_name_ads(const struct winbindd_domain *domain,
 		saf_store(domain->alt_name, name);
 	}
 
+	D_DEBUG("DC name for domain '%s' at IP '%s' is '%s'\n",
+		domain->name,
+		addr,
+		name);
 	*namep = talloc_move(mem_ctx, &name);
 
 out:
@@ -1414,9 +1448,12 @@ static bool connect_preferred_dc(TALLOC_CTX *mem_ctx,
 	 * Check the negative connection cache before talking to it. It going
 	 * down may have triggered the reconnection.
 	 */
-	status = check_negative_conn_cache(domain->name, saf_servername);
-	if (!NT_STATUS_IS_OK(status)) {
-		saf_servername = NULL;
+	if (saf_servername != NULL) {
+		status = check_negative_conn_cache(domain->name,
+						   saf_servername);
+		if (!NT_STATUS_IS_OK(status)) {
+			saf_servername = NULL;
+		}
 	}
 
 	if (saf_servername != NULL) {
@@ -1465,7 +1502,10 @@ static bool connect_preferred_dc(TALLOC_CTX *mem_ctx,
 				 NULL, -1, NULL, -1,
 				 fd, NULL, 10);
 	if (!NT_STATUS_IS_OK(status)) {
-		goto fail;
+		winbind_add_failed_connection_entry(domain,
+						    domain->dcname,
+						    NT_STATUS_UNSUCCESSFUL);
+		return false;
 	}
 	return true;
 
@@ -1508,6 +1548,9 @@ static bool find_dc(TALLOC_CTX *mem_ctx,
 
 	*fd = -1;
 
+	D_NOTICE("First try to connect to the closest DC (using server "
+		 "affinity cache). If this fails, try to lookup the DC using "
+		 "DNS afterwards.\n");
 	ok = connect_preferred_dc(mem_ctx, domain, request_flags, fd);
 	if (ok) {
 		return true;
@@ -1518,9 +1561,11 @@ static bool find_dc(TALLOC_CTX *mem_ctx,
 	}
 
  again:
+	D_DEBUG("Retrieving a list of IP addresses for DCs.\n");
 	if (!get_dcs(mem_ctx, domain, &dcs, &num_dcs, request_flags) || (num_dcs == 0))
 		return False;
 
+	D_DEBUG("Retrieved IP addresses for %d DCs.\n", num_dcs);
 	for (i=0; i<num_dcs; i++) {
 
 		if (!add_string_to_array(mem_ctx, dcs[i].name,
@@ -1539,6 +1584,9 @@ static bool find_dc(TALLOC_CTX *mem_ctx,
 	if ((addrs == NULL) || (dcnames == NULL))
 		return False;
 
+	D_DEBUG("Trying to establish a connection to one of the %d DCs "
+		"(timeout of 10 sec for each DC).\n",
+		num_dcs);
 	status = smbsock_any_connect(addrs, dcnames, NULL, NULL, NULL,
 				     num_addrs, 0, 10, fd, &fd_index, NULL);
 	if (!NT_STATUS_IS_OK(status)) {
@@ -1553,6 +1601,7 @@ static bool find_dc(TALLOC_CTX *mem_ctx,
 		}
 		return False;
 	}
+	D_NOTICE("Successfully connected to DC '%s'.\n", dcs[fd_index].name);
 
 	domain->dcaddr = addrs[fd_index];
 
@@ -1596,6 +1645,11 @@ static bool find_dc(TALLOC_CTX *mem_ctx,
 		*fd = -1;
 	}
 
+	/*
+	 * This should not be an infinite loop, since get_dcs() will not return
+	 * the DC added to the negative connection cache in the above
+	 * winbind_add_failed_connection_entry() call.
+	 */
 	goto again;
 }
 
@@ -1725,11 +1779,17 @@ static NTSTATUS cm_open_connection(struct winbindd_domain *domain,
 		return NT_STATUS_NO_MEMORY;
 	}
 
+	D_NOTICE("Creating connection to domain controller. This is a start of "
+		 "a new connection or a DC failover. The failover only happens "
+		 "if the domain has more than one DC. We will try to connect 3 "
+		 "times at most.\n");
 	for (retries = 0; retries < 3; retries++) {
 		bool found_dc;
 
-		DEBUG(10, ("cm_open_connection: dcname is '%s' for domain %s\n",
-			   domain->dcname ? domain->dcname : "", domain->name));
+		D_DEBUG("Attempt %d/3: DC '%s' of domain '%s'.\n",
+			retries,
+			domain->dcname ? domain->dcname : "",
+			domain->name);
 
 		found_dc = find_dc(mem_ctx, domain, request_flags, &fd);
 		if (!found_dc) {
@@ -1749,8 +1809,6 @@ static NTSTATUS cm_open_connection(struct winbindd_domain *domain,
 		if (NT_STATUS_IS_OK(result)) {
 			break;
 		}
-		close(fd);
-		fd = -1;
 		if (!retry) {
 			break;
 		}
@@ -2137,12 +2195,19 @@ static bool set_dc_type_and_flags_trustinfo( struct winbindd_domain *domain )
 static void set_dc_type_and_flags_connect( struct winbindd_domain *domain )
 {
 	NTSTATUS status, result;
+	NTSTATUS close_status = NT_STATUS_UNSUCCESSFUL;
 	WERROR werr;
 	TALLOC_CTX              *mem_ctx = NULL;
 	struct rpc_pipe_client  *cli = NULL;
-	struct policy_handle pol;
+	struct policy_handle pol = { .handle_type = 0 };
 	union dssetup_DsRoleInfo info;
 	union lsa_PolicyInformation *lsa_info = NULL;
+	union lsa_revision_info out_revision_info = {
+		.info1 = {
+			.revision = 0,
+		},
+	};
+	uint32_t out_version = 0;
 
 	if (!domain->internal && !connection_ok(domain)) {
 		return;
@@ -2231,10 +2296,17 @@ no_dssetup:
 		return;
 	}
 
-	status = rpccli_lsa_open_policy2(cli, mem_ctx, True,
-					 SEC_FLAG_MAXIMUM_ALLOWED, &pol);
+	status = dcerpc_lsa_open_policy_fallback(cli->binding_handle,
+						 mem_ctx,
+						 cli->srv_name_slash,
+						 true,
+						 SEC_FLAG_MAXIMUM_ALLOWED,
+						 &out_version,
+						 &out_revision_info,
+						 &pol,
+						 &result);
 
-	if (NT_STATUS_IS_OK(status)) {
+	if (NT_STATUS_IS_OK(status) && NT_STATUS_IS_OK(result)) {
 		/* This particular query is exactly what Win2k clients use
 		   to determine that the DC is active directory */
 		status = dcerpc_lsa_QueryInfoPolicy2(cli->binding_handle, mem_ctx,
@@ -2244,6 +2316,10 @@ no_dssetup:
 						     &result);
 	}
 
+	/*
+	 * If the status and result will not be OK we will fallback to
+	 * OpenPolicy.
+	 */
 	if (NT_STATUS_IS_OK(status) && NT_STATUS_IS_OK(result)) {
 		domain->active_directory = True;
 
@@ -2394,6 +2470,12 @@ no_dssetup:
 		}
 	}
 done:
+	if (is_valid_policy_hnd(&pol)) {
+		dcerpc_lsa_Close(cli->binding_handle,
+				 mem_ctx,
+				 &pol,
+				 &close_status);
+	}
 
 	DEBUG(5, ("set_dc_type_and_flags_connect: domain %s is %sin native mode.\n",
 		  domain->name, domain->native_mode ? "" : "NOT "));
@@ -2530,7 +2612,11 @@ retry:
 	 * anonymous.
 	 */
 
-	result = get_trust_credentials(domain, talloc_tos(), false, &creds);
+	result = winbindd_get_trust_credentials(domain,
+						talloc_tos(),
+						false, /* netlogon */
+						true, /* ipc_fallback */
+						&creds);
 	if (!NT_STATUS_IS_OK(result)) {
 		DEBUG(10, ("cm_connect_sam: No user available for "
 			   "domain %s, trying schannel\n", domain->name));
@@ -2857,14 +2943,18 @@ retry:
 
 	TALLOC_FREE(conn->lsa_pipe);
 
-	if (IS_AD_DC) {
+	if (IS_DC) {
 		/*
 		 * Make sure we only use schannel as AD DC.
 		 */
 		goto schannel;
 	}
 
-	result = get_trust_credentials(domain, talloc_tos(), false, &creds);
+	result = winbindd_get_trust_credentials(domain,
+						talloc_tos(),
+						false, /* netlogon */
+						true, /* ipc_fallback */
+						&creds);
 	if (!NT_STATUS_IS_OK(result)) {
 		DEBUG(10, ("cm_connect_lsa: No user available for "
 			   "domain %s, trying schannel\n", domain->name));
@@ -2983,7 +3073,7 @@ retry:
 		goto done;
 	}
 
-	if (IS_AD_DC) {
+	if (IS_DC) {
 		/*
 		 * Make sure we only use schannel as AD DC.
 		 */
@@ -2997,7 +3087,7 @@ retry:
 
  anonymous:
 
-	if (IS_AD_DC) {
+	if (IS_DC) {
 		/*
 		 * Make sure we only use schannel as AD DC.
 		 */
@@ -3063,7 +3153,7 @@ retry:
 }
 
 /****************************************************************************
-Open a LSA connection to a DC, suiteable for LSA lookup calls.
+Open a LSA connection to a DC, suitable for LSA lookup calls.
 ****************************************************************************/
 
 NTSTATUS cm_connect_lsat(struct winbindd_domain *domain,
@@ -3116,7 +3206,7 @@ static NTSTATUS cm_connect_netlogon_transport(struct winbindd_domain *domain,
 
 	*cli = NULL;
 
-	if (IS_AD_DC) {
+	if (IS_DC) {
 		if (domain->secure_channel_type == SEC_CHAN_NULL) {
 			/*
 			 * Make sure we don't even try to
@@ -3142,7 +3232,11 @@ static NTSTATUS cm_connect_netlogon_transport(struct winbindd_domain *domain,
 	TALLOC_FREE(conn->netlogon_pipe);
 	TALLOC_FREE(conn->netlogon_creds_ctx);
 
-	result = get_trust_credentials(domain, talloc_tos(), true, &creds);
+	result = winbindd_get_trust_credentials(domain,
+						talloc_tos(),
+						true, /* netlogon */
+						false, /* ipc_fallback */
+						&creds);
 	if (!NT_STATUS_IS_OK(result)) {
 		DBG_DEBUG("No user available for domain %s when trying "
 			  "schannel\n", domain->name);
@@ -3219,7 +3313,7 @@ static NTSTATUS cm_connect_netlogon_transport(struct winbindd_domain *domain,
 }
 
 /****************************************************************************
-Open a NETLOGON connection to a DC, suiteable for SamLogon calls.
+Open a NETLOGON connection to a DC, suitable for SamLogon calls.
 ****************************************************************************/
 
 NTSTATUS cm_connect_netlogon(struct winbindd_domain *domain,
@@ -3320,7 +3414,7 @@ void winbind_msg_ip_dropped(struct messaging_context *msg_ctx,
 		 */
 		slash = strchr(addr, '/');
 		if (slash == NULL) {
-			DEBUG(1, ("invalid msg_ip_dropped message: %s",
+			DEBUG(1, ("invalid msg_ip_dropped message: %s\n",
 				  addr));
 			return;
 		}
