@@ -1,20 +1,20 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
 
    Wrapper around winbindd_rpc.c to centralize retry logic.
 
    Copyright (C) Volker Lendecke 2005
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
@@ -131,18 +131,19 @@ static NTSTATUS name_to_sid(struct winbindd_domain *domain,
 			    const char *domain_name,
 			    const char *name,
 			    uint32_t flags,
+			    const char **pdom_name,
 			    struct dom_sid *sid,
 			    enum lsa_SidType *type)
 {
 	NTSTATUS result;
 
 	result = msrpc_methods.name_to_sid(domain, mem_ctx, domain_name, name,
-					   flags, sid, type);
+					   flags, pdom_name, sid, type);
 
 	if (reconnect_need_retry(result, domain))
 		result = msrpc_methods.name_to_sid(domain, mem_ctx,
 						   domain_name, name, flags,
-						   sid, type);
+						   pdom_name, sid, type);
 
 	return result;
 }
@@ -234,6 +235,34 @@ static NTSTATUS lookup_useraliases(struct winbindd_domain *domain,
 	return result;
 }
 
+/* Lookup alias membership given */
+static NTSTATUS lookup_aliasmem(struct winbindd_domain *domain,
+				TALLOC_CTX *mem_ctx,
+				const struct dom_sid *sid,
+				enum lsa_SidType type,
+				uint32_t *num_sids,
+				struct dom_sid **sids)
+{
+	NTSTATUS result;
+
+	result = msrpc_methods.lookup_aliasmem(domain,
+					       mem_ctx,
+					       sid,
+					       type,
+					       num_sids,
+					       sids);
+
+	if (reconnect_need_retry(result, domain))
+		result = msrpc_methods.lookup_aliasmem(domain,
+						       mem_ctx,
+						       sid,
+						       type,
+						       num_sids,
+						       sids);
+
+	return result;
+}
+
 /* Lookup group membership given a rid.   */
 static NTSTATUS lookup_groupmem(struct winbindd_domain *domain,
 				TALLOC_CTX *mem_ctx,
@@ -260,21 +289,8 @@ static NTSTATUS lookup_groupmem(struct winbindd_domain *domain,
 	return result;
 }
 
-/* find the sequence number for a domain */
-static NTSTATUS sequence_number(struct winbindd_domain *domain, uint32_t *seq)
-{
-	NTSTATUS result;
-
-	result = msrpc_methods.sequence_number(domain, seq);
-
-	if (reconnect_need_retry(result, domain))
-		result = msrpc_methods.sequence_number(domain, seq);
-
-	return result;
-}
-
 /* find the lockout policy of a domain */
-static NTSTATUS lockout_policy(struct winbindd_domain *domain, 
+static NTSTATUS lockout_policy(struct winbindd_domain *domain,
 			       TALLOC_CTX *mem_ctx,
 			       struct samr_DomInfo12 *policy)
 {
@@ -289,17 +305,17 @@ static NTSTATUS lockout_policy(struct winbindd_domain *domain,
 }
 
 /* find the password policy of a domain */
-static NTSTATUS password_policy(struct winbindd_domain *domain, 
+static NTSTATUS password_policy(struct winbindd_domain *domain,
 				TALLOC_CTX *mem_ctx,
 				struct samr_DomInfo1 *policy)
 {
  	NTSTATUS result;
- 
+
 	result = msrpc_methods.password_policy(domain, mem_ctx, policy);
 
 	if (reconnect_need_retry(result, domain))
 		result = msrpc_methods.password_policy(domain, mem_ctx, policy);
-	
+
 	return result;
 }
 
@@ -331,7 +347,7 @@ struct winbindd_methods reconnect_methods = {
 	lookup_usergroups,
 	lookup_useraliases,
 	lookup_groupmem,
-	sequence_number,
+	lookup_aliasmem,
 	lockout_policy,
 	password_policy,
 	trusted_domains,
