@@ -25,6 +25,7 @@
 #include "includes.h"
 #include "ldb_module.h"
 #include "lib/audit_logging/audit_logging.h"
+#include "librpc/gen_ndr/windows_event_ids.h"
 
 #include "dsdb/samdb/samdb.h"
 #include "dsdb/samdb/ldb_modules/util.h"
@@ -36,10 +37,11 @@
 #define AUDIT_JSON_TYPE "groupChange"
 #define AUDIT_HR_TAG "Group Change"
 #define AUDIT_MAJOR 1
-#define AUDIT_MINOR 0
+#define AUDIT_MINOR 1
 #define GROUP_LOG_LVL 5
 
-static const char * const member_attr[] = {"member", NULL};
+static const char *const group_attrs[] = {"member", "groupType", NULL};
+static const char *const group_type_attr[] = {"groupType", NULL};
 static const char * const primary_group_attr[] = {
 	"primaryGroupID",
 	"objectSID",
@@ -90,7 +92,6 @@ static struct GUID *get_transaction_id(
 	return &transaction_id->transaction_guid;
 }
 
-#ifdef HAVE_JANSSON
 /*
  * @brief generate a JSON log entry for a group change.
  *
@@ -106,13 +107,13 @@ static struct GUID *get_transaction_id(
  * @return A json object containing the details.
  * 	   NULL if an error was detected
  */
-static struct json_object audit_group_json(
-	const struct ldb_module *module,
-	const struct ldb_request *request,
-	const char *action,
-	const char *user,
-	const char *group,
-	const int status)
+static struct json_object audit_group_json(const struct ldb_module *module,
+					   const struct ldb_request *request,
+					   const char *action,
+					   const char *user,
+					   const char *group,
+					   const enum event_id_type event_id,
+					   const int status)
 {
 	struct ldb_context *ldb = NULL;
 	const struct dom_sid *sid = NULL;
@@ -138,6 +139,12 @@ static struct json_object audit_group_json(
 	if (rc != 0) {
 		goto failure;
 	}
+	if (event_id != EVT_ID_NONE) {
+		rc = json_add_int(&audit, "eventId", event_id);
+		if (rc != 0) {
+			goto failure;
+		}
+	}
 	rc = json_add_int(&audit, "statusCode", status);
 	if (rc != 0) {
 		goto failure;
@@ -208,7 +215,6 @@ failure:
 	DBG_ERR("Failed to create group change JSON log message\n");
 	return wrapper;
 }
-#endif
 
 /*
  * @brief generate a human readable log entry for a group change.
@@ -285,23 +291,18 @@ static struct parsed_dn *get_parsed_dns(
 	TALLOC_CTX *mem_ctx,
 	struct ldb_message_element *el)
 {
+	int ret;
 	struct parsed_dn *pdn = NULL;
 
-	int i;
-
 	if (el == NULL || el->num_values == 0) {
 		return NULL;
 	}
 
-	pdn = talloc_zero_array(mem_ctx, struct parsed_dn, el->num_values);
-	if (pdn == NULL) {
+	ret = get_parsed_dns_trusted(mem_ctx, el, &pdn);
+	if (ret == LDB_ERR_OPERATIONS_ERROR) {
 		DBG_ERR("Out of memory\n");
 		return NULL;
 	}
-
-	for (i = 0; i < el->num_values; i++) {
-		pdn[i].v = &el->values[i];
-	}
 	return pdn;
 
 }
@@ -317,7 +318,7 @@ enum dn_compare_result {
  *
  * Compare two parsed_dn structures, using GUID ordering.
  * To avoid the overhead of parsing the DN's this function does a binary
- * compare first. The DN's tre only parsed if they are not equal at a binary
+ * compare first. The DN's are only parsed if they are not equal at a binary
  * level.
  *
  * @param ctx talloc context that will own the parsed dsdb_dn
@@ -451,9 +452,11 @@ static const char *get_primary_group_dn(
  * @brief Log details of a change to a users primary group.
  *
  * Log details of a change to a users primary group.
+ * There is no windows event id associated with a Primary Group change.
+ * However for a new user we generate an added to group event.
  *
  * @param module The ldb module.
- * @param request The request deing logged.
+ * @param request The request being logged.
  * @param action Description of the action being performed.
  * @param group The linearized for of the group DN
  * @param status the LDB status code for the processing of the request.
@@ -494,18 +497,12 @@ static void log_primary_group_change(
 		TALLOC_FREE(message);
 	}
 
-#ifdef HAVE_JANSSON
 	if (CHECK_DEBUGLVLC(DBGC_DSDB_GROUP_AUDIT_JSON, GROUP_LOG_LVL) ||
 		(ac->msg_ctx && ac->send_events)) {
 
 		struct json_object json;
 		json = audit_group_json(
-			module,
-			request,
-			action,
-			user,
-			group,
-			status);
+		    module, request, action, user, group, EVT_ID_NONE, status);
 		audit_log_json(
 			&json,
 			DBGC_DSDB_GROUP_AUDIT_JSON,
@@ -518,8 +515,14 @@ static void log_primary_group_change(
 				&json);
 		}
 		json_free(&json);
+		if (request->operation == LDB_ADD) {
+			/*
+			 * Have just added a user, generate a groupChange
+			 * message indicating the user has been added to their
+			 * new PrimaryGroup.
+			 */
+		}
 	}
-#endif
 	TALLOC_FREE(ctx);
 }
 
@@ -536,12 +539,12 @@ static void log_primary_group_change(
  * @param status the LDB status code for the processing of the request.
  *
  */
-static void log_membership_change(
-	struct ldb_module *module,
-	const struct ldb_request *request,
-	const char *action,
-	const char *user,
-	const int  status)
+static void log_membership_change(struct ldb_module *module,
+				  const struct ldb_request *request,
+				  const char *action,
+				  const char *user,
+				  const enum event_id_type event_id,
+				  const int status)
 {
 	const char *group = NULL;
 	struct audit_context *ac =
@@ -569,17 +572,11 @@ static void log_membership_change(
 		TALLOC_FREE(message);
 	}
 
-#ifdef HAVE_JANSSON
 	if (CHECK_DEBUGLVLC(DBGC_DSDB_GROUP_AUDIT_JSON, GROUP_LOG_LVL) ||
 		(ac->msg_ctx && ac->send_events)) {
 		struct json_object json;
 		json = audit_group_json(
-			module,
-			request,
-			action,
-			user,
-			group,
-			status);
+		    module, request, action, user, group, event_id, status);
 		audit_log_json(
 			&json,
 			DBGC_DSDB_GROUP_AUDIT_JSON,
@@ -593,10 +590,71 @@ static void log_membership_change(
 		}
 		json_free(&json);
 	}
-#endif
 	TALLOC_FREE(ctx);
 }
 
+/*
+ * @brief Get the windows event type id for removing a user from a group type.
+ *
+ * @param group_type the type of the current group, see libds/common/flags.h
+ *
+ * @return the Windows Event Id
+ *
+ */
+static enum event_id_type get_remove_member_event(uint32_t group_type)
+{
+
+	switch (group_type) {
+	case GTYPE_SECURITY_BUILTIN_LOCAL_GROUP:
+		return EVT_ID_USER_REMOVED_FROM_LOCAL_SEC_GROUP;
+	case GTYPE_SECURITY_GLOBAL_GROUP:
+		return EVT_ID_USER_REMOVED_FROM_GLOBAL_SEC_GROUP;
+	case GTYPE_SECURITY_DOMAIN_LOCAL_GROUP:
+		return EVT_ID_USER_REMOVED_FROM_LOCAL_SEC_GROUP;
+	case GTYPE_SECURITY_UNIVERSAL_GROUP:
+		return EVT_ID_USER_REMOVED_FROM_UNIVERSAL_SEC_GROUP;
+	case GTYPE_DISTRIBUTION_GLOBAL_GROUP:
+		return EVT_ID_USER_REMOVED_FROM_GLOBAL_GROUP;
+	case GTYPE_DISTRIBUTION_DOMAIN_LOCAL_GROUP:
+		return EVT_ID_USER_REMOVED_FROM_LOCAL_GROUP;
+	case GTYPE_DISTRIBUTION_UNIVERSAL_GROUP:
+		return EVT_ID_USER_REMOVED_FROM_UNIVERSAL_GROUP;
+	default:
+		return EVT_ID_NONE;
+	}
+}
+
+/*
+ * @brief Get the windows event type id for adding a user to a group type.
+ *
+ * @param group_type the type of the current group, see libds/common/flags.h
+ *
+ * @return the Windows Event Id
+ *
+ */
+static enum event_id_type get_add_member_event(uint32_t group_type)
+{
+
+	switch (group_type) {
+	case GTYPE_SECURITY_BUILTIN_LOCAL_GROUP:
+		return EVT_ID_USER_ADDED_TO_LOCAL_SEC_GROUP;
+	case GTYPE_SECURITY_GLOBAL_GROUP:
+		return EVT_ID_USER_ADDED_TO_GLOBAL_SEC_GROUP;
+	case GTYPE_SECURITY_DOMAIN_LOCAL_GROUP:
+		return EVT_ID_USER_ADDED_TO_LOCAL_SEC_GROUP;
+	case GTYPE_SECURITY_UNIVERSAL_GROUP:
+		return EVT_ID_USER_ADDED_TO_UNIVERSAL_SEC_GROUP;
+	case GTYPE_DISTRIBUTION_GLOBAL_GROUP:
+		return EVT_ID_USER_ADDED_TO_GLOBAL_GROUP;
+	case GTYPE_DISTRIBUTION_DOMAIN_LOCAL_GROUP:
+		return EVT_ID_USER_ADDED_TO_LOCAL_GROUP;
+	case GTYPE_DISTRIBUTION_UNIVERSAL_GROUP:
+		return EVT_ID_USER_ADDED_TO_UNIVERSAL_GROUP;
+	default:
+		return EVT_ID_NONE;
+	}
+}
+
 /*
  * @brief Log all the changes to a users group membership.
  *
@@ -610,12 +668,12 @@ static void log_membership_change(
  * @param status the LDB status code for the processing of the request.
  *
  */
-static void log_membership_changes(
-	struct ldb_module *module,
-	const struct ldb_request *request,
-	struct ldb_message_element *el,
-	struct ldb_message_element *old_el,
-	int status)
+static void log_membership_changes(struct ldb_module *module,
+				   const struct ldb_request *request,
+				   struct ldb_message_element *el,
+				   struct ldb_message_element *old_el,
+				   uint32_t group_type,
+				   int status)
 {
 	unsigned int i, old_i, new_i;
 	unsigned int old_num_values;
@@ -680,6 +738,7 @@ static void log_membership_changes(
 			 * the new record. So it's been deleted
 			 */
 			const char *user = NULL;
+			enum event_id_type event_id;
 			if (old_val->dsdb_dn == NULL) {
 				really_parse_trusted_dn(
 					ctx,
@@ -688,12 +747,9 @@ static void log_membership_changes(
 					LDB_SYNTAX_DN);
 			}
 			user = ldb_dn_get_linearized(old_val->dsdb_dn->dn);
+			event_id = get_remove_member_event(group_type);
 			log_membership_change(
-				module,
-				request,
-				"Removed",
-				user,
-				status);
+			    module, request, "Removed", user, event_id, status);
 			old_i++;
 		} else if (cmp == BINARY_EQUAL) {
 			/*
@@ -745,27 +801,31 @@ static void log_membership_changes(
 				 * DN has been deleted.
 				 */
 				const char *user = NULL;
+				enum event_id_type event_id;
 				user = ldb_dn_get_linearized(
 					old_val->dsdb_dn->dn);
-				log_membership_change(
-					module,
-					request,
-					"Removed",
-					user,
-					status);
+				event_id = get_remove_member_event(group_type);
+				log_membership_change(module,
+						      request,
+						      "Removed",
+						      user,
+						      event_id,
+						      status);
 			} else {
 				/*
 				 * DN has been re-added
 				 */
 				const char *user = NULL;
+				enum event_id_type event_id;
 				user = ldb_dn_get_linearized(
 					new_val->dsdb_dn->dn);
-				log_membership_change(
-					module,
-					request,
-					"Added",
-					user,
-					status);
+				event_id = get_add_member_event(group_type);
+				log_membership_change(module,
+						      request,
+						      "Added",
+						      user,
+						      event_id,
+						      status);
 			}
 			old_i++;
 			new_i++;
@@ -775,6 +835,7 @@ static void log_membership_changes(
 			 * original, so it must have been added.
 			 */
 			const char *user = NULL;
+			enum event_id_type event_id;
 			if ( new_val->dsdb_dn == NULL) {
 				really_parse_trusted_dn(
 					ctx,
@@ -783,12 +844,9 @@ static void log_membership_changes(
 					LDB_SYNTAX_DN);
 			}
 			user = ldb_dn_get_linearized(new_val->dsdb_dn->dn);
+			event_id = get_add_member_event(group_type);
 			log_membership_change(
-				module,
-				request,
-				"Added",
-				user,
-				status);
+			    module, request, "Added", user, event_id, status);
 			new_i++;
 		}
 	}
@@ -796,6 +854,46 @@ static void log_membership_changes(
 	TALLOC_FREE(ctx);
 }
 
+/*
+ * @brief log a group change message for a newly added user.
+ *
+ * When a user is added we need to generate a GroupChange Add message to
+ * log that the user has been added to their PrimaryGroup
+ */
+static void log_new_user_added_to_primary_group(
+    TALLOC_CTX *ctx,
+    struct audit_callback_context *acc,
+    const char *group,
+    const int status)
+{
+	uint32_t group_type;
+	enum event_id_type event_id = EVT_ID_NONE;
+	struct ldb_result *res = NULL;
+	struct ldb_dn *group_dn = NULL;
+	struct ldb_context *ldb = NULL;
+	int ret;
+
+	ldb = ldb_module_get_ctx(acc->module);
+	group_dn = ldb_dn_new(ctx, ldb, group);
+	ret = dsdb_module_search_dn(acc->module,
+				    ctx,
+				    &res,
+				    group_dn,
+				    group_type_attr,
+				    DSDB_FLAG_NEXT_MODULE |
+					DSDB_SEARCH_REVEAL_INTERNALS |
+					DSDB_SEARCH_SHOW_DN_IN_STORAGE_FORMAT,
+				    NULL);
+	if (ret == LDB_SUCCESS) {
+		const char *user = NULL;
+		group_type =
+		    ldb_msg_find_attr_as_uint(res->msgs[0], "groupType", 0);
+		event_id = get_add_member_event(group_type);
+		user = dsdb_audit_get_primary_dn(acc->request);
+		log_membership_change(
+		    acc->module, acc->request, "Added", user, event_id, status);
+	}
+}
 
 /*
  * @brief Log the details of a primary group change.
@@ -813,10 +911,11 @@ static void log_user_primary_group_change(
 	const int status)
 {
 	TALLOC_CTX *ctx = talloc_new(NULL);
-	uint32_t new_rid;
+	uint32_t new_rid = UINT32_MAX;
 	struct dom_sid *account_sid = NULL;
 	int ret;
 	const struct ldb_message *msg = dsdb_audit_get_message(acc->request);
+
 	if (status == LDB_SUCCESS && msg != NULL) {
 		struct ldb_result *res = NULL;
 		ret = dsdb_module_search_dn(
@@ -846,7 +945,7 @@ static void log_user_primary_group_change(
 	 * Otherwise only log if the primary group has actually changed.
 	 */
 	if (account_sid != NULL &&
-	    new_rid != ~0 &&
+	    new_rid != UINT32_MAX &&
 	    acc->primary_group != new_rid) {
 		const char* group = get_primary_group_dn(
 			ctx,
@@ -859,6 +958,16 @@ static void log_user_primary_group_change(
 			"PrimaryGroup",
 			group,
 			status);
+		/*
+		 * Are we adding a new user with the primaryGroupID
+		 * set. If so and we're generating JSON audit logs, will need to
+		 * generate an "Add" message with the appropriate windows
+		 * event id.
+		 */
+		if (acc->request->operation == LDB_ADD) {
+			log_new_user_added_to_primary_group(
+			    ctx, acc, group, status);
+		}
 	}
 	TALLOC_FREE(ctx);
 }
@@ -872,7 +981,6 @@ static void log_user_primary_group_change(
  * @param acc details of the group memberships before the operation.
  * @param status The status code returned by the operation.
  *
- * @return an LDB status code.
  */
 static void log_group_membership_changes(
 	struct audit_callback_context *acc,
@@ -881,6 +989,7 @@ static void log_group_membership_changes(
 	TALLOC_CTX *ctx = talloc_new(NULL);
 	struct ldb_message_element *new_val = NULL;
 	int ret;
+	uint32_t group_type = 0;
 	const struct ldb_message *msg = dsdb_audit_get_message(acc->request);
 	if (status == LDB_SUCCESS && msg != NULL) {
 		struct ldb_result *res = NULL;
@@ -889,21 +998,42 @@ static void log_group_membership_changes(
 			ctx,
 			&res,
 			msg->dn,
-			member_attr,
+			group_attrs,
 			DSDB_FLAG_NEXT_MODULE |
 			DSDB_SEARCH_REVEAL_INTERNALS |
 			DSDB_SEARCH_SHOW_DN_IN_STORAGE_FORMAT,
 			NULL);
 		if (ret == LDB_SUCCESS) {
 			new_val = ldb_msg_find_element(res->msgs[0], "member");
+			group_type = ldb_msg_find_attr_as_uint(
+			    res->msgs[0], "groupType", 0);
+			log_membership_changes(acc->module,
+					       acc->request,
+					       new_val,
+					       acc->members,
+					       group_type,
+					       status);
+			TALLOC_FREE(ctx);
+			return;
 		}
 	}
-	log_membership_changes(
-		acc->module,
-		acc->request,
-		new_val,
-		acc->members,
-		status);
+	/*
+	 * If we get here either
+	 *   one of the lower level modules failed and the group record did
+	 *   not get updated
+	 * or
+	 *   the updated group record could not be read.
+	 *
+	 * In both cases it does not make sense to log individual membership
+	 * changes so we log a group membership change "Failure" message.
+	 *
+	 */
+	log_membership_change(acc->module,
+	                      acc->request,
+			      "Failure",
+			      "",
+			      EVT_ID_NONE,
+			      status);
 	TALLOC_FREE(ctx);
 }
 
@@ -1306,7 +1436,7 @@ static int set_group_modify_callback(
 		context,
 		&res,
 		req->op.add.message->dn,
-		member_attr,
+		group_attrs,
 		DSDB_FLAG_NEXT_MODULE |
 		DSDB_SEARCH_REVEAL_INTERNALS |
 		DSDB_SEARCH_SHOW_DN_IN_STORAGE_FORMAT,
