@@ -24,6 +24,7 @@
 #include "librpc/gen_ndr/ndr_drsuapi.h"
 #include "librpc/gen_ndr/ndr_drsblobs.h"
 #include "../lib/util/asn1.h"
+#include "lib/util/smb_strtox.h"
 
 
 /**
@@ -216,6 +217,7 @@ static WERROR _dsdb_pfm_make_binary_oid(const char *full_oid, TALLOC_CTX *mem_ct
 {
 	uint32_t last_subid;
 	const char *oid_subid;
+	int error = 0;
 
 	/* make last sub-identifier value */
 	oid_subid = strrchr(full_oid, '.');
@@ -223,7 +225,10 @@ static WERROR _dsdb_pfm_make_binary_oid(const char *full_oid, TALLOC_CTX *mem_ct
 		return WERR_INVALID_PARAMETER;
 	}
 	oid_subid++;
-	last_subid = strtoul(oid_subid, NULL, 10);
+	last_subid = smb_strtoul(oid_subid, NULL, 10, &error, SMB_STR_STANDARD);
+	if (error != 0) {
+		return WERR_INVALID_PARAMETER;
+	}
 
 	/* encode oid in BER format */
 	if (!ber_write_OID_String(mem_ctx, _bin_oid, full_oid)) {
