@@ -152,8 +152,9 @@ static void connect_multi_next_socket(struct composite_context *result)
 	if (composite_nomem(state, result)) return;
 
 	state->result = result;
-	result->status = socket_create(multi->server_address[multi->current_address]->family,
-					SOCKET_TYPE_STREAM, &state->sock, 0);
+	result->status = socket_create(
+		state, multi->server_address[multi->current_address]->family,
+		SOCKET_TYPE_STREAM, &state->sock, 0);
 	if (!composite_is_ok(result)) return;
 
 	state->addr = socket_address_copy(state, multi->server_address[multi->current_address]);
@@ -161,8 +162,6 @@ static void connect_multi_next_socket(struct composite_context *result)
 
 	socket_address_set_port(state->addr, multi->ports[multi->current_port]);
 
-	talloc_steal(state, state->sock);
-
 	creq = socket_connect_send(state->sock, NULL, 
 				   state->addr, 0,
 				   result->event_ctx);
