@@ -21,6 +21,7 @@
 
 #include "includes.h"
 #include "libnet/libnet.h"
+#include "libnet/libnet_join_proto.h"
 #include "librpc/gen_ndr/ndr_drsuapi_c.h"
 #include <ldb.h>
 #include <ldb_errors.h>
@@ -487,8 +488,8 @@ NTSTATUS libnet_JoinDomain(struct libnet_context *ctx, TALLOC_CTX *mem_ctx, stru
 	const char *password_str = NULL;
 	
 	r->out.error_string = NULL;
-	r2.samr_handle.out.error_string = NULL;
-	
+	ZERO_STRUCT(r2);
+
 	tmp_ctx = talloc_named(mem_ctx, 0, "libnet_Join temp context");
 	if (!tmp_ctx) {
 		r->out.error_string = NULL;
@@ -610,10 +611,13 @@ NTSTATUS libnet_JoinDomain(struct libnet_context *ctx, TALLOC_CTX *mem_ctx, stru
 		status = od.out.result;
 	}
 	if (!NT_STATUS_IS_OK(status)) {
-		r->out.error_string = talloc_asprintf(mem_ctx,
-						      "samr_OpenDomain for [%s] failed: %s",
-						      dom_sid_string(tmp_ctx, connect_with_info->out.domain_sid),
-						      nt_errstr(status));
+		struct dom_sid_buf buf;
+		r->out.error_string = talloc_asprintf(
+			mem_ctx,
+			"samr_OpenDomain for [%s] failed: %s",
+			dom_sid_str_buf(connect_with_info->out.domain_sid,
+					&buf),
+			nt_errstr(status));
 		talloc_free(tmp_ctx);
 		return status;
 	}
