@@ -21,6 +21,7 @@
 */
 
 #include "includes.h"
+#include "lib/util/util_file.h"
 #include "system/filesys.h"
 #include "../lib/util/dlinklist.h"
 #include "lib/events/events.h"
@@ -436,7 +437,7 @@ static bool dcerpc_bh_use_ndr64(struct dcerpc_binding_handle *h)
 }
 
 static void dcerpc_bh_do_ndr_print(struct dcerpc_binding_handle *h,
-				   int ndr_flags,
+				   ndr_flags_type ndr_flags,
 				   const void *_struct_ptr,
 				   const struct ndr_interface_call *call)
 {
