@@ -40,6 +40,8 @@ struct schannel_key_state {
 	bool dcerpc_schannel_auto;
 	struct cli_credentials *credentials;
 	struct netlogon_creds_CredentialState *creds;
+	uint32_t requested_negotiate_flags;
+	uint32_t required_negotiate_flags;
 	uint32_t local_negotiate_flags;
 	uint32_t remote_negotiate_flags;
 	struct netr_Credential credentials1;
@@ -55,7 +57,8 @@ static void continue_secondary_connection(struct composite_context *ctx);
 static void continue_bind_auth_none(struct composite_context *ctx);
 static void continue_srv_challenge(struct tevent_req *subreq);
 static void continue_srv_auth2(struct tevent_req *subreq);
-static void continue_get_capabilities(struct tevent_req *subreq);
+static void continue_get_negotiated_capabilities(struct tevent_req *subreq);
+static void continue_get_client_capabilities(struct tevent_req *subreq);
 
 
 /*
@@ -183,7 +186,7 @@ static void continue_srv_challenge(struct tevent_req *subreq)
 	s->a.in.secure_channel_type =
 		cli_credentials_get_secure_channel_type(s->credentials);
 	s->a.in.computer_name    = cli_credentials_get_workstation(s->credentials);
-	s->a.in.negotiate_flags  = &s->local_negotiate_flags;
+	s->a.in.negotiate_flags  = &s->requested_negotiate_flags;
 	s->a.in.credentials      = &s->credentials3;
 	s->a.out.negotiate_flags = &s->remote_negotiate_flags;
 	s->a.out.return_credentials     = &s->credentials3;
@@ -194,6 +197,7 @@ static void continue_srv_challenge(struct tevent_req *subreq)
 					      s->a.in.secure_channel_type,
 					      &s->credentials1, &s->credentials2,
 					      s->mach_pwd, &s->credentials3,
+					      s->requested_negotiate_flags,
 					      s->local_negotiate_flags);
 	if (composite_nomem(s->creds, c)) {
 		return;
@@ -218,10 +222,17 @@ static void continue_srv_auth2(struct tevent_req *subreq)
 {
 	struct composite_context *c;
 	struct schannel_key_state *s;
+	enum dcerpc_AuthType auth_type;
+	enum dcerpc_AuthLevel auth_level;
+	NTSTATUS status;
 
 	c = tevent_req_callback_data(subreq, struct composite_context);
 	s = talloc_get_type(c->private_data, struct schannel_key_state);
 
+	dcerpc_binding_handle_auth_info(s->pipe2->binding_handle,
+					&auth_type,
+					&auth_level);
+
 	/* receive rpc request result - auth2 credentials */ 
 	c->status = dcerpc_netr_ServerAuthenticate2_r_recv(subreq, s);
 	TALLOC_FREE(subreq);
@@ -233,6 +244,29 @@ static void continue_srv_auth2(struct tevent_req *subreq)
 		return;
 	}
 
+	{
+		uint32_t rqf = s->required_negotiate_flags;
+		uint32_t rf = s->remote_negotiate_flags;
+		uint32_t lf = s->local_negotiate_flags;
+
+		if ((rf & NETLOGON_NEG_SUPPORTS_AES) &&
+		    (lf & NETLOGON_NEG_SUPPORTS_AES))
+		{
+			rqf &= ~NETLOGON_NEG_ARCFOUR;
+			rqf &= ~NETLOGON_NEG_STRONG_KEYS;
+		}
+
+		if ((rqf & rf) != rqf) {
+			rqf = s->required_negotiate_flags;
+			DBG_ERR("The client capabilities don't match "
+				"the server capabilities: local[0x%08X] "
+				"required[0x%08X] remote[0x%08X]\n",
+				lf, rqf, rf);
+			composite_error(c, NT_STATUS_DOWNGRADE_DETECTED);
+			return;
+		}
+	}
+
 	/*
 	 * Strong keys could be unsupported (NT4) or disabled. So retry with the
 	 * flags returned by the server. - asn
@@ -243,6 +277,14 @@ static void continue_srv_auth2(struct tevent_req *subreq)
 		uint32_t rf = s->remote_negotiate_flags;
 		const char *rn = NULL;
 
+		if ((lf & rf) == lf) {
+			/*
+			 * without a change in flags
+			 * there's no need to retry...
+			 */
+			s->dcerpc_schannel_auto = false;
+		}
+
 		if (!s->dcerpc_schannel_auto) {
 			composite_error(c, s->a.out.result);
 			return;
@@ -277,7 +319,7 @@ static void continue_srv_auth2(struct tevent_req *subreq)
 			  "and retry! local[0x%08X] remote[0x%08X]\n",
 			  ln, rn, lf, rf));
 
-		s->local_negotiate_flags = s->remote_negotiate_flags;
+		s->local_negotiate_flags &= s->remote_negotiate_flags;
 
 		generate_random_buffer(s->credentials1.data,
 				       sizeof(s->credentials1.data));
@@ -292,11 +334,30 @@ static void continue_srv_auth2(struct tevent_req *subreq)
 		return;
 	}
 
-	s->creds->negotiate_flags = s->remote_negotiate_flags;
-
 	/* verify credentials */
-	if (!netlogon_creds_client_check(s->creds, s->a.out.return_credentials)) {
-		composite_error(c, NT_STATUS_UNSUCCESSFUL);
+	status = netlogon_creds_client_verify(s->creds,
+					      s->a.out.return_credentials,
+					      auth_type,
+					      auth_level);
+	if (!NT_STATUS_IS_OK(status)) {
+		composite_error(c, status);
+		return;
+	}
+
+	if (s->requested_negotiate_flags == s->local_negotiate_flags) {
+		/*
+		 * Without a downgrade in the crypto we proposed
+		 * we can adjust the otherwise downgraded flags
+		 * before storing.
+		 */
+		s->creds->negotiate_flags &= s->remote_negotiate_flags;
+	} else if (s->local_negotiate_flags != s->remote_negotiate_flags) {
+		/*
+		 * We downgraded our crypto once, we should not
+		 * allow any additional downgrade!
+		 */
+		DBG_ERR("%s: NT_STATUS_DOWNGRADE_DETECTED\n", __location__);
+		composite_error(c, NT_STATUS_DOWNGRADE_DETECTED);
 		return;
 	}
 
@@ -317,6 +378,8 @@ static struct composite_context *dcerpc_schannel_key_send(TALLOC_CTX *mem_ctx,
 	struct composite_context *epm_map_req;
 	enum netr_SchannelType schannel_type = cli_credentials_get_secure_channel_type(credentials);
 	struct cli_credentials *epm_creds = NULL;
+	bool reject_md5_servers = false;
+	bool require_strong_key = false;
 
 	/* composite context allocation and setup */
 	c = composite_create(mem_ctx, p->conn->event_ctx);
@@ -330,19 +393,48 @@ static struct composite_context *dcerpc_schannel_key_send(TALLOC_CTX *mem_ctx,
 	s->pipe        = p;
 	s->credentials = credentials;
 	s->local_negotiate_flags = NETLOGON_NEG_AUTH2_FLAGS;
+	s->required_negotiate_flags = NETLOGON_NEG_AUTHENTICATED_RPC;
 
 	/* allocate credentials */
 	if (s->pipe->conn->flags & DCERPC_SCHANNEL_128) {
 		s->local_negotiate_flags = NETLOGON_NEG_AUTH2_ADS_FLAGS;
+		require_strong_key = true;
 	}
 	if (s->pipe->conn->flags & DCERPC_SCHANNEL_AES) {
 		s->local_negotiate_flags = NETLOGON_NEG_AUTH2_ADS_FLAGS;
-		s->local_negotiate_flags |= NETLOGON_NEG_SUPPORTS_AES;
+		reject_md5_servers = true;
 	}
 	if (s->pipe->conn->flags & DCERPC_SCHANNEL_AUTO) {
 		s->local_negotiate_flags = NETLOGON_NEG_AUTH2_ADS_FLAGS;
 		s->local_negotiate_flags |= NETLOGON_NEG_SUPPORTS_AES;
 		s->dcerpc_schannel_auto = true;
+		reject_md5_servers = lpcfg_reject_md5_servers(lp_ctx);
+		require_strong_key = lpcfg_require_strong_key(lp_ctx);
+	}
+
+	if (lpcfg_weak_crypto(lp_ctx) == SAMBA_WEAK_CRYPTO_DISALLOWED) {
+		reject_md5_servers = true;
+	}
+
+	if (reject_md5_servers) {
+		require_strong_key = true;
+	}
+
+	if (require_strong_key) {
+		s->required_negotiate_flags |= NETLOGON_NEG_ARCFOUR;
+		s->required_negotiate_flags |= NETLOGON_NEG_STRONG_KEYS;
+	}
+
+	if (reject_md5_servers) {
+		s->required_negotiate_flags |= NETLOGON_NEG_PASSWORD_SET2;
+		s->required_negotiate_flags |= NETLOGON_NEG_SUPPORTS_AES;
+	}
+
+	s->local_negotiate_flags |= s->required_negotiate_flags;
+
+	if (s->required_negotiate_flags & NETLOGON_NEG_SUPPORTS_AES) {
+		s->required_negotiate_flags &= ~NETLOGON_NEG_ARCFOUR;
+		s->required_negotiate_flags &= ~NETLOGON_NEG_STRONG_KEYS;
 	}
 
 	/* type of authentication depends on schannel type */
@@ -350,9 +442,7 @@ static struct composite_context *dcerpc_schannel_key_send(TALLOC_CTX *mem_ctx,
 		s->local_negotiate_flags |= NETLOGON_NEG_RODC_PASSTHROUGH;
 	}
 
-	if (lpcfg_weak_crypto(lp_ctx) == SAMBA_WEAK_CRYPTO_DISALLOWED) {
-		s->local_negotiate_flags &= ~NETLOGON_NEG_ARCFOUR;
-	}
+	s->requested_negotiate_flags = s->local_negotiate_flags;
 
 	epm_creds = cli_credentials_init_anon(s);
 	if (composite_nomem(epm_creds, c)) return c;
@@ -379,7 +469,8 @@ static struct composite_context *dcerpc_schannel_key_send(TALLOC_CTX *mem_ctx,
  */
 static NTSTATUS dcerpc_schannel_key_recv(struct composite_context *c,
 				TALLOC_CTX *mem_ctx,
-				struct netlogon_creds_CredentialState **creds)
+				struct netlogon_creds_CredentialState **creds,
+				uint32_t *requested_negotiate_flags)
 {
 	NTSTATUS status = composite_wait(c);
 
@@ -388,6 +479,7 @@ static NTSTATUS dcerpc_schannel_key_recv(struct composite_context *c,
 			talloc_get_type_abort(c->private_data,
 			struct schannel_key_state);
 		*creds = talloc_move(mem_ctx, &s->creds);
+		*requested_negotiate_flags = s->requested_negotiate_flags;
 	}
 
 	talloc_free(c);
@@ -398,6 +490,7 @@ static NTSTATUS dcerpc_schannel_key_recv(struct composite_context *c,
 struct auth_schannel_state {
 	struct dcerpc_pipe *pipe;
 	struct cli_credentials *credentials;
+	uint32_t requested_negotiate_flags;
 	const struct ndr_interface_table *table;
 	struct loadparm_context *lp_ctx;
 	uint8_t auth_level;
@@ -406,7 +499,9 @@ struct auth_schannel_state {
 	struct netr_Authenticator auth;
 	struct netr_Authenticator return_auth;
 	union netr_Capabilities capabilities;
+	union netr_Capabilities client_caps;
 	struct netr_LogonGetCapabilities c;
+	union netr_CONTROL_QUERY_INFORMATION ctrl_info;
 };
 
 
@@ -427,7 +522,11 @@ static void continue_schannel_key(struct composite_context *ctx)
 	NTSTATUS status;
 
 	/* receive schannel key */
-	status = c->status = dcerpc_schannel_key_recv(ctx, s, &s->creds_state);
+	c->status = dcerpc_schannel_key_recv(ctx,
+					     s,
+					     &s->creds_state,
+					     &s->requested_negotiate_flags);
+	status = c->status;
 	if (!composite_is_ok(c)) {
 		DEBUG(1, ("Failed to setup credentials: %s\n", nt_errstr(status)));
 		return;
@@ -496,36 +595,52 @@ static void continue_bind_auth(struct composite_context *ctx)
 								 &s->c);
 		if (composite_nomem(subreq, c)) return;
 
-		tevent_req_set_callback(subreq, continue_get_capabilities, c);
+		tevent_req_set_callback(subreq,
+					continue_get_negotiated_capabilities,
+					c);
 		return;
 	}
 
 	composite_done(c);
 }
 
+static void continue_logon_control_do(struct composite_context *c);
+
 /*
   Stage 4 of auth_schannel: Get the Logon Capabilities and verify them.
 */
-static void continue_get_capabilities(struct tevent_req *subreq)
+static void continue_get_negotiated_capabilities(struct tevent_req *subreq)
 {
 	struct composite_context *c;
 	struct auth_schannel_state *s;
+	enum dcerpc_AuthType auth_type;
+	enum dcerpc_AuthLevel auth_level;
+	NTSTATUS status;
 
 	c = tevent_req_callback_data(subreq, struct composite_context);
 	s = talloc_get_type(c->private_data, struct auth_schannel_state);
 
+	dcerpc_binding_handle_auth_info(s->pipe->binding_handle,
+					&auth_type,
+					&auth_level);
+
 	/* receive rpc request result */
 	c->status = dcerpc_netr_LogonGetCapabilities_r_recv(subreq, s);
 	TALLOC_FREE(subreq);
 	if (NT_STATUS_EQUAL(c->status, NT_STATUS_RPC_PROCNUM_OUT_OF_RANGE)) {
 		if (s->creds_state->negotiate_flags & NETLOGON_NEG_SUPPORTS_AES) {
-			composite_error(c, NT_STATUS_INVALID_NETWORK_RESPONSE);
+			DBG_ERR("%s: NT_STATUS_DOWNGRADE_DETECTED\n", __location__);
+			composite_error(c, NT_STATUS_DOWNGRADE_DETECTED);
 			return;
-		} else {
-			/* This is probably NT */
-			composite_done(c);
+		} else if (s->creds_state->negotiate_flags & NETLOGON_NEG_STRONG_KEYS) {
+			DBG_ERR("%s: NT_STATUS_DOWNGRADE_DETECTED\n", __location__);
+			composite_error(c, NT_STATUS_DOWNGRADE_DETECTED);
 			return;
 		}
+
+		/* This is probably NT */
+		continue_logon_control_do(c);
+		return;
 	} else if (!composite_is_ok(c)) {
 		return;
 	}
@@ -533,7 +648,8 @@ static void continue_get_capabilities(struct tevent_req *subreq)
 	if (NT_STATUS_EQUAL(s->c.out.result, NT_STATUS_NOT_IMPLEMENTED)) {
 		if (s->creds_state->negotiate_flags & NETLOGON_NEG_SUPPORTS_AES) {
 			/* This means AES isn't supported. */
-			composite_error(c, NT_STATUS_INVALID_NETWORK_RESPONSE);
+			DBG_ERR("%s: NT_STATUS_DOWNGRADE_DETECTED\n", __location__);
+			composite_error(c, NT_STATUS_DOWNGRADE_DETECTED);
 			return;
 		}
 
@@ -543,9 +659,12 @@ static void continue_get_capabilities(struct tevent_req *subreq)
 	}
 
 	/* verify credentials */
-	if (!netlogon_creds_client_check(&s->save_creds_state,
-					 &s->c.out.return_authenticator->cred)) {
-		composite_error(c, NT_STATUS_UNSUCCESSFUL);
+	status = netlogon_creds_client_verify(&s->save_creds_state,
+					      &s->c.out.return_authenticator->cred,
+					      auth_type,
+					      auth_level);
+	if (!NT_STATUS_IS_OK(status)) {
+		composite_error(c, status);
 		return;
 	}
 
@@ -559,19 +678,175 @@ static void continue_get_capabilities(struct tevent_req *subreq)
 
 	/* compare capabilities */
 	if (s->creds_state->negotiate_flags != s->capabilities.server_capabilities) {
-		DEBUG(2, ("The client capabilities don't match the server "
-			  "capabilities: local[0x%08X] remote[0x%08X]\n",
-			  s->creds_state->negotiate_flags,
-			  s->capabilities.server_capabilities));
-		composite_error(c, NT_STATUS_INVALID_NETWORK_RESPONSE);
+		DBG_ERR("The client capabilities don't match the server "
+			"capabilities: local[0x%08X] remote[0x%08X]\n",
+			s->creds_state->negotiate_flags,
+			s->capabilities.server_capabilities);
+		composite_error(c, NT_STATUS_DOWNGRADE_DETECTED);
 		return;
 	}
 
-	/* TODO: Add downgrade detection. */
+	if ((s->requested_negotiate_flags & NETLOGON_NEG_SUPPORTS_AES) &&
+	    (!(s->creds_state->negotiate_flags & NETLOGON_NEG_SUPPORTS_AES)))
+	{
+		DBG_ERR("%s: NT_STATUS_DOWNGRADE_DETECTED\n", __location__);
+		composite_error(c, NT_STATUS_DOWNGRADE_DETECTED);
+		return;
+	}
+
+	ZERO_STRUCT(s->return_auth);
+
+	s->save_creds_state = *s->creds_state;
+	status = netlogon_creds_client_authenticator(&s->save_creds_state,
+						     &s->auth);
+	if (!NT_STATUS_IS_OK(status)) {
+		composite_error(c, status);
+		return;
+	}
+
+	s->c.in.credential            = &s->auth;
+	s->c.in.return_authenticator  = &s->return_auth;
+	s->c.in.query_level           = 2;
+
+	s->c.out.capabilities         = &s->client_caps;
+	s->c.out.return_authenticator = &s->return_auth;
+
+	subreq = dcerpc_netr_LogonGetCapabilities_r_send(s,
+							 c->event_ctx,
+							 s->pipe->binding_handle,
+							 &s->c);
+	if (composite_nomem(subreq, c)) return;
+
+	tevent_req_set_callback(subreq, continue_get_client_capabilities, c);
+	return;
+}
+
+static void continue_get_client_capabilities(struct tevent_req *subreq)
+{
+	struct composite_context *c;
+	struct auth_schannel_state *s;
+	enum dcerpc_AuthType auth_type;
+	enum dcerpc_AuthLevel auth_level;
+	NTSTATUS status;
+
+	c = tevent_req_callback_data(subreq, struct composite_context);
+	s = talloc_get_type(c->private_data, struct auth_schannel_state);
+
+	dcerpc_binding_handle_auth_info(s->pipe->binding_handle,
+					&auth_type,
+					&auth_level);
+
+	/* receive rpc request result */
+	c->status = dcerpc_netr_LogonGetCapabilities_r_recv(subreq, s);
+	TALLOC_FREE(subreq);
+	if (NT_STATUS_EQUAL(c->status, NT_STATUS_RPC_BAD_STUB_DATA)) {
+		/*
+		 * unpatched Samba server, see
+		 * https://bugzilla.samba.org/show_bug.cgi?id=15418
+		 */
+		c->status = NT_STATUS_RPC_ENUM_VALUE_OUT_OF_RANGE;
+	}
+	if (NT_STATUS_EQUAL(c->status, NT_STATUS_RPC_ENUM_VALUE_OUT_OF_RANGE)) {
+		/*
+		 * Here we know the negotiated flags were already
+		 * verified with query_level=1, which means
+		 * the server supported NETLOGON_NEG_SUPPORTS_AES
+		 * and also NETLOGON_NEG_AUTHENTICATED_RPC
+		 *
+		 * As we're using DCERPC_AUTH_TYPE_SCHANNEL with
+		 * DCERPC_AUTH_LEVEL_INTEGRITY or DCERPC_AUTH_LEVEL_PRIVACY
+		 * we should detect a faked
+		 * NT_STATUS_RPC_ENUM_VALUE_OUT_OF_RANGE
+		 * with the next request as the sequence number processing
+		 * gets out of sync.
+		 *
+		 * So we'll do a LogonControl message to check that...
+		 */
+		continue_logon_control_do(c);
+		return;
+	}
+	if (!composite_is_ok(c)) {
+		return;
+	}
+
+	/* verify credentials */
+	status = netlogon_creds_client_verify(&s->save_creds_state,
+					      &s->c.out.return_authenticator->cred,
+					      auth_type,
+					      auth_level);
+	if (!NT_STATUS_IS_OK(status)) {
+		composite_error(c, status);
+		return;
+	}
+
+	if (!NT_STATUS_IS_OK(s->c.out.result)) {
+		composite_error(c, s->c.out.result);
+		return;
+	}
+
+	/* compare capabilities */
+	if (s->requested_negotiate_flags != s->client_caps.requested_flags) {
+		DBG_ERR("The client requested capabilities did not reach"
+			"the server! local[0x%08X] remote[0x%08X]\n",
+			s->requested_negotiate_flags,
+			s->client_caps.requested_flags);
+		composite_error(c, NT_STATUS_DOWNGRADE_DETECTED);
+		return;
+	}
+
+	*s->creds_state = s->save_creds_state;
+	cli_credentials_set_netlogon_creds(s->credentials, s->creds_state);
 
 	composite_done(c);
 }
 
+static void continue_logon_control_done(struct tevent_req *subreq);
+
+static void continue_logon_control_do(struct composite_context *c)
+{
+	struct auth_schannel_state *s = NULL;
+	struct tevent_req *subreq = NULL;
+
+	s = talloc_get_type(c->private_data, struct auth_schannel_state);
+
+	subreq = dcerpc_netr_LogonControl_send(s,
+					       c->event_ctx,
+					       s->pipe->binding_handle,
+					       s->c.in.server_name,
+					       NETLOGON_CONTROL_QUERY,
+					       2,
+					       &s->ctrl_info);
+	if (composite_nomem(subreq, c)) return;
+
+	tevent_req_set_callback(subreq, continue_logon_control_done, c);
+}
+
+static void continue_logon_control_done(struct tevent_req *subreq)
+{
+	struct composite_context *c = NULL;
+	struct auth_schannel_state *s = NULL;
+	WERROR werr;
+
+	c = tevent_req_callback_data(subreq, struct composite_context);
+	s = talloc_get_type(c->private_data, struct auth_schannel_state);
+
+	/* receive rpc request result */
+	c->status = dcerpc_netr_LogonControl_recv(subreq, s, &werr);
+	TALLOC_FREE(subreq);
+	if (!NT_STATUS_IS_OK(c->status)) {
+		DBG_ERR("%s: NT_STATUS_DOWNGRADE_DETECTED\n", __location__);
+		composite_error(c, NT_STATUS_DOWNGRADE_DETECTED);
+		return;
+	}
+
+	if (!W_ERROR_EQUAL(werr, WERR_NOT_SUPPORTED)) {
+		DBG_ERR("%s: NT_STATUS_DOWNGRADE_DETECTED\n", __location__);
+		composite_error(c, NT_STATUS_DOWNGRADE_DETECTED);
+		return;
+	}
+
+	composite_done(c);
+}
 
 /*
   Initiate schannel authentication request
