@@ -27,9 +27,6 @@
 #include "system/filesys.h"
 #include "../lib/util/dlinklist.h"
 #include "libcli/libcli.h"
-#include "libcli/raw/libcliraw.h"
-#include "torture/torture.h"
-#include "libcli/libcli.h"
 #include "torture/util.h"
 #include "torture/nbench/proto.h"
 
@@ -49,8 +46,8 @@ struct lock_info {
 
 struct createx_params {
 	char *fname;
-	uint_t create_options;
-	uint_t create_disposition;
+	unsigned int create_options;
+	unsigned int create_disposition;
 	int handle;
 };
 
@@ -75,8 +72,8 @@ static struct {
 
 static bool nb_do_createx(struct ftable *f,
 			  const char *fname,
-			  uint_t create_options,
-			  uint_t create_disposition,
+			  unsigned int create_options,
+			  unsigned int create_disposition,
 			  int handle,
 			  NTSTATUS status,
 			  bool retry);
@@ -85,8 +82,8 @@ static bool nb_do_lockx(bool relock, int handle, off_t offset, int size, NTSTATU
 
 static void nb_set_createx_params(struct ftable *f,
 				  const char *fname,
-				  uint_t create_options,
-				  uint_t create_disposition,
+				  unsigned int create_options,
+				  unsigned int create_disposition,
 				  int handle)
 {
 	struct createx_params *cp = &f->cp;
@@ -195,7 +192,7 @@ void nbio_target_rate(double rate)
 
 	tdelay = (children[nbio_id].bytes - last_bytes)/(1.0e6*rate) - timeval_elapsed(&last_time);
 	if (tdelay > 0) {
-		msleep(tdelay*1000);
+		smb_msleep(tdelay*1000);
 	} else {
 		children[nbio_id].max_latency = MAX(children[nbio_id].max_latency, -tdelay);
 	}
@@ -213,7 +210,7 @@ void nbio_time_delay(double targett)
 {
 	double elapsed = timeval_elapsed(&children[nbio_id].starttime);
 	if (targett > elapsed) {
-		msleep(1000*(targett - elapsed));
+		smb_msleep(1000*(targett - elapsed));
 	} else if (elapsed - targett > children[nbio_id].max_latency) {
 		children[nbio_id].max_latency = MAX(elapsed - targett, children[nbio_id].max_latency);
 	}
@@ -317,7 +314,7 @@ next:
 void nbio_shmem(int n, int t_timelimit, int t_warmup)
 {
 	nprocs = n;
-	children = shm_setup(sizeof(*children) * nprocs);
+	children = anonymous_shared_allocate(sizeof(*children) * nprocs);
 	if (!children) {
 		printf("Failed to setup shared memory!\n");
 		nb_exit(1);
@@ -458,8 +455,8 @@ bool nb_unlink(const char *fname, int attr, NTSTATUS status, bool retry)
 
 static bool nb_do_createx(struct ftable *f,
 			  const char *fname,
-			  uint_t create_options,
-			  uint_t create_disposition,
+			  unsigned int create_options,
+			  unsigned int create_disposition,
 			  int handle,
 			  NTSTATUS status,
 			  bool retry)
@@ -468,7 +465,7 @@ static bool nb_do_createx(struct ftable *f,
 	uint32_t desired_access;
 	NTSTATUS ret;
 	TALLOC_CTX *mem_ctx;
-	uint_t flags = 0;
+	unsigned int flags = 0;
 
 	mem_ctx = talloc_init("raw_open");
 
@@ -487,7 +484,7 @@ static bool nb_do_createx(struct ftable *f,
 
 	io.ntcreatex.level = RAW_OPEN_NTCREATEX;
 	io.ntcreatex.in.flags = flags;
-	io.ntcreatex.in.root_fid = 0;
+	io.ntcreatex.in.root_fid.fnum = 0;
 	io.ntcreatex.in.access_mask = desired_access;
 	io.ntcreatex.in.file_attr = 0;
 	io.ntcreatex.in.alloc_size = 0;
@@ -524,7 +521,7 @@ static bool nb_do_createx(struct ftable *f,
 		f = talloc (NULL, struct ftable);
 		f->locks = NULL;
 		nb_set_createx_params(f, fname, create_options, create_disposition, handle);
-		DLIST_ADD_END(ftable, f, struct ftable *);
+		DLIST_ADD_END(ftable, f);
 	}
 
 	f->handle = handle;
@@ -534,7 +531,7 @@ static bool nb_do_createx(struct ftable *f,
 }
 
 bool nb_createx(const char *fname, 
-	       uint_t create_options, uint_t create_disposition, int handle,
+	       unsigned int create_options, unsigned int create_disposition, int handle,
 	       NTSTATUS status)
 {
 	return nb_do_createx(NULL, fname, create_options, create_disposition, handle, status, false);
@@ -656,7 +653,7 @@ static bool nb_do_lockx(bool relock, int handle, off_t offset, int size, NTSTATU
 		linfo = talloc (f, struct lock_info);
 		linfo->offset = offset;
 		linfo->size = size;
-		DLIST_ADD_END(f->locks, linfo, struct lock_info *);
+		DLIST_ADD_END(f->locks, linfo);
 	}
 
 	return true;
@@ -667,7 +664,7 @@ bool nb_lockx(int handle, off_t offset, int size, NTSTATUS status)
 	return nb_do_lockx(false, handle, offset, size, status);
 }
 
-bool nb_unlockx(int handle, uint_t offset, int size, NTSTATUS status)
+bool nb_unlockx(int handle, unsigned int offset, int size, NTSTATUS status)
 {
 	union smb_lock io;
 	int i;
