@@ -30,6 +30,7 @@
 #include "libcli/auth/libcli_auth.h"
 #include "torture/rpc/torture_rpc.h"
 #include "param/param.h"
+#include "source3/rpc_client/init_lsa.h"
 
 #include <gnutls/gnutls.h>
 #include <gnutls/crypto.h>
@@ -47,34 +48,178 @@ static bool test_get_policy_handle(struct torture_context *tctx,
 				   uint32_t access_mask,
 				   struct policy_handle **handle  )
 {
-	struct policy_handle *h;
-	struct lsa_OpenPolicy2 pr;
-	struct lsa_ObjectAttribute attr;
+	struct policy_handle *h = NULL;
+	struct lsa_ObjectAttribute attr = {
+		.len = 0,
+	};
+	union lsa_revision_info in_rinfo = {
+		.info1 = {
+			.revision = 1,
+			.supported_features = LSA_FEATURE_TDO_AUTH_INFO_AES_CIPHER,
+		},
+	};
+	union lsa_revision_info out_rinfo = {
+		.info1.revision = 0,
+	};
+	uint32_t out_version = 0;
+	struct lsa_OpenPolicy3 r = {
+		.in = {
+			.system_name = "\\",
+			.attr = &attr,
+			.access_mask = access_mask,
+			.in_version = 1,
+			.in_revision_info = &in_rinfo,
+		},
+		.out = {
+			.out_version = &out_version,
+			.out_revision_info = &out_rinfo,
+		}
+	};
 	NTSTATUS status;
 
-	h = talloc(tctx, struct policy_handle);
-	torture_assert(tctx, h != NULL, "talloc(tctx, struct policy_handle)");
+	*handle = NULL;
 
-	attr.len = 0;
-	attr.root_dir = NULL;
-	attr.object_name = NULL;
-	attr.attributes = 0;
-	attr.sec_desc = NULL;
-	attr.sec_qos = NULL;
+	h = talloc_zero(tctx, struct policy_handle);
+	torture_assert_not_null(tctx, h, "talloc_zero(tctx, struct policy_handle)");
+	r.out.handle = h;
 
-	pr.in.system_name = "\\";
-	pr.in.attr = &attr;
-	pr.in.access_mask = access_mask;
-	pr.out.handle = h;
+	status = dcerpc_lsa_OpenPolicy3_r(p->binding_handle, tctx, &r);
+	torture_assert_ntstatus_ok(tctx, status, "OpenPolicy3 failed");
+	torture_assert_ntstatus_ok(tctx, r.out.result, "OpenPolicy3 failed");
 
-	status = dcerpc_lsa_OpenPolicy2_r(p->binding_handle, tctx, &pr);
-	torture_assert_ntstatus_ok(tctx, status, "OpenPolicy2 failed");
-	torture_assert_ntstatus_ok(tctx, pr.out.result, "OpenPolicy2 failed");
+	torture_assert_int_equal(tctx,
+				 out_version,
+				 1,
+				 "Invalid version");
+	torture_assert_int_equal(tctx,
+				 out_rinfo.info1.revision,
+				 1,
+				 "Invalid revision");
+	torture_assert_int_equal(tctx,
+				 out_rinfo.info1.supported_features,
+				 LSA_FEATURE_TDO_AUTH_INFO_AES_CIPHER,
+				 "Invalid supported feature set");
 
 	*handle = h;
 	return true;
 }
 
+static bool test_set_forest_trust_info(struct dcerpc_pipe *p,
+				       struct torture_context *tctx,
+				       struct policy_handle *handle,
+				       struct dom_sid *domsid,
+				       const char *trust_name,
+				       const char *trust_name_dns)
+{
+	struct lsa_ForestTrustCollisionInfo *collision_info = NULL;
+	struct lsa_ForestTrustInformation *fti = NULL;
+	struct lsa_lsaRSetForestTrustInformation ft_info = {
+		.in = {
+			.handle = handle,
+			.highest_record_type = 2,
+		},
+		.out = {
+			.collision_info = &collision_info,
+		}
+	};
+
+	torture_comment(tctx, "\nTesting lsaRSetForestTrustInformation\n");
+
+	ft_info.in.trusted_domain_name = talloc_zero(tctx, struct lsa_StringLarge);
+	torture_assert_not_null(tctx, ft_info.in.trusted_domain_name, "No memory");
+	ft_info.in.trusted_domain_name->string = trust_name_dns;
+
+	fti = talloc_zero(tctx, struct lsa_ForestTrustInformation);
+	torture_assert_not_null(tctx, fti, "No memory");
+	fti->count = 2;
+	fti->entries = talloc_array(tctx, struct lsa_ForestTrustRecord *, 2);
+	fti->entries[0] = talloc_zero(tctx, struct lsa_ForestTrustRecord);
+	fti->entries[0]->flags = 0;
+	fti->entries[0]->type = LSA_FOREST_TRUST_TOP_LEVEL_NAME;
+	fti->entries[0]->time = 0;
+	fti->entries[0]->forest_trust_data.top_level_name.string = trust_name_dns;
+	fti->entries[1] = talloc_zero(tctx, struct lsa_ForestTrustRecord);
+	fti->entries[1]->flags = 0;
+	fti->entries[1]->type = LSA_FOREST_TRUST_DOMAIN_INFO;
+	fti->entries[1]->time = 0;
+	fti->entries[1]->forest_trust_data.domain_info.domain_sid = domsid;
+	fti->entries[1]->forest_trust_data.domain_info.dns_domain_name.string =
+		trust_name_dns;
+	fti->entries[1]
+		->forest_trust_data.domain_info.netbios_domain_name
+		.string = trust_name;
+
+	ft_info.in.forest_trust_info = fti;
+
+	torture_assert_ntstatus_ok(tctx,
+				   dcerpc_lsa_lsaRSetForestTrustInformation_r(
+					   p->binding_handle, tctx, &ft_info),
+				   "lsaRSetForestTrustInformation failed");
+	torture_assert_ntstatus_ok(tctx,
+				   ft_info.out.result,
+				   "lsaRSetForestTrustInformation failed");
+
+	/* There should be no collisions */
+	torture_assert(tctx, collision_info == NULL, "collision info returned");
+
+	return true;
+}
+
+static bool test_query_forest_trust_info(struct dcerpc_pipe *p,
+					 struct torture_context *tctx,
+					 struct policy_handle *handle,
+					 const char *trust_name_dns)
+{
+	struct lsa_String trusted_domain_name = {
+		.size = 0,
+	};
+	struct lsa_ForestTrustInformation *fti = NULL;
+	struct lsa_lsaRQueryForestTrustInformation r = {
+		.in = {
+			.handle = handle,
+			.highest_record_type = 2,
+			.trusted_domain_name = &trusted_domain_name,
+		},
+		.out = {
+			.forest_trust_info = &fti,
+		}
+	};
+	struct lsa_ForestTrustRecord *rec = NULL;
+
+	init_lsa_String(&trusted_domain_name, trust_name_dns);
+
+	torture_comment(tctx, "\nTesting lsaRQueryForestTrustInformation\n");
+
+
+	torture_assert_ntstatus_ok(tctx,
+				   dcerpc_lsa_lsaRQueryForestTrustInformation_r(
+					   p->binding_handle, tctx, &r),
+				   "lsaRQueryForestTrustInformation failed");
+	torture_assert_ntstatus_ok(tctx,
+				   r.out.result,
+				   "lsaRQueryForestTrustInformation failed");
+
+	torture_assert_not_null(tctx,
+				r.out.forest_trust_info,
+				"forest_trust_info is not set");
+	torture_assert_int_equal(tctx,
+				 fti->count,
+				 2,
+				 "Unexpected forest_trust_info count");
+
+	rec = fti->entries[0];
+	torture_assert_int_equal(tctx,
+				 rec->type,
+				 LSA_FOREST_TRUST_TOP_LEVEL_NAME,
+				 "Unexpedted type");
+	torture_assert_str_equal(tctx,
+				 rec->forest_trust_data.top_level_name.string,
+				 trust_name_dns,
+				 "Unexpected top level name");
+
+	return true;
+}
+
 static bool test_create_trust_and_set_info(struct dcerpc_pipe *p,
 					   struct torture_context *tctx,
 					   const char *trust_name,
@@ -83,8 +228,6 @@ static bool test_create_trust_and_set_info(struct dcerpc_pipe *p,
 					   struct lsa_TrustDomainInfoAuthInfoInternal *authinfo)
 {
 	struct policy_handle *handle;
-	struct lsa_lsaRSetForestTrustInformation fti;
-	struct lsa_ForestTrustCollisionInfo *collision_info = NULL;
 	struct lsa_Close cr;
 	struct policy_handle closed_handle;
 	struct lsa_CreateTrustedDomainEx2 r;
@@ -92,6 +235,7 @@ static bool test_create_trust_and_set_info(struct dcerpc_pipe *p,
 	struct policy_handle trustdom_handle;
 	struct lsa_QueryTrustedDomainInfo q;
 	union lsa_TrustedDomainInfo *info = NULL;
+	bool ok;
 
 	if (!test_get_policy_handle(tctx, p,
 				   (LSA_POLICY_VIEW_LOCAL_INFORMATION |
@@ -155,34 +299,16 @@ static bool test_create_trust_and_set_info(struct dcerpc_pipe *p,
 	torture_assert_int_equal(tctx, info->info_ex.trust_direction, trustinfo.trust_direction,
 				 "QueryTrustedDomainInfo of returned incorrect trust direction");
 
-	fti.in.handle = handle;
-	fti.in.trusted_domain_name = talloc_zero(tctx, struct lsa_StringLarge);
-	fti.in.trusted_domain_name->string = trust_name_dns;
-	fti.in.highest_record_type = 2;
-	fti.in.forest_trust_info = talloc_zero(tctx, struct lsa_ForestTrustInformation);
-	fti.in.forest_trust_info->count = 2;
-	fti.in.forest_trust_info->entries = talloc_array(tctx, struct lsa_ForestTrustRecord *, 2);
-	fti.in.forest_trust_info->entries[0] = talloc_zero(tctx, struct lsa_ForestTrustRecord);
-	fti.in.forest_trust_info->entries[0]->flags = 0;
-	fti.in.forest_trust_info->entries[0]->type = LSA_FOREST_TRUST_TOP_LEVEL_NAME;
-	fti.in.forest_trust_info->entries[0]->time = 0;
-	fti.in.forest_trust_info->entries[0]->forest_trust_data.top_level_name.string = trust_name_dns;
-	fti.in.forest_trust_info->entries[1] = talloc_zero(tctx, struct lsa_ForestTrustRecord);
-	fti.in.forest_trust_info->entries[1]->flags = 0;
-	fti.in.forest_trust_info->entries[1]->type = LSA_FOREST_TRUST_DOMAIN_INFO;
-	fti.in.forest_trust_info->entries[1]->time = 0;
-	fti.in.forest_trust_info->entries[1]->forest_trust_data.domain_info.domain_sid = domsid;
-	fti.in.forest_trust_info->entries[1]->forest_trust_data.domain_info.dns_domain_name.string = trust_name_dns;
-	fti.in.forest_trust_info->entries[1]->forest_trust_data.domain_info.netbios_domain_name.string = trust_name;
-	fti.in.check_only = 0;
-	fti.out.collision_info = &collision_info;
+	ok = test_set_forest_trust_info(
+		p, tctx, handle, domsid, trust_name, trust_name_dns);
+	if (!ok) {
+		return false;
+	}
 
-	torture_comment(tctx, "\nTesting SetForestTrustInformation\n");
-
-	torture_assert_ntstatus_ok(tctx,
-				   dcerpc_lsa_lsaRSetForestTrustInformation_r(p->binding_handle, tctx, &fti),
-				   "lsaRSetForestTrustInformation failed");
-	torture_assert_ntstatus_ok(tctx, fti.out.result, "lsaRSetForestTrustInformation failed");
+	ok = test_query_forest_trust_info(p, tctx, handle, trust_name_dns);
+	if (!ok) {
+		return false;
+	}
 
 	cr.in.handle = handle;
 	cr.out.handle = &closed_handle;
@@ -208,7 +334,7 @@ static bool get_and_set_info(struct dcerpc_pipe *p,
 	NTSTATUS status;
 	struct lsa_QueryTrustedDomainInfoByName qr;
 	struct lsa_SetTrustedDomainInfoByName sr;
-	union lsa_TrustedDomainInfo *info;
+	union lsa_TrustedDomainInfo *info = NULL;
 	struct lsa_Close cr;
 	struct policy_handle closed_handle;
 	size_t c;
@@ -245,7 +371,6 @@ static bool get_and_set_info(struct dcerpc_pipe *p,
 	sr.in.handle = handle;
 	sr.in.trusted_domain = talloc_zero(tctx, struct lsa_String);
 	sr.in.trusted_domain->string = name;
-	sr.in.info = info;
 
 	for (c = 0; il[c].info_level != -1; c++) {
 		torture_comment(tctx, "\nGetting/Setting dom info [%d]\n",il[c].info_level);
@@ -260,7 +385,7 @@ static bool get_and_set_info(struct dcerpc_pipe *p,
 					      "%s but %s\n",
 					      nt_errstr(il[c].get_result),
 					      nt_errstr(qr.out.result));
-			
+
 			/* We may be testing a server without support for this level */
 			if (qr.in.level == LSA_TRUSTED_DOMAIN_SUPPORTED_ENCRYPTION_TYPES && NT_STATUS_EQUAL(qr.out.result, NT_STATUS_INVALID_PARAMETER)) {
 				return true;
@@ -533,6 +658,8 @@ static bool test_validate_trust(struct torture_context *tctx,
 	struct samr_Password *old_nt_hash;
 	char *dummy;
 	uint32_t trust_attributes = LSA_TRUST_ATTRIBUTE_FOREST_TRANSITIVE;
+	enum dcerpc_AuthType auth_type = DCERPC_AUTH_TYPE_NONE;
+	enum dcerpc_AuthLevel auth_level = DCERPC_AUTH_LEVEL_NONE;
 
 	status = dcerpc_parse_binding(tctx, binding, &b);
 	torture_assert_ntstatus_ok(tctx, status, "Bad binding string");
@@ -608,8 +735,19 @@ static bool test_validate_trust(struct torture_context *tctx,
 	old_nt_hash = cli_credentials_get_old_nt_hash(credentials, tctx);
 	torture_assert(tctx, old_nt_hash != NULL, "cli_credentials_get_old_nt_hash()");
 
-	netlogon_creds_des_decrypt(creds, &new_owf_password);
-	netlogon_creds_des_decrypt(creds, &old_owf_password);
+	dcerpc_binding_handle_auth_info(p->binding_handle,
+					&auth_type,
+					&auth_level);
+	status = netlogon_creds_decrypt_samr_Password(creds,
+						      &new_owf_password,
+						      auth_type,
+						      auth_level);
+	torture_assert_ntstatus_ok(tctx, status, "decrypt_samr_Password");
+	status = netlogon_creds_decrypt_samr_Password(creds,
+						      &old_owf_password,
+						      auth_type,
+						      auth_level);
+	torture_assert_ntstatus_ok(tctx, status, "decrypt_samr_Password");
 
 	dump_data(1, new_owf_password.hash, 16);
 	dump_data(1, new_nt_hash->hash, 16);
@@ -666,7 +804,7 @@ static bool test_validate_trust(struct torture_context *tctx,
 			break;
 		default:
 			torture_assert_int_equal(tctx, e->type, LSA_FOREST_TRUST_TOP_LEVEL_NAME,
-						 "Unexptected LSA_FOREST_TRUST_* type");
+						 "Unexpected LSA_FOREST_TRUST_* type");
 		}
 	}
 
